/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.firma.klient.webservice;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.communication.Credentials;
import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.Request;
import dk.gov.oiosi.communication.Response;
import dk.gov.oiosi.communication.SendPolicy;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * Invoke RASP call against test RASP service defined by Service Skeleton.
 * 
 * @author Lenio, ani
 */
public class LocalTestClient {

    private static Log log = LogFactory.getLog(LocalTestClient.class);
    public static final String TEST_DOCUMENT_XML = "./Xml/OIOUBL_Invoice_v2p1.xml";

    /**
     * Endpoint for testservice defined in Java/RASP Service skeleton.
     */
    private static final String ENDPOINT = "http://localhost:8080/myws/services/RaspService";

    /**
     * RASP Test application
     * 
     * This test client goes through the following steps:
     * 
     *  1. Load a document
     *  2. Validate the document
     *  3. Wrap the document in a SOAP message and add the custom RASP headers
     *  4. Sends it to a service deployed on localhost (by default: the service_skeleton service)
     *
     */
    public static void main(final String[] args) {
        try {
            log.info("Try to send message to service");

            // Set the path to the custom RASP configuration file. 
            // This file contains setup for things like:
            //   - Addresses to OCSP, LDAP and UDDI servers
            //   - The path to the Axis2 modules needed (RASP, WS-Security and WS-ReliableMessaging)
            //   - Configuration for the different OIO UBL document types
            RaspConfigurationHandler.setConfigurationFile("./conf/RaspConfiguration.xml");

            
            // Step 1: Load a document
            
            // Load the document from disc
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.load(TEST_DOCUMENT_XML);
            log.info("Document created from file : ");

            // Search our configuration file for the document type we are currently about to send.
            // The UBL type can be inferred from the XML root element name of the file, and the configuration we get
            // contains paths to XML schemas, schematron stylesheets and XPath expressions used to find the EAN number for example
            DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
            DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);
            log.info("Document is known in configuration");

            // Step 2: Validate the document
            
            // Schema validate document before sending...
            SchemaValidator schemaValidator = new SchemaValidator();
            XmlSchema schema = new XmlSchema(documentTypeConfig.getSchemaPath());
            schemaValidator.schemaValidateXmlDocument(xmlDocument, schema);
            log.info("Document is schema valid");

            // Schematron validate Document before sending...
            SchematronValidator schematronValidator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());
            schematronValidator.schematronValidateXmlDocument(xmlDocument);
            log.info("Document is schematron/logically valid");

            // Step 3: Wrap the document in a SOAP message and add the custom RASP headers

            // Create message wrapper
            OiosiMessage oiosiMessage = new OiosiMessage(xmlDocument);
            oiosiMessage.setRequestAction(documentTypeConfig.getEndpointType().getRequestAction());
            oiosiMessage.setReplyAction(documentTypeConfig.getEndpointType().getReplyAction());

            // Find the sender and receiver IDs (e.g. EAN numbers)
            IIdentifier identifier = Utilities.getReceiverEndpointKey(xmlDocument);
            IIdentifier senderIdentifier = Utilities.getSenderEndpointKey(xmlDocument);
            
            // Add them as headers, and add an unique MessageIdentifier header
            oiosiMessage.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            oiosiMessage.addMessageHeader("ReceiverPartyIdentifier", identifier.getAsString());
            oiosiMessage.addMessageHeader("SenderPartyIdentifier", senderIdentifier.getAsString());
            log.info("We have defined Document to send.");

            
            //Step 4: Send it
            
            Uri endpointAddress = new Uri(ENDPOINT);
            log.info("Destination service : " + endpointAddress.toString());

            // Load the server certificate from keystore
            KeyStore ks = KeyStore.getInstance("JKS");
            ks.load(new FileInputStream("./keys/oiosi_server.jks"), "Test1234".toCharArray());
            X509Certificate server = (X509Certificate) ks.getCertificate("server");
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(server);
            Credentials credentials = new Credentials(ocesCertificate);
            log.info("Destination service certificate : " + server.getSubjectX500Principal());

            // Use a sending policy defined in the configuration file
            SendPolicy sendPolicy = RaspConfigurationHandler.getInstance().getSendPolicy();
            log.info("Sendpolicy loaded from configuration");

            // Create the request
            Request request = new Request(endpointAddress, credentials, sendPolicy);
            log.info("Request created - send...");
            
            // Enable proxy
            // Use this code to send through a proxy (eg TcpMon) for logging purposes
            // request.setProxyHost("localhost");
            // request.setProxyPort( 2222 );
  
            // Send request to service
            Response response = request.getResponse(oiosiMessage);
            log.info("Received response from service : " + response.getResponseMessage().getMessageXml().getOMDocument());

        } catch (Exception e) {
            log.error("Failed to send message to service : " + e, e);
        }

        try {
            // Let RM Session finish...
            Thread.sleep(5000);
            log.info("Done...");
        } catch (Exception igore) {
        }

    }
}
