/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.firma.klient.webservice;

import java.security.cert.X509Certificate;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.TimeSpan;
import dk.gov.oiosi.common.Utilities;
import dk.gov.oiosi.common.cache.TimedCache;
import dk.gov.oiosi.communication.Credentials;
import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.Request;
import dk.gov.oiosi.communication.Response;
import dk.gov.oiosi.communication.SendPolicy;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.ldap.LdapLookupFactory;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.oces.OcesX509Certificate;
import dk.gov.oiosi.security.ocsp.IOcspLookup;
import dk.gov.oiosi.security.ocsp.OcspLookupFactory;
import dk.gov.oiosi.security.ocsp.OcspResponse;
import dk.gov.oiosi.uddi.IUddiLookupClient;
import dk.gov.oiosi.uddi.LookupParameters;
import dk.gov.oiosi.uddi.LookupReturnOption;
import dk.gov.oiosi.uddi.PreferredEndpointType;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupClientFactory;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * Invoke RASP call against ITST published RASP test service.
 * 
 */
public class OiosiRaspClient {


    private static Log log = LogFactory.getLog(OiosiRaspClient.class);
    public static final String PATH_INVOICE_XML = "./Xml/OIOUBL_Invoice_v2p1.xml";

    
    /**
     * RASP Test application
     * 
     * This test client goes through the following steps:
     * 
     *  1. Load a document
     *  2. Validate the document
     *  3. Wrap the document in a SOAP message and add the custom RASP headers
     *  4. Make an UDDI lookup to find the service address
     *  5. Use information from the UDDI response to download the server certificate via LDAP
     *  6. Validate the certificate downloaded against an OCSP server
     *  7. Sends it to a service deployed on localhost (by default: the service_skeleton service)
     *
     */
    public static void main(final String[] args) {
        try {
            log.info("Try to send message to service");
            
            // Set the path to the custom RASP configuration file. 
            // This file contains setup for things like:
            //   - Addresses to OCSP, LDAP and UDDI servers
            //   - The path to the Axis2 modules needed (RASP, WS-Security and WS-ReliableMessaging)
            //   - Configuration for the different OIO UBL document types
            RaspConfigurationHandler.setConfigurationFile("./conf/RaspConfiguration.xml");

            
            // Step 1: Load a document
            
            // Load the document from disc
            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.load(PATH_INVOICE_XML);
            log.info("Document created from file");

            // Search our configuration file for the document type we are currently about to send.
            // The UBL type can be inferred from the XML root element name of the file, and the configuration we get
            // contains paths to XML schemas, schematron stylesheets and XPath expressions used to find the EAN number for example
            DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
            DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);
            log.info("Document is known in configuration");

            
            // Step 2: Validate the document
            
            // Schema validate document before sending...
            SchemaValidator schemaValidator = new SchemaValidator();
            XmlSchema schema = new XmlSchema(documentTypeConfig.getSchemaPath());
            schemaValidator.schemaValidateXmlDocument(xmlDocument, schema);
            log.info("Document is schema valid");

            // Schematron validate Document before sending...
            SchematronValidator schematronValidator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());
            schematronValidator.schematronValidateXmlDocument(xmlDocument);
            log.info("Document is schematron/logically valid");

            
            // Step 3: Wrap the document in a SOAP message and add the custom RASP headers

            // Create message wrapper
            OiosiMessage oiosiMessage = new OiosiMessage(xmlDocument);
            oiosiMessage.setRequestAction(documentTypeConfig.getEndpointType().getRequestAction());
            oiosiMessage.setReplyAction(documentTypeConfig.getEndpointType().getReplyAction());
            log.info("We have defined Document to send.");

            // Find the sender and receiver IDs (e.g. EAN numbers)
            IIdentifier identifier = Utilities.getReceiverEndpointKey(xmlDocument);
            IIdentifier senderIdentifier = Utilities.getSenderEndpointKey(xmlDocument);

            // Add them as headers, and add an unique MessageIdentifier header
            oiosiMessage.addMessageHeader(Constants.RASP_MESSAGE_IDENTIFIER, "urn:uuid:" + System.nanoTime());
            oiosiMessage.setReceiverPartyIdentifier(identifier);
            oiosiMessage.setSenderPartyIdentifier(senderIdentifier);

            
            // Step 4: Make UDDI Lookup
            
            // Configure the lookup
            EndpointKeytype endpointKeyType = new EndpointKeytype(new EndpointKeyTypeCode(identifier.getType()));
            LookupParameters parameters = new LookupParameters(
            		identifier,		  // The identifier 
            		endpointKeyType,  // What type of identifier is it (eg EAN)
            		(List) null, 	  // We have no filters
            		new PreferredEndpointType(PreferredEndpointType.http),  // We prefer a http endpoint...
                    new LookupReturnOption(LookupReturnOption.firstResult),  //... and just the first that is returned              
                    new UddiGuidId(documentTypeConfig.getServiceContractTModel(), true), // The RASP service contract tModel (id in UDDI)
                    (BusinessProcessRoleIdentifierType) null, // role identifier type
                    (BusinessProcessRoleIdentifier) null, // role identifier
                    (UddiId) null // process definition tModel key
            );

            // Create a cache for the UDDI lookup
            TimedCache uddiCache = new TimedCache(new TimeSpan().setHours(24));
            parameters.setLookupCache(uddiCache);

            // Look it up...
            log.info("Ask UDDI :");
            IUddiLookupClient uddiClient = UddiLookupClientFactory.createUddiLookupClient();
            List uddiResult = uddiClient.lookup(parameters);
            log.info("Found # in uddi : " + uddiResult.size());
            if (uddiResult.size() == 0) {
                log.error("No Information found in UDDI");
                return;
            }
            UddiLookupResponse uddiResponse = (UddiLookupResponse) uddiResult.get(0);
            log.info("Found endpoint         : " + uddiResponse.getEndpointAddress().getKeyAsString());
            log.info("Found Certificate Info : " + uddiResponse.getCertificateSubjectSerialNumber().getSerialNumberValue());

            
            // Step 5: LDAP Lookup
            
            // Get the certificate identifier from the UDDI results
            CertificateSubject certificateSubject = uddiResponse.getCertificateSubjectSerialNumber();
            
            // Use a factory to create an LDAP client, and download the certicate
            ICertificateLookup lookup = LdapLookupFactory.createLdapLookupClient();
            X509Certificate server = lookup.getCertificate(certificateSubject);
            log.info("Found Certificate in LDAP : " + server.getSubjectDN());

          
            // Step 6: OCSP Check
            
            // Use a factory to create an OCSP client, and check the certificate status
            IOcspLookup ocspLookup = new OcspLookupFactory().createOcspLookupClient();
            OcspResponse ocspResponse = ocspLookup.checkCertificate(server);
            log.info("Certificate is valid : " + ocspResponse.isValid());

            
            //Step 7: Send it
            
            // Use the downloaded certificate as a server side credential
            OcesX509Certificate ocesCertificate = new OcesX509Certificate(server);
            Credentials credentials = new Credentials(ocesCertificate);
            log.info("Destination service certificate : " + ocesCertificate.getSubject().getSerialNumberValue());

            // Use policy defined in RaspConfiguration.xml
            SendPolicy sendPolicy = RaspConfigurationHandler.getInstance().getSendPolicy();
            log.info("Sendpolicy loaded from configuration");

            // Create a request
            Request request = new Request(uddiResponse.getEndpointAddress().getAsUri(), credentials, sendPolicy);

            // Enable proxy
            // Use this code to send through a proxy (eg TcpMon) for logging purposes
            // request.setProxyHost("localhost");
            // request.setProxyPort( 2222 );

            // Send request to service
            Response response = request.getResponse(oiosiMessage);
            log.info("Received response from service : " + response.getResponseMessage().getMessageXml().getOMDocument());

        } catch (Exception e) {
            log.error("Failed to send message to service : " + e, e);
        }

        try {
            // Let the ReliableMessaging session finish...
            Thread.sleep(5000);
            log.info("Done...");
        } catch (Exception ignore) {
        }
    }
}
