/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.net.Socket;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.ldap.LdapLookupFactory;
import dk.gov.oiosi.security.lookup.ICertificateLookup;
import dk.gov.oiosi.security.ocsp.CertificateRevokedException;
import dk.gov.oiosi.security.ocsp.IOcspLookup;
import dk.gov.oiosi.security.ocsp.OcspLookupFactory;
import dk.gov.oiosi.security.ocsp.OcspResponse;

/**
 * Performs basic check of the environment to check if RASP will run at all.
 */
public class RaspEnvironmentCheck {
    
    private static Log log = LogFactory.getLog(RaspEnvironmentCheck.class);
    
    static {
        try {
            String testXml = "RaspConfigurationTestServers.xml";
            RaspConfigurationHandler.setConfigurationFile(testXml);
            RaspConfigurationHandler.getInstance();
        } catch (Exception e) {
            log.error("Could not init RaspConfiguration");
        }
    }

    private static final String SERIALNUMBER_CVR_RID = "SERIALNUMBER=CVR:25767535-RID:1118061020232";

    private static final String OKMSG = "Check OK...";

    /**
     * @param args
     */
    public static void main(String[] args) {
        RaspEnvironmentCheck raspEnv = new RaspEnvironmentCheck();

        System.out.print("Checking OS ");
        System.out.println(" - " + raspEnv.checkOS());
        System.out.println();
        
        /*
         * These checks are only needed for a binary release
         * 
         * 
	        System.out.print("Checking Axis2");
	        System.out.println(" - " + raspEnv.checkForAxis());
	        System.out.print("Checking Mercury");
	        System.out.println(" - " + raspEnv.checkForMercury());
	        System.out.print("Checking Rampart");
	        System.out.println(" - " + raspEnv.checkForRampart());
	        System.out.println();
        */
        
        System.out.print("Checking JVM version ");
        System.out.println(" - " + raspEnv.checkJVM());

        System.out.print("Checking encryption ");
        System.out.println(" - " + raspEnv.checkEncryption());
        

        System.out.print("Checking for Ant ");
        System.out.println(" - " + raspEnv.checkForAnt());
        System.out.println();
        
        System.out.print("Connecting to server (Tomcat or similar) ");
        System.out.println(" - " + raspEnv.checkForServer());
        System.out.println();
        
        System.out.print("Checking LDAP ");
        System.out.println(" - " + raspEnv.checkLDAP());

        System.out.print("Checking and OCSP validating certificate ");
        System.out.println(" - " + raspEnv.checkOCSPLookup());

    }

    private static final String SUBJECTDN = "SERIALNUMBER=CVR:26769388-UID:1172691221366 + CN=IT- og Telestyrelsen - OIOSI test virksomhedssignatur, O=IT- og Telestyrelsen // CVR:26769388, C=DK";

    X509Certificate cert;

    /**
     * 
     */
    public String checkOCSPLookup() {
        String check =OKMSG;

        OcspLookupFactory lookupFactory = new OcspLookupFactory();

        // check ocsp for known certificate
        IOcspLookup lookup = null;
        try {
            lookup = lookupFactory.createOcspLookupClient();
            
            if( cert != null ) {
                OcspResponse response = lookup.checkCertificate( cert );

                if( ! response.isValid() ) {
                    check = "Certifcate is not valid : " + cert.getSubjectDN();
                }
            } else {
                check = "Certifice == null - no OCSP check";
            }
        } catch(Exception e) {
            check = "Failed to perform OCSP check for : " + cert.getSubjectDN() + " - Failure : " + e.getMessage();
        }
        // Check certificates in keystore
        if( lookup != null ) {
            String key = null;
            try {
                check = "Checking certificates : " ;

                KeyStore ks = KeyStore.getInstance( "JKS" );
                try {
                    // first try client keystore :
                    ks.load( this.getClass().getClassLoader().getResourceAsStream("keys/oiosi_test_certificates.jks"), "Test1234".toCharArray() );
                    if( ks.size() == 0 ) {
                        ks.load( new FileInputStream("keys/oiosi_test_certificates.jks"), "Test1234".toCharArray() );
                    }
                } catch(Exception e) {
                    System.out.println("Failed to load client keystore " + e );
                }
                if( ks == null || ks.size() == 0 ) {
                    // fall back to server keystore :
                    try {
                        ks.load( this.getClass().getClassLoader().getResourceAsStream("../keys/oiosi_server.jks"), "Test1234".toCharArray() );
                    } catch( Exception e) {
                        return "Could not load filestore : keys/oiosi_test_certificates.jks or keys/oiosi_server.jks : " + e.getMessage();
                    }
                }
                if( ks.size() == 0 ) {
                    return "No certificates found in keystore : "+ ks.size();
                }
                Enumeration enumeration = ks.aliases();
                while( enumeration.hasMoreElements() ) {
                    key = enumeration.nextElement().toString();
                    if( ks.isKeyEntry( key ) ) {
                        try {
                            X509Certificate cert = (X509Certificate) ks.getCertificate( key );
                            lookup.checkCertificate(cert);
                            check += "\n  -  ";
                            check += "keylabel : " + key + " - OK : " + cert.getSubjectDN() ;
                        } catch(CertificateRevokedException e ) {
                            check += "\n  -  ";
                            check += "keylabel : " + key + " - has been Revoked";
                        }
                    } else {
                        check += "\n  -  ";
                        check += "keylabel : " + key + " - has no key - is this root : " + key;
                    }                    
                }
            } catch (Exception e) {
                check += "\n";
                check += "keylabel : " + key + " - failed with : " + e.getMessage();
                // check="Checking and validating OCSP local certificate - unable to check/validate certificate due to: "+ e.getMessage()+ "!";
                check+=" - Is the certifcate imported at the client ?";
                
            }
        }
        return check;
    }

    /**
     * 
     */
    public String checkLDAP() {
        String check = OKMSG;
        ICertificateLookup ldapLookup = LdapLookupFactory.createLdapLookupClient();
        try {
            CertificateSubject certSubject = new CertificateSubject(SUBJECTDN);
            cert = ldapLookup.getCertificate(certSubject);

            if (cert == null) {
                check = "Checking LDAP - unable to get certificate: " + SUBJECTDN;
            }
        } catch (Exception e) {
            check = "Failed to lookup certificate " + SUBJECTDN;
        }
        return check;
    }

    public String checkForServer() {
        String check = OKMSG;
        String host = "localhost";
        int port = 8080;
        try {
            Socket s = new Socket(host, port);
        } catch (Exception e) {
            check = "Unable to connect to '" + host + ":" + port + "' ! Is the server started ?";
        }
        return check;
    }

    /**
     * 
     */
    public String checkForTomcat() {
        String check = OKMSG;
        String catalina = "catalina.properties";
        File rootFile = File.listRoots()[0];
        try {
            if (!(getTomcatPath(rootFile, catalina).size() > 0)) {
                check = "Checking for Tomcat - not able to find " + catalina;
            }
        } catch (FileNotFoundException e) {
            check = "Checking for Tomcat - not able to find " + catalina;
        }
        return check;
    }

    
    public String checkForAxis() {
        String check = OKMSG;
        URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
        try {
			sysloader.loadClass("org.apache.axiom.om.OMNode");
		} catch (ClassNotFoundException e) {
			return "Couldn't find Axis2 on classpath. ";
		}
       
        return check;
    }
    
    public String checkForMercury() {
    	String check = OKMSG;
        URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
        try {
			sysloader.loadClass("org.wso2.mercury.handlers.MercuryOutHandler");
		} catch (ClassNotFoundException e) {
			return "Couldn't find Mercury on classpath. ";
		}
       
        return check;
    }
    
    public String checkForRampart() {
        String check = OKMSG;
        URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
        try {
			sysloader.loadClass("org.apache.rampart.RampartEngine");
		} catch (ClassNotFoundException e) {
			return "Couldn't find Rampart on classpath. ";
		}
       
        return check;
    }
    
    /**
     * 
     */
    public String checkForAnt() {
        String check = OKMSG;
        String path = System.getProperty("java.library.path");
        if (!path.contains("ant") && !path.contains("Ant")) {
            check = "Checking for Ant - unable to locate Ant library in path: " + path + "! ";
            check += " - Is Ant installed ? Can ant be run from a command prompt ?";
        }
        return check;
    }

    /**
     * 
     */
    public String checkJVM() {
        String check = OKMSG;
        String jversion = System.getProperty("java.vm.version");
        if (!jversion.startsWith("1.4.2") && !jversion.startsWith("1.5") && !jversion.startsWith("1.6") && !jversion.startsWith("10.")) {
            check = "Unsupported JVM version: " + jversion;
            check += "- Is JDK 1.4.2, 1.5 or 1.6 installed ? Is java_home set ?";
        }
        return check;
    }

    /**
     * 
     */
    public String checkOS() {
        String check = OKMSG;
        String os = System.getProperty("os.name");
        if (!os.startsWith("Windows") && !os.startsWith("Mac") && !os.startsWith("Linux")) {
            check = "Untested os: " + os;
        }
        return check;
    }

    /**
     * 
     */
    public String checkEncryption() {
        String check = OKMSG;
        try {
            KeyGenerator kgen = KeyGenerator.getInstance("AES");
            kgen.init(256); // 192 and 256 bits may not be available
            kgen.init(new SecureRandom());
            SecretKey derived_key = kgen.generateKey();
            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
            cipher.init(Cipher.ENCRYPT_MODE, derived_key);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            baos.write(cipher.getIV());
            baos.write(cipher.doFinal("Very secret secret".getBytes()));
            baos.close();

        } catch (Exception e) {
            check = "Checking encryption - not able to encrypt due to: " + e.getMessage();
            check += " - Is the policy files installed in the JRE ?";
        }
        return check;
    }

    /**
     * Directory is valid if it exists, does not represent a file, and can be
     * read.
     */
    private void validateDirectory(File aDirectory) throws FileNotFoundException {
        if (aDirectory == null) {
            throw new IllegalArgumentException("Directory should not be null.");
        }
        if (!aDirectory.exists()) {
            throw new FileNotFoundException("Directory does not exist: " + aDirectory);
        }
        if (!aDirectory.isDirectory()) {
            throw new IllegalArgumentException("Is not a directory: " + aDirectory);
        }
        if (!aDirectory.canRead()) {
            throw new IllegalArgumentException("Directory cannot be read: " + aDirectory);
        }
    }

    private List getTomcatPath(File rootFile, String stopFileName) throws FileNotFoundException {
        validateDirectory(rootFile);
        List result = new ArrayList();

        File[] filesAndDirs = rootFile.listFiles();
        List filesDirs = Arrays.asList(filesAndDirs);
        Iterator filesIter = filesDirs.iterator();
        File file = null;
        while (filesIter.hasNext()) {
            file = (File) filesIter.next();
            if (!file.isFile()) {
                // must be a directory
                // recursive call!
                List deeperList = getTomcatPath(file, stopFileName);
                result.addAll(deeperList);
                if (result.size() > 0) {
                    break;
                }
            } else if (file.getName().equalsIgnoreCase(stopFileName)) {
                result.add(file);
                break;
            }
        }
        Collections.sort(result);
        return result;
    }
}
