/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.axis2.module.signatureproof;

import java.security.cert.X509Certificate;
import java.util.Calendar;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.rampart.handler.WSSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerConstants;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.security.SignatureValidationProofAllreadyCompletedException;
import dk.gov.oiosi.security.ocsp.IOcspLookup;
import dk.gov.oiosi.security.ocsp.OcspException;
import dk.gov.oiosi.security.ocsp.OcspLookupFactory;
import dk.gov.oiosi.security.ocsp.OcspResponse;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;
import dk.gov.oiosi.util.AxisWSSUtil;

/**
 * Create signature proof in axis stack.
 */
public class SignatureProofHandler extends RaspHandler {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(SignatureProofHandler.class);

    /**
     * name.
     */
    public static final String NAME = "SignatureProofHandler";

    /**
     * Description.
     */
    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("deafult Handler");

    /**
     * Description.
     */
    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public SignatureProofHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#init(org.apache.axis2.description.HandlerDescription)
     */
    public void init(HandlerDescription handlerdesc) {
        this.handlerDesc = handlerdesc;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#invoke(org.apache.axis2.context.MessageContext)
     */
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {

        log.debug("SignatureProofHandler : invoke");
        log.error("SignatureProofHandler : invoke : " + msgContext.getTo());

        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
            return InvocationResponse.CONTINUE;
        }
        if (!msgContext.isEngaged(WSSHandlerConstants.SECURITY_MODULE_NAME)) {
            // bail out
            return InvocationResponse.CONTINUE;
        }

        if (RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction())) {
            // only for payload - skip
            return InvocationResponse.CONTINUE;
        }

        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);
        if (!oiosiPolicyBean.doSignatureProof()) {
            // - policy defined skip...");
            log.error("SignatureProofHandler : policy defined skip ??? +" + oiosiPolicyBean.doSignatureProof());
            return InvocationResponse.CONTINUE;
        }

        // Extract data for SignatureProof

        X509Certificate signerCertificate = null;

        Object recv_results = msgContext.getProperty(WSHandlerConstants.RECV_RESULTS);
        if (recv_results != null) {
            signerCertificate = AxisWSSUtil.getCertificateFrom_RECV_RESULTS(recv_results);

            if (signerCertificate != null) {
            	msgContext.setProperty(Constants.RASP_SIGNING_CERTIFICATE, signerCertificate);
            	
                // Set validation time to rifht now
                Calendar now = Calendar.getInstance();

                // Check the validity of the certificate via an OCSP lookup
                boolean certificateValid = true;
                log.debug("Check Certificate : " + signerCertificate.getSubjectX500Principal());
                OcspResponse response = null;
                try {
                    OcspLookupFactory lookupFactory = new OcspLookupFactory();
                    IOcspLookup lookup;
                    lookup = lookupFactory.createOcspLookupClient();

                    response = lookup.checkCertificate(signerCertificate);

                    if (!response.isValid()) {
                        log.error("Certifcate is not valid.");
                        certificateValid = false;
                    }
                } catch (OcspException e) {
                    log.warn("Validation of certificate failed " + signerCertificate.getSubjectX500Principal() + " : " + e);
                    certificateValid = false;
                } catch (MainException e) {
                    log.error("Validation of certificate failed " + signerCertificate.getSubjectX500Principal() + " : " + e, e);
                    certificateValid = false;
                } catch (Exception e) {
                    log.error("Could not validate incoming signer certificate : " + e, e);
                    certificateValid = false;
                }

                try {
                    CertificateValidator.validateCertificate(signerCertificate);
                } catch (CertificateNotActiveException e) {
                    log.warn("Incoming certificate not valid yet : " + signerCertificate.getSubjectX500Principal());
                    certificateValid = false;
                } catch (CertificateExpiredException e) {
                    log.warn("Incoming certificate is expired : " + signerCertificate.getSubjectX500Principal());
                    certificateValid = false;
                }
                try {
                    SignatureValidationProof signatureProof = new SignatureValidationProof();
                    signatureProof.setCertificateSubject(signerCertificate.getSubjectX500Principal().toString());
                    signatureProof.setValidCertificate(certificateValid);
    
                    // Rampart do not let invalid signatures through
                    signatureProof.setValidSignature(true);
                    signatureProof.setUnchangedMessage(true);
    
                    // Check if message was encrypted
                    signatureProof.setEncryptedMessage(AxisWSSUtil.getMessageEncryptedFrom_RECV_RESULTS(recv_results));
    
                    // Get timestamp from SecurityHeader 
                    signatureProof.setTimeStamp(AxisWSSUtil.getTimestampCreatedFrom_RECV_RESULTS(recv_results));
    
                    // set completed
                    signatureProof.setCompleted();
    
                    msgContext.setProperty(Constants.RASP_SIGNATURE_PROOF, signatureProof);
                } catch (SignatureValidationProofAllreadyCompletedException thisShouldNotHappen) {
                    log.error("SignatureProof could not be created : " + thisShouldNotHappen, thisShouldNotHappen);
                }

            }

        }

        return InvocationResponse.CONTINUE;

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#flowComplete(org.apache.axis2.context.MessageContext)
     */
    public void flowComplete(MessageContext arg0) {
        // Do nothing...
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getName()
     */
    public String getName() {
        return NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getParameter(java.lang.String)
     */
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getHandlerDesc()
     */
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }

}
