/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.axis2.module.xml;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.Parameter;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.axis2.module.RaspHandler;
import dk.gov.oiosi.axis2.module.RaspModule;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.AxisFaultUtil;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.policy.OiosiPolicyBean;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.schematron.SchematronErrorException;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * Schematron validation in axis stack.
 */
public class SchematronHandler extends RaspHandler {

    /**
     * local log.
     */
    private static final Log log = LogFactory.getLog(SchematronHandler.class);

    /**
     * name.
     */
    public static final String NAME = "SchematronHandler";

    /**
     * Description.
     */
    private static HandlerDescription EMPTY_HANDLER_METADATA = new HandlerDescription("deafult Handler");

    /**
     * Description.
     */
    private HandlerDescription handlerDesc;

    /**
     * Constructor.
     */
    public SchematronHandler() {
        this.handlerDesc = EMPTY_HANDLER_METADATA;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#init(org.apache.axis2.description.HandlerDescription)
     */
    public void init(HandlerDescription handlerdesc) {
        this.handlerDesc = handlerdesc;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#invoke(org.apache.axis2.context.MessageContext)
     */
    public InvocationResponse invoke(MessageContext msgContext) throws AxisFault {

        log.debug("SchematronHandler : invoke");

        if (!msgContext.isEngaged(RaspModule.MODULE_NAME)) {
            return InvocationResponse.CONTINUE;
        }

        if (RaspModule.getInstance().isRMNamespaceByAction(msgContext.getSoapAction())) {
            // only for payload requests
            return InvocationResponse.CONTINUE;
        }

        OiosiPolicyBean oiosiPolicyBean = getPolicyBean(msgContext);

        if (oiosiPolicyBean.doSchematron()) {
            log.debug("SchematronHandler : do schematron ");

            OMElement doc = msgContext.getEnvelope().getBody().getFirstElement();
            XmlDocument xmlDocument = null;
            // Find Document
            DocumentTypeConfig documentTypeConfig = null;
            try {
                log.debug("Find DocumentTypeConfig");
                xmlDocument = new XmlDocument(doc.cloneOMElement());

                DocumentTypeConfigSearcher documentSearcher = new DocumentTypeConfigSearcher();
                documentTypeConfig = documentSearcher.findUniqueDocumentType(xmlDocument);
            } catch (MainException e) {
                log.error("Failed init/find documnet config : " + e);
                throw AxisFaultUtil.buildAxisFault(msgContext, e);
            } catch (Exception e) {
                log.error("Failed init/find documnet config : " + e, e);
                throw AxisFaultUtil.buildAxisFault(msgContext, new MainException(e));
            }

            // Schematron Check document
            try {
                log.debug("Schematron Validate Document.");
                if (documentTypeConfig.getSchematronValidationConfig() == null) {
                    log.debug(" - skip ");
                } else {
                    log.debug(" - doit ");
                    SchematronValidator validator = new SchematronValidator(documentTypeConfig.getSchematronValidationConfig());

                    validator.schematronValidateXmlDocument(xmlDocument);
                    log.error("- Schematron validate OK ");
                }
            } catch (SchematronErrorException e) {
                log.debug("Failed schema validate document : " + e);
                throw AxisFaultUtil.buildAxisFault(msgContext, e);
            } catch (Exception e) {
                log.error("Failed schema validate document : " + e, e);
                throw AxisFaultUtil.buildAxisFault(msgContext, new MainException(e));
            }
        }
        return InvocationResponse.CONTINUE;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#flowComplete(org.apache.axis2.context.MessageContext)
     */
    public void flowComplete(MessageContext arg0) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getName()
     */
    public String getName() {
        return NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getParameter(java.lang.String)
     */
    public Parameter getParameter(String name) {
        return this.handlerDesc.getParameter(name);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.Handler#getHandlerDesc()
     */
    public HandlerDescription getHandlerDesc() {
        return handlerDesc;
    }
}
