/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.common;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.EndpointAddressSMTP;
import dk.gov.oiosi.addressing.EndpointAddressHttp;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.addressing.IdentifierOvt;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.exception.ArgumentException;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiNonGuidId;
import dk.gov.oiosi.uddi.category.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.category.UnknownEndpointTypeException;

/**
 * Utilities for converting identifiers between strings and other types.
 */
public class IdentifierUtility {

    /**
     * Hide constructor.
     */
    private IdentifierUtility() {
    }

    /**
     * Returns an UDDI identifier (e.g. a tModel key) as an UddiId object (e.g.
     * UddiGuidId or UddiNonGuidId). Throws an exception if the format is not
     * right.
     * 
     * @param uddiIdentifier
     *            The UDDI identifier to convert
     * @return the UddiId subclass intance. Throws an exception if the format is
     *         not right
     */
    public static UddiId getUddiIDFromString(final String uddiIdentifier) {
        UddiId idObject;
        if (uddiIdentifier.toLowerCase().startsWith("uddi:")) {
            idObject = new UddiGuidId(uddiIdentifier, true);
            return idObject;
        } else {
            if (UddiGuidId.isValidGuidId(uddiIdentifier, false)) {
                idObject = new UddiGuidId(uddiIdentifier, false);
                return idObject;
            } else {
                idObject = new UddiNonGuidId(uddiIdentifier);
                return idObject;
            }
        }
    }

    /**
     * From an identifier string and identifier type, returns the right
     * IIdentifier subclass.
     * 
     * @param endpointKey
     *            The endpoint key as string
     * @param endpointKeyType
     *            The type of the key
     * @return the relevant IIdentifier subclass
     * @throws UnknownEndpointTypeException
     *             unknow endpoint type
     */
    public static IIdentifier getIdentifierFromKeyType(final String endpointKey, final EndpointKeyTypeCode endpointKeyType)
            throws UnknownEndpointTypeException {
        IIdentifier id;
        switch (endpointKeyType.getCode()) {
        case EndpointKeyTypeCode.cvr:
            id = new IdentifierCvr(endpointKey);
            break;
        case EndpointKeyTypeCode.ean:
            id = new IdentifierEan(endpointKey);
            break;
        case EndpointKeyTypeCode.ovt:
            id = new IdentifierOvt(endpointKey);
            break;
        default:
            id = null;
            throw new UnknownEndpointTypeException("Unknown : " + endpointKeyType.getCode());
        }

        return id;
    }

    /**
     * Takes an endpoint an returns a endpointaddress
     * 
     * @param endpointAddress
     *            the endpoint address
     * @return a specific endpointaddress
     * @throws ArgumentException
     *             argument is illegal
     */
    public static EndpointAddress getEndpointAddressFromString(final String endpointAddress) {

        EndpointAddress address = null;

        if (endpointAddress.toLowerCase().startsWith("http://")) {
            address = new EndpointAddressHttp(new Uri(endpointAddress));
        } else if (endpointAddress.toLowerCase().startsWith("mailto:")) {
            address = new EndpointAddressSMTP(new Uri(endpointAddress));
        } else {
            address = new EndpointAddressSMTP(new Uri(endpointAddress));
        }
        return address;
    }
}
