/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.common;

import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.communication.OiosiMessage;
import dk.gov.oiosi.communication.configuration.KeyTypeMapping;
import dk.gov.oiosi.communication.configuration.KeyTypeMappingExpression;
import dk.gov.oiosi.communication.configuration.KeyTypeMappingFailedException;
import dk.gov.oiosi.communication.configuration.DocumentEndpointInformation;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.category.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.category.UnknownEndpointTypeException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.xpath.DocumentXPathResolver;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;

/**
 * Utilities that can be used for UDDI endpoint handling.
 */
public class Utilities {

    /**
     * Hide constructor.
     */
    private Utilities() {
    }

    /**
     * From an XPath expression, gets the endpoint key.
     * 
     * @param xmlDoc
     *            The XML-document to search
     * @param xpath
     *            The XPath-expression to apply
     * @param keyType
     *            The type of key to return
     * @param prefixedNamespaces
     *            The list of namespaces-prefix pairs used in the xpath
     *            expression
     * @return Returns the endpoint key
     * @throws UnknownEndpointTypeException
     * @throws XmlException
     */
    public final static IIdentifier getEndpointKeyByXpath(XmlDocument xmlDoc, String xpath, PrefixedNamespace[] prefixedNamespaces,
            EndpointKeyTypeCode keyType) throws UnknownEndpointTypeException, XmlException {
        // 1. Get the endpoint as string:
        String endpointKeyString = DocumentXPathResolver.getElementValueByXpath(xmlDoc, xpath, prefixedNamespaces);

        // 2. Convert to the correct IIdentifierType:
        IIdentifier id = IdentifierUtility.getIdentifierFromKeyType(endpointKeyString, keyType);

        return id;
    }

    /**
     * Gets the endpoint key type code from a RaspMessage and a
     * RaspDocumentTypeConfig.
     * 
     * @param message
     * @param documentType
     * @return the endpoint key type code
     */
    public static EndpointKeyTypeCode getEndpointKeyTypeCode(OiosiMessage message, DocumentTypeConfig documentType)
            throws KeyTypeMappingFailedException, XmlException {
        XmlDocument xmlDocument = message.getMessageXml();
        return getEndpointKeyTypeCode(xmlDocument, documentType);
    }

    /**
     * Gets the endpoint key type code from a RaspMessage and a
     * RaspDocumentTypeConfig.
     * 
     * @param xmlDocument
     * @param documentType
     * @return the endpoint key type code
     */
    public static EndpointKeyTypeCode getEndpointKeyTypeCode(XmlDocument xmlDocument, DocumentTypeConfig documentType)
            throws XmlException, KeyTypeMappingFailedException {

        DocumentEndpointInformation endpointType = documentType.getEndpointType();
        KeyTypeMappingExpression mappingExpression = endpointType.getKey().getMappingExpression("EndpointKeyType");
        
        return getKeyTypeCode(xmlDocument, documentType, mappingExpression);
    }

    /**
     * Gets the endpoint key type code from a RaspMessage and a
     * RaspDocumentTypeConfig.
     * 
     * @param xmlDocument
     * @param documentType
     * @return the endpoint key type code
     */
    public static EndpointKeyTypeCode getKeyTypeCode(XmlDocument xmlDocument, DocumentTypeConfig documentType, KeyTypeMappingExpression mappingExpression)
                throws XmlException, KeyTypeMappingFailedException {

        // Finds the endpoint key type value from the given XPath
        String xpathExpression = mappingExpression.getXPathExpression();
        String endPointKeyTypeValue = DocumentXPathResolver.getElementValueByXpath(xmlDocument, xpathExpression, documentType
                .getNamespaces());
        KeyTypeMapping mapping = mappingExpression.getMapping(endPointKeyTypeValue);

        String mapsTo = mapping.getMapsTo();

        // Statically maps the maps to to a code
        return new EndpointKeyTypeCode(mapsTo);
    }

    
    /**
     * Retrieves Receiver endpoint key from document.
     * 
     * @param xmlDocument document
     * @return the identifier
     * @throws MainException
     */
    public static IIdentifier getReceiverEndpointKey(XmlDocument xmlDocument) throws MainException {
        DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);
        
        return getReceiverEndpointKey(xmlDocument, documentTypeConfig);
    }

    /**
     * Retrieves Receiver endpoint key from document.
     * 
     * @param xmlDocument document
     * @param documentTypeConfig configuration for document
     * @return the identifier
     * @throws MainException
     */
    public static IIdentifier getReceiverEndpointKey(XmlDocument xmlDocument, DocumentTypeConfig documentTypeConfig) throws MainException {
        PrefixedNamespace[] namespaces = documentTypeConfig.getNamespaces();
        DocumentEndpointInformation endpointType = documentTypeConfig.getEndpointType();
        String keyXpath = endpointType.getKey().getXPath();
        KeyTypeMappingExpression mappingExpression = endpointType.getKey().getMappingExpression("EndpointKeyType");
        EndpointKeyTypeCode code = Utilities.getKeyTypeCode(xmlDocument, documentTypeConfig, mappingExpression);
        IIdentifier identifier = Utilities.getEndpointKeyByXpath(xmlDocument, keyXpath, namespaces, code);
        return identifier;
    }
    
    /**
     * Retrieves Receiver endpoint key from document.
     * @param xmlDocument document
     * @return the identifier
     * @throws MainException
     */
    public static IIdentifier getSenderEndpointKey(XmlDocument xmlDocument) throws MainException {
        DocumentTypeConfigSearcher documentTypeSearcher = new DocumentTypeConfigSearcher();
        DocumentTypeConfig documentTypeConfig = documentTypeSearcher.findUniqueDocumentType(xmlDocument);
        
        return getSenderEndpointKey(xmlDocument, documentTypeConfig);
    }

    /**
     * Retrieves Receiver endpoint key from document.
     * 
     * @param xmlDocument document
     * @param documentTypeConfig configuration for document
     * @return identifier
     * @throws MainException
     */
    public static IIdentifier getSenderEndpointKey(XmlDocument xmlDocument, DocumentTypeConfig documentTypeConfig) throws MainException {
        PrefixedNamespace[] namespaces = documentTypeConfig.getNamespaces();
        DocumentEndpointInformation endpointType = documentTypeConfig.getEndpointType();
        String senderKeyXpath = endpointType.getSenderKey().getXPath();
        KeyTypeMappingExpression senderMappingExpression = endpointType.getSenderKey().getMappingExpression("EndpointKeyType");
        EndpointKeyTypeCode code = Utilities.getKeyTypeCode(xmlDocument, documentTypeConfig, senderMappingExpression);
        IIdentifier identifier = Utilities.getEndpointKeyByXpath(xmlDocument, senderKeyXpath, namespaces, code);
        return identifier;
    }
}
