/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.common.cache;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import dk.gov.oiosi.common.TimeSpan;

/**
 * Simple, generic timed cache type. The cache is traversed each time an element is inserted or extracted.
 * 
 * NOTE: Keys and Values in cached are not 'typed'
 */
public class TimedCache implements ITimedCache {

    /**
     * Lifetime of cache element.
     */
    private TimeSpan cacheTimeOut;

    /**
     * Internal map.
     */
    private Hashtable cache;

    /**
     * TimedCache constructor.
     * 
     * @param t
     *            timespan for cache
     */
    public TimedCache(final TimeSpan t) {
        cacheTimeOut = t;
        cache = new Hashtable();
    }

    /**
     * TimedCache constructor.
     * 
     * @param t
     *            timespan for cache
     */
    public TimedCache(final long t) {
        cacheTimeOut = new TimeSpan(t);
        cache = new Hashtable();
    }

    /**
     * Add to chache.
     * 
     * @param key
     *            search key
     * @param value
     *            object to cache
     */
    public final void add(final Object key, final Object value) {
        long timeOut = System.currentTimeMillis() + cacheTimeOut.getSpanMillis();
        RaspCacheValue cacheValue = new RaspCacheValue(timeOut, value);
        synchronized (cache) {
            checkExpired();
            cache.put(key, cacheValue);
        }
    }

    /**
     * Get from cache. Return null if not in cache.
     * 
     * @param key
     *            search key
     * @return value or null if unknown
     */
    public final Object tryGetValue(final Object key) {
        synchronized (cache) {
            checkExpired();
            RaspCacheValue cacheValue = null;
            cacheValue = (RaspCacheValue) cache.get(key);
            if (cacheValue != null) {
                return cacheValue.getValue();
            } else {
                return null;
            }
        }
    }

    /**
     * Does the cache contain an element with the key Key.
     * 
     * @param key
     *            check key
     * @return true if in cache
     */
    public final boolean containsKey(final Object key) {
        synchronized (cache) {
            checkExpired();
            return cache.containsKey(key);
        }
    }

    /**
     * Remove from cache.
     * 
     * @param key
     *            remove element with key
     */
    public final void remove(final Object key) {
        synchronized (cache) {
            checkExpired();
            cache.remove(key);
        }
    }

    /**
     * Check and remove expired entries.
     */
    private void checkExpired() {
        long now = System.currentTimeMillis();
        List expiredList = new ArrayList();
        Enumeration enumeration = cache.keys();
        while (enumeration.hasMoreElements()) {
            Object key = enumeration.nextElement();
            RaspCacheValue pair = (RaspCacheValue) cache.get(key);
            if (pair.getTimeOut() < now) {
                expiredList.add(key);
            }
        }
        for (int i = 0; i < expiredList.size(); i++) {
            cache.remove(expiredList.get(i));
        }
    }

    /**
     * Internal cache entry class.
     */
    private class RaspCacheValue {

        /**
         * timeout for entry.
         */
        private long timeOut;

        /**
         * cached element.
         */
        private Object value;

        /**
         * Construct cache entry.
         * 
         * @param t
         *            timeout for entry
         * @param v
         *            value of entry
         */
        public RaspCacheValue(final long t, final Object v) {
            timeOut = t;
            value = v;
        }

        /**
         * timeout for entry.
         * 
         * @return timeout
         */
        public final long getTimeOut() {
            return timeOut;
        }

        /**
         * Value.
         * 
         * @return value
         */
        public final Object getValue() {
            return value;
        }
    }
}
