/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication;

import java.security.cert.X509Certificate;
import java.util.Hashtable;

import org.apache.axiom.soap.SOAPHeaderBlock;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlDocumentException;

/**
 * Represents an outbound/incoming message (xml + metadata).
 */
public class OiosiMessage {
    /**
     * The user-defined ID of the message.
     */
    private Guid messageId;
    /**
     * Custom properties.
     */
    private Hashtable properties = new Hashtable();
    /**
     * Mandatory RASP Soap header : SenderPartyIdentifier
     */
    private IIdentifier senderPartyIdentifier;
    /**
     * The soap request action for the message.
     */
    private String requestAction = "*";
    /**
     * Mandatory RASP Soap header : ReceiverPartyIdentifier
     */
    private IIdentifier receiverPartyIdentifier;
    /**
     * Actual Message Payload.
     */
    private XmlDocument messageXml;
        
    /**
     * Constructor.
     * 
     * @param xml
     *            the message
     * @param guid
     *            message id
     * @param relatesTo
     *            relates to id
     */
    public OiosiMessage(final XmlDocument xml, final Guid guid, final Guid relatesTo) {
        messageXml = xml;
        messageId = guid;
        messageRelatesToId = relatesTo;
    }

    /**
     * Constructor.
     * 
     * @param xml
     *            the message
     * @param guid
     *            message id
     */
    public OiosiMessage(final XmlDocument xml, final Guid guid) {
        messageXml = xml;
        messageId = guid;
    }

    /**
     * Constructor that takes the an XML-document as parameter. The XML-Document
     * represents the message body.
     * 
     * @param xml
     *            the message
     */
    public OiosiMessage(final XmlDocument xml) {
        messageXml = xml;
        messageId = Guid.newGuid();
    }

    /**
     * A list of custom headers.
     */
    private Hashtable messageHeaders = new Hashtable();

    /**
     * A list of custom headers.
     * 
     * @return custom headers
     */
    public Hashtable getMessageHeaders() {
        return messageHeaders;
    }

    /**
     * Add a custom headers.
     * 
     * @param key
     *            key for custom header. XML element name
     * @param value
     *            custom header value
     */
    public void addMessageHeader(String key, IIdentifier value) {
        messageHeaders.put(key, value);
    }

    /**
     * Add a custom headers.
     * 
     * @param key
     *            key for custom header. xml element name
     * @param value
     *            custom header value
     */
    public void addMessageHeader(String key, String value) {
        messageHeaders.put(key, value);
    }

    /**
     * Add a custom headers.
     * 
     * @param key
     *            key for custom header
     * @param value
     *            actual soap header to add
     */
    public void addMessageHeader(String key, SOAPHeaderBlock value) {
        messageHeaders.put(key, value);
    }

    /**
     * Set mandatory RASP Soap header : MessageIdentifier
     * @param value 
     */
    public void setMessageIdentifier(Guid value) {
        messageHeaders.put(Constants.RASP_MESSAGE_IDENTIFIER, value);
    }

    /**
     * Set mandatory RASP Soap header : SenderPartyIdentifier
     * @param value 
     */
    public void setSenderPartyIdentifier(IIdentifier value) {
        messageHeaders.put(Constants.RASP_SENDER_PARTY_IDENTIFIER, value);
        senderPartyIdentifier = value;
    }

    /**
     * Get mandatory RASP Soap header : SenderPartyIdentifier
     * @return SenderPartyIdentifier
     */
    public IIdentifier getSenderPartyIdentifier() {
        return senderPartyIdentifier;
    }

    /**
     * Set mandatory RASP Soap header : ReceiverPartyIdentifier
     * @param value 
     */
    public void setReceiverPartyIdentifier(IIdentifier value) {
        messageHeaders.put(Constants.RASP_RECEIVER_PARTY_IDENTIFIER, value);
        receiverPartyIdentifier = value;
    }

    /**
     * Get mandatory RASP Soap header : SenderPartyIdentifier
     * @return SenderPartyIdentifier
     */
    public IIdentifier getReceiverPartyIdentifier() {
        return receiverPartyIdentifier;
    }

    /**
     * Custom properties.
     * 
     * @return properties
     */
    public Hashtable getProperties() {
        return properties;
    }

    /**
     * The soap request action for the message.
     * 
     * @return soap request action
     */
    public String getRequestAction() {
        return requestAction;
    }

    /**
     * The soap request action for the message.
     * 
     * @param value
     *            soap request action
     */
    public void setRequestAction(String value) {
        requestAction = value;
    }

    /**
     * The soap reply action for the message.
     */
    private String replyAction = "*";

    /**
     * The soap reply action for the message.
     * 
     * @return soap reply action
     */
    public String getReplyAction() {
        return replyAction;
    }

    /**
     * The soap reply action for the message.
     * 
     * @param value
     *            soap reply action
     */
    public void setReplyAction(String value) {
        replyAction = value;
    }

    /**
     * Actual Message Payload.
     * 
     * @return message
     */
    public XmlDocument getMessageXml() {
        return messageXml;
    }

    /**
     * Actual Message Payload.
     * 
     * @param e
     *            message
     */
    public void setMessageXml(XmlDocument e) {
        messageXml = e;
    }

    /**
     * Do we have a message body?
     */
    public boolean hasBody() {
        try {
            if (messageXml == null || messageXml.getOMDocument() == null) {
                return false;
            }
        } catch (XmlDocumentException e) {
            return false;
        }
        return true;
    }

    /**
     * MessageId.
     * 
     * @return message id
     */
    public Guid getMessageId() {
        return messageId;
    }

    /**
     * TODO: Find out how to get/set/use this!! Id of a message to which this
     * message relates. Only used for incoming messages.
     */
    private Guid messageRelatesToId;

    /**
     * MessageRelatesToId.
     * 
     * @return relates to
     */
    public Guid getMessageRelatesToId() {
        return messageRelatesToId;
    }

    /**
     * The Certificate which signed the message.
     */
    private X509Certificate signerCertificate;

    /**
     * The Certificate which signed the message.
     * 
     * @param cert
     *            certificate
     */
    public void setSignerCertificate(X509Certificate cert) {
        signerCertificate = cert;
    }

    /**
     * The Certificate which signed the message.
     * 
     * @return certificate
     */
    public X509Certificate getSignerCertificate() {
        return signerCertificate;
    }

    /**
     * The SignatureProof of received message.
     */
    private SignatureValidationProof signatureProof;

    /**
     * The SignatureProof of received message.
     * 
     * @param s
     *            signature proof
     */
    public void setSignatureProof(SignatureValidationProof s) {
        signatureProof = s;
    }

    /**
     * The SignatureProof of received message.
     * 
     * @return signature proof
     */
    public SignatureValidationProof getSignatureProof() {
        return signatureProof;
    }
}
