/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication;

import java.util.Hashtable;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.AxisFault;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.Constants;
import dk.gov.oiosi.common.RaspLibraryException;
import dk.gov.oiosi.communication.client.IAxisDispatcher;
import dk.gov.oiosi.communication.client.ProxyGenerationException;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.security.SignatureValidationProof;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlDocumentException;
import dk.gov.oiosi.xml.XmlSchema;
import dk.gov.oiosi.xml.documenttype.DocumentTypeConfigSearcher;
import dk.gov.oiosi.xml.documenttype.NoDocumentTypeFoundFromXmlDocumentException;
import dk.gov.oiosi.xml.documenttype.SearchForDocumentTypeFromXmlDocumentFailedException;
import dk.gov.oiosi.xml.schema.SchemaValidator;
import dk.gov.oiosi.xml.schematron.SchematronValidator;

/**
 * Represents a request to a RASP HTTP or email endpoint.
 */
public class Request {

    /**
     * Local logger.
     */
    private Log log = LogFactory.getLog(Request.class);

    /**
     * Credentials set by caller.
     */
    Credentials credentials;

    /**
     * Request/Destination URI.
     */
    private Uri requestUri;

    /**
     * Lookup from UDDI.
     */
    private UddiLookupResponse[] uddiResponse;

    /**
     * Policy.
     */
    private SendPolicy policy = null;

    /**
     * Default constructor.
     * 
     * @throws MainException
     */
    public Request() throws MainException {
    }

    /**
     * Constructor
     * 
     * @param endpointAddress
     *            endpoint URI
     */
    public Request(final Uri endpointAddress) throws MainException {
        // Test that a valid endpoint address was given
        testEndpointAddressCompability(endpointAddress.toString());
        setRequestUri(endpointAddress);
        setPolicy(RaspConfigurationHandler.getInstance().getSendPolicy());
    }

    /**
     * Constructor.
     * 
     * @param endpointAddress
     *            endpoint uri
     * @param credentials
     *            credentials
     * @throws MainException
     */
    public Request(final Uri endpointAddress, final Credentials credentials) throws MainException {
        this(endpointAddress);
        setCredentials(credentials);
        setPolicy(RaspConfigurationHandler.getInstance().getSendPolicy());
    }

    /**
     * Creates a new RaspRequest instance bound to the specific endpoint. During
     * construction, the type of endpoint is inferred from the Uri scheme. After
     * construction, additional properties may be set for the request, such as
     * credentials
     * 
     * @param credentials
     *            Overrides the credentials set in config
     * @param endpointAddress
     *            endpoint uri
     * @param sendPolicy
     *            The send policy of the request
     */
    public Request(final Uri endpointAddress, final Credentials credentials, final SendPolicy sendPolicy) throws MainException {
        this(endpointAddress, credentials);
        setPolicy(sendPolicy);
    }

    /**
     * modify RASP call not to use RM.
     */
    private boolean useRM = true;

    /**
     * modify RASP call not to use RM.
     * 
     * @param b
     *            use rm
     */
    public void setUseRM(boolean b) {
        useRM = b;
    }

    /**
     * modify RASP call not to use WSS.
     */
    private boolean useWSS = true;

    /**
     * modify RASP call not to use WSS.
     * 
     * @param b
     *            use wss
     */
    public void setUseWSS(boolean b) {
        useWSS = b;
    }

    /**
     * use this proxyport in HTTP soap request.
     */
    private int proxyPort = -1;

    /**
     * use this proxyport in HTTP soap request.
     * 
     * @param port
     *            proxy port
     */
    public void setProxyPort(int port) {
        proxyPort = port;
    }

    /**
     * use this proxyhost in HTTP soap request.
     */
    private String proxyHost = null;

    /**
     * use this proxyhost in HTTP soap request.
     * 
     * @param host
     *            proxy host
     */
    public void setProxyHost(String host) {
        proxyHost = host;
    }

    /**
     * Fallback axis configuration file.
     */
    private String axisConfigurationFile = "./conf/client_axis2.xml";

    /**
     * Fallback axis configuration file.
     * 
     * @param value
     *            path to configuration file
     */
    public void setAxisConfigurationFile(String value) {
        axisConfigurationFile = value;
    }

    /**
     * Fallback axis repository folder.
     */
    private String axisRepositoryDir = "./test/conf/client";

    /**
     * Fallback axis repository folder.
     * 
     * @param value
     *            path to repository folder
     */
    public void setRepositoryDir(String value) {
        axisRepositoryDir = value;
    }

    /**
     * @deprecated WAS NEVER USED... Configuration directory.
     * @param value
     *            configuration directory
     */
    public void setConfDir(String value) {}

    /**
     * Tests that and endpoint address is compatible with RaspRequest
     * 
     * @param endpointAddress
     *            endpoint URI
     * @throws RequestNoEndpointGivenException
     *             if input == null
     * @throws RequestNotSupportedSchemeException
     *             if unknown transport
     */
    protected void testEndpointAddressCompability(String endpointAddress) throws RequestNoEndpointGivenException,
            RequestNotSupportedSchemeException {

        if (endpointAddress == null)
            throw new RequestNoEndpointGivenException();

        // We only support 'http' and 'mailto'
        String scheme = endpointAddress.toLowerCase();
        if (!scheme.startsWith("mailto") && !scheme.startsWith("http"))
            throw new RequestNotSupportedSchemeException(scheme);
    }

    /**
     * The actual 'request'.
     */
    private IAxisDispatcher dispatcher = null;

    /**
     * Creates the proxy object. Note: This method is inherited from .Net...
     * 
     * @throws ProxyGenerationException
     */
    protected void createProxy() throws ProxyGenerationException {
        log.debug("Proxy being created");

        try {
            dispatcher = IAxisDispatcher.Factory.getDispatcher(axisConfigurationFile, axisRepositoryDir, useWSS, useRM, policy
                    .getWSSPolicyFileName(), policy.getHttpRMPolicy(), policy.getMailRMPolicy() , proxyPort,
                    proxyHost);
        } catch (Exception e) {
            log.error("Failed to crate Axis Dispatcher");
            throw new ProxyGenerationException(e);
        }
    }

    private DocumentTypeConfig documentTypeConfig = null;
    
    private DocumentTypeConfig getDocumentTypeConfig(XmlDocument xmlDocument) throws NoDocumentTypeFoundFromXmlDocumentException, NullArgumentException, SearchForDocumentTypeFromXmlDocumentFailedException, MainException {
        if(documentTypeConfig == null) {
            DocumentTypeConfigSearcher docSearcher = new DocumentTypeConfigSearcher();
            documentTypeConfig = docSearcher.findUniqueDocumentType(xmlDocument);
        }
        return documentTypeConfig;
    }   
    
    private void validateParameters(OiosiMessage message) throws MainException {
        // test endpoint address
        if(requestUri == null) {
            throw new MainException("Missing : endpoint address");
        }
        
        // test credentials
        if(credentials == null) {
            throw new MainException("Missing : credentials");
        }
        
        // test policy
        if(policy == null) {
            throw new MainException("Missing : policy");
        }

        if(policy.doCustomHeaders()) {
            Hashtable customHeaders = message.getMessageHeaders();
            if( customHeaders.get(Constants.RASP_MESSAGE_IDENTIFIER) == null) {
                throw new MainException("Message is missing mandatory custom header : " + Constants.RASP_MESSAGE_IDENTIFIER);
            }
            if( customHeaders.get(Constants.RASP_SENDER_PARTY_IDENTIFIER) == null) {
                throw new MainException("Message is missing mandatory custom header : " + Constants.RASP_SENDER_PARTY_IDENTIFIER);
            }
            if( customHeaders.get(Constants.RASP_RECEIVER_PARTY_IDENTIFIER) == null) {
                throw new MainException("Message is missing mandatory custom header : " + Constants.RASP_RECEIVER_PARTY_IDENTIFIER);
            }
        }
        XmlDocument xmlDocument= message.getMessageXml();
        if(policy.doXmlValidation()) {
            DocumentTypeConfig config = getDocumentTypeConfig(xmlDocument);
            SchemaValidator validator = new SchemaValidator();
            XmlSchema schema = new XmlSchema(config.getSchemaPath());
            validator.schemaValidateXmlDocument(xmlDocument, schema);
        }

        if(policy.doXmlSchematron()) {
            DocumentTypeConfig config = getDocumentTypeConfig(xmlDocument);
            SchematronValidator validator = new SchematronValidator(config.getSchematronValidationConfig());
            validator.schematronValidateXmlDocument(xmlDocument);
        }
    }
    
    /**
     * Synchronously sends a request and gets a response
     * 
     * @param message
     *            Request message
     * @return Response message
     */
    public Response getResponse(OiosiMessage message) throws Exception {
    	log.debug("Send Message ");
    	Response response = null;
    	validateParameters(message);
    	createProxy();
    	response = sendMessage(message);
    	return response;
    }


    
    /**
     * Send message. Gets response. Handles exception.
     * 
     * @param message message
     * @return
     */
    private Response sendMessage(OiosiMessage message) throws MainException {
        Response response = null;
        OMElement soapReply;
    	SignatureValidationProof[] signatureProof = new SignatureValidationProof[1];
    	
        try {
            log.debug("Send request using Axis dispatcher");
            soapReply = dispatcher.sendRaspRequest(message.getMessageXml().getOMDocument(), getRequestUri().toString(), message
                    .getRequestAction(), credentials.getServerCertificate().getCertificate(), message.getMessageHeaders(), message.getProperties(), signatureProof);

            if (soapReply == null) {
                // this is OK, and means that the server replied with an empty SOAP message
                response = new Response(null, null, null, null);
            } else {
                response = new Response(soapReply, null, null, null);
            }
            
            if(signatureProof != null)
            	response.getResponseMessage().setSignatureProof(signatureProof[0]);

        } catch (XmlDocumentException e) {
            log.error("Failed to get document : " + e, e);
            throw e;
        } catch (AxisFault e) {
            log.error("Failed to get document : " + e, e);
            throw new OiosiCommunicationException(e.getMessage());
        } catch (MainException e) {
            throw e;
        } catch (Exception e) {
            log.error("Failed to get document : " + e, e);
            throw new RaspLibraryException(e.getMessage());
        }

        // 4. Return
        return response;
    }

    /**
     * Gets the credentials 
     * @return the credentials
     */
    public Credentials getCredentials() {
        return credentials;
    }

    /**
     * Sets the credentials
     * @param value
     */
    public void setCredentials(Credentials value) {
        credentials = value;
    }

    /**
     * Gets the request URI
     * @return the request URI
     */
    public Uri getRequestUri() {
        return requestUri;
    }

    /**
     * Sets the request URI
     * @param u
     */
    public void setRequestUri(Uri u) {
        requestUri = u;
    }

    /**
     * Gets the send policy
     * @return the send policy
     */
    public SendPolicy getPolicy() {
        return policy;
    }

    /**
     * Sets the send policy
     * @param value
     */
    public void setPolicy(SendPolicy value) {
        policy = value;
        axisConfigurationFile = value.getAxisConfigurationFile();
        axisRepositoryDir = value.getAxisRepositoryDir();
    }
}
