/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and 
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication.client;

import java.security.cert.X509Certificate;
import java.util.Hashtable;

import org.apache.axiom.om.OMElement;
import org.apache.axis2.context.ConfigurationContext;
//import org.apache.sandesha2.policy.SandeshaPolicyBean;

import dk.gov.oiosi.communication.RMPolicy;
import dk.gov.oiosi.security.SignatureValidationProof;

/**
 * Interface and Factory for Axis low-level dispatchers.
 */
public interface IAxisDispatcher {

    /**
     * Factory for constucting low level dispatchers.
     */
    public static class Factory {

        // private static Hashtable dispatchers = new Hashtable();

        /**
         * Create dispatcher.
         * 
         * @param axisXml
         *            axis configuration file
         * @param axisRepository
         *            axis repository folder
         * @param wss
         *            use wss
         * @param rm
         *            use rm
         * @param wssPolicy
         *            wss policy file
         * @param httpPolicy
         *            rm http policy
         * @param mailPolicy
         *            rm mail policy
         * @param proxyPort
         *            proxy port
         * @param proxyHost
         *            proxy host
         * @return dispatcher
         * @throws ProxyGenerationException
         */
        public static IAxisDispatcher getDispatcher(String axisXml, String axisRepository, boolean wss, boolean rm,
                String wssPolicy, RMPolicy httpPolicy, RMPolicy mailPolicy, int proxyPort, String proxyHost)
                throws ProxyGenerationException {
            synchronized (Factory.class) {
                try {
                    IAxisDispatcher dispatcher = new AxisMercuryDispatcher(axisXml, axisRepository, wssPolicy);
                    dispatcher.setUseWSS(wss);
                    dispatcher.setUseRM(rm);

                    if (httpPolicy != null) {
                        dispatcher.setHttpRMPolicy(httpPolicy);
                    }
                    if (mailPolicy != null) {
                        dispatcher.setMailRMPolicy(mailPolicy);
                    }
                    if (proxyPort > 0) {
                        dispatcher.setProxy(proxyHost, proxyPort);
                    }
                    return dispatcher;
                } catch (Exception e) {
                    throw new ProxyGenerationException(e);
                }
                
                //TODO: Remove?
                /*
                 * String key = axisXml + axisRepository + wss + rm + axisPolicy +
                 * proxyPort + proxyHost; IAxisDispatcher dispatcher =
                 * (IAxisDispatcher) dispatchers.get( key ); if( dispatcher ==
                 * null ) { dispatcher = new Axis13Dispatcher(axisXml,
                 * axisRepository ); // dispatcher = new
                 * Axis12Dispatcher(axisXml, axisRepository ); // dispatcher =
                 * new TestRMClient(axisXml, axisRepository );
                 * 
                 * dispatcher.setUseWSS( wss ); dispatcher.setUseRM( rm );
                 * if(axisPolicy != null ) {
                 * dispatcher.setPolicyFile(axisPolicy); } if( proxyPort > 0 ) {
                 * dispatcher.setProxy( proxyHost, proxyPort); }
                 * dispatchers.put( key, dispatcher ); return dispatcher; } else {
                 * return dispatcher; }
                 */
            }
        }

        public static void registerConfigurationContext(String axisXml, String repository, ConfigurationContext configurationContext) {
           
            AxisMercuryDispatcher.registerConfigurationContext(axisXml, repository, configurationContext);
            
        }

        //TODO: Remove
        // private void deregisterDispatcher(String key) {
        // synchronized (Factory.class) {
        // dispatchers.remove(key);
        // }
        // }
    }

    /**
     * Method for dispatching message.
     * @deprecated Use sendRaspRequest() with signature validation proof instead
     */
    public OMElement sendRaspRequest(OMElement doc, String endpointAddress, String action, X509Certificate cert,
            Hashtable customHeaders, Hashtable properties) throws Exception;

    
    /**
     * Method for dispatching message.
     * @param doc
     *            the document to send
     * @param endpointAddress
     *            endpoint uri
     * @param action
     *            request action
     * @param cert
     *            receiver certificate
     * @param customHeaders
     *            custom headers
     * @return response message
     * @throws Exception
     */
    public OMElement sendRaspRequest(OMElement doc, String endpointAddress, String action, X509Certificate cert,
            Hashtable customHeaders, Hashtable properties, SignatureValidationProof[] signatureValidationProof) throws Exception;

    
//    public void setRMMailPolicy(SandeshaPolicyBean mailPolicy);
//
//    public void setRMHttpPolicy(SandeshaPolicyBean httpPolicy);

    public void setMailRMPolicy(RMPolicy mailPolicy);

    public void setHttpRMPolicy(RMPolicy httpPolicy);

    /**
     * Use WSS.
     * 
     * @param b
     *            use wss
     */
    public void setUseWSS(boolean b);

    /**
     * Use RM.
     * 
     * @param b
     */
    public void setUseRM(boolean b);

    /**
     * Use Proxy.
     * 
     * @param proxyHost
     *            host
     * @param proxyPort
     *            port
     */
    public void setProxy(String proxyHost, int proxyPort);

    /**TODO: Remove?
     * WSS policy file.
     * 
     * @param policy
     *            wss policy
     */
    //public void setPolicyFile(String policy);

    /**
     * clean up dispatcher. 
     */
    public void cleanUp();


}
