/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.communication.configuration;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.axiom.om.OMElement;

import dk.gov.oiosi.common.Guid;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlDocumentException;
import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminator;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfigCollection;

/**
 * A RASP UBL Document type (such as an Invoice or a Notification).
 */
public class DocumentTypeConfig {

    /**
     * Name of document.
     */
    private String friendlyName = "";

    /**
     * Namespace.
     */
    private String rootNamespace = "";

    /**
     * Rootname.
     */
    private String rootName = "";

    /**
     * serviceContractTModel.
     */
    private String serviceContractTModel = "";

    /**
     * schemaPath.
     */
    private String schemaPath = "";

    /**
     * stylesheetPath.
     */
    private String stylesheetPath = "";

    /**
     * xsltTransformStylesheetPath.
     */
    private String xsltTransformStylesheetPath = "";

    /**
     * id.
     */
    private Guid id = Guid.newGuid();

    /**
     * SchematronValidationConfig.
     */
    private SchematronValidationConfig schematronValidationConfig = new SchematronValidationConfig();

    /**
     * List of PrefixedNamespace.
     */
    private List namespaces = new ArrayList();

    /**
     * DocumentEndpointInformation.
     */
    private DocumentEndpointInformation endpointType = new DocumentEndpointInformation();

    /**
     * XPathDiscriminatorConfigCollection.
     */
    private XPathDiscriminatorConfigCollection identifierDiscriminators = new XPathDiscriminatorConfigCollection();

    /**
     * CustomHeaderConfiguration.
     */
    private CustomHeaderConfiguration customHeaderConfiguration = new CustomHeaderConfiguration();

    /**
     * Constructor.
     */
    public DocumentTypeConfig() {
    }

    /**
     * Constructor that only takes the parameters to uniquely identify the
     * document type.
     * 
     * @param rootName rootname of document
     * @param rootNamespace document namespace
     * @param identifierDiscriminators identifierDiscriminators
     */
    public DocumentTypeConfig(String rootName, String rootNamespace, XPathDiscriminatorConfigCollection identifierDiscriminators) {
        if (rootName == null)
            throw new NullArgumentException("rootName");
        if (rootNamespace == null)
            throw new NullArgumentException("rootNamespace");
        if (identifierDiscriminators == null)
            throw new NullArgumentException("identifierDiscriminators");
        this.rootName = rootName;
        this.rootNamespace = rootNamespace;
        this.identifierDiscriminators = identifierDiscriminators;
    }

    /**
     * Constructor
     * 
     * @param friendlyName
     *            Friendly name of the document
     * @param rootName
     *            The document element name
     * @param rootNamespace
     *            The document element namespace
     * @param schemaPath
     *            Path to the schema
     * @param stylesheetPath
     *            Path to the presentation stylesheet
     * @param serviceContractTModel
     *            The tModel ID of the service contract
     * @param xsltTransformStylesheetPath
     *            The path to the transformation stylesheet
     * @param endpointType
     *            The endpoint type
     * @param identifierDiscriminators
     *            Identifier discriminators
     * @param schematronValidationConfig
     *            Settings to the schematron validation
     */
    public DocumentTypeConfig(String friendlyName, String rootName, String rootNamespace, String schemaPath, String stylesheetPath,
            String serviceContractTModel, String xsltTransformStylesheetPath, DocumentEndpointInformation endpointType,
            XPathDiscriminatorConfigCollection identifierDiscriminators, SchematronValidationConfig schematronValidationConfig)
    {
        this(rootName, rootNamespace, identifierDiscriminators);

        if (friendlyName == null)
            throw new NullArgumentException("friendlyName");
        if (schemaPath == null)
            throw new NullArgumentException("schemaPath");
        if (stylesheetPath == null)
            throw new NullArgumentException("stylesheetPath");
        if (serviceContractTModel == null)
            throw new NullArgumentException("serviceContractTModel");
        if (xsltTransformStylesheetPath == null)
            throw new NullArgumentException("xsltTransformStylesheetPath");
        if (endpointType == null)
            throw new NullArgumentException("endpointType");
        if (schematronValidationConfig == null)
            throw new NullArgumentException("schematronValidationConfig");
        this.friendlyName = friendlyName;
        this.schemaPath = schemaPath;
        this.stylesheetPath = stylesheetPath;
        this.serviceContractTModel = serviceContractTModel;
        this.xsltTransformStylesheetPath = xsltTransformStylesheetPath;
        this.endpointType = endpointType;
        this.schematronValidationConfig = schematronValidationConfig;
    }

    /**
     * Constructor
     * 
     * @param friendlyName
     *            Friendly name of the document type e.g. for use in the UI
     * @param rootName
     *            The name of the document root element
     * @param rootNamespace
     *            The namespace of the document root element
     * @param schemaPath
     *            Path to the schema used for validating this document type
     * @param stylesheetPath
     *            Path to the stylesheet used for displaying documents of this
     *            type
     * @param serviceContractTModel
     *            The Service contract TModel, used for UDDI lookups
     * @param xsltTransformStylesheetPath
     *            Pthe to the xslt stylesheet
     * @param namespaces
     *            Namespaces used by the document.
     * @param endpointType
     *            Definition of a RASP endpoint type
     * @param identifierDiscriminators
     *            XPath discriminators used for aditionel identification of
     *            document type
     * @param schematronValidationConfig
     *            Settings to the schematron validation
     */
    public DocumentTypeConfig(String friendlyName, String rootName, String rootNamespace, String schemaPath, String stylesheetPath,
            String serviceContractTModel, String xsltTransformStylesheetPath,
            List namespaces, DocumentEndpointInformation endpointType, XPathDiscriminatorConfigCollection identifierDiscriminators,
            SchematronValidationConfig schematronValidationConfig) {
        this(friendlyName, rootName, rootNamespace, schemaPath, stylesheetPath, serviceContractTModel, xsltTransformStylesheetPath,
                endpointType, identifierDiscriminators, schematronValidationConfig);
        if (namespaces == null)
            throw new NullArgumentException("namespaces");
        this.namespaces = namespaces;
    }

    /**
     * Gets the ID of the document type.'
     * @return Guid id
     */
    public Guid getId() {
        return id;
    }

    /**
     * Gets the ID of the document type.
     */    
    public void setId(Guid value) {
        id = value;
    }

    /**
     * Gets the friendly name of the document type, e.g. for use in the UI.
     * @return FriendlyName
     */
    public String getFriendlyName() {
        return friendlyName;
    }

    /**
     * Sets the friendly name of the document type, e.g. for use in the UI.
     * @param value FriendlyName
     */
    public void setFriendlyName(String value) {
        if (value == null)
            throw new NullArgumentException("value");
        friendlyName = value;
    }

    /**
     * Gets the root namespace of the root node of the document.
     * @return RootNamespace
     */
    public String getRootNamespace() {
        return rootNamespace;
    }

    /**
     * Sets the root namespace of the root node of the document.
     * @param value RootNamespace
     */
    public void setRootNamespace(String value) {
        if (value == null)
            throw new NullArgumentException("value");

        rootNamespace = value;
    }

    /**
     * Gets the name of the document type.
     * @return RootName
     */
    public String getRootName() {
        return rootName;
    }

    /**
     * Sets the name of the document type.
     * @param value RootName
     */
    public void setRootName(String value) {
        if (value == null)
            throw new NullArgumentException("value");

        rootName = value;
    }

    /**
     * Gets the Service contract TModel, used for UDDI lookups.
     * @return ServiceContractTModel
     */
    public String getServiceContractTModel() {
        return serviceContractTModel;
    }

    /**
     * Sets the service contract TModel, used for UDDI lookups.
     * @param value ServiceContractTModel
     */
    public void setServiceContractTModel(String value) {
        if (value == null)
            throw new NullArgumentException("value");

        serviceContractTModel = value;
    }

    /**
     * Gets the path to a XML schema file, defining the document type.
     * @return SchemaPath
     */
    public String getSchemaPath() {
        return schemaPath;
    }

    /**
     * Sets the path to a XML schema file, defining the document type.
     * @param value SchemaPath
     */
    public void setSchemaPath(String value) {
        if (value == null)
            throw new NullArgumentException("value");

        schemaPath = value;
    }

    /**
     * Gets the path to a stylesheet, defining how documents of this type should be
     * presented.
     * @return StylesheetPath
     */
    public String getStylesheetPath() {
        return stylesheetPath;
    }

    /**
     * Sets the path to a stylesheet, defining how documents of this type should be
     * presented.
     * @param value StylesheetPath
     */
    public void setStylesheetPath(String value) {
        if (value == null)
            throw new NullArgumentException("value");

        stylesheetPath = value;
    }

    /**
     * Gets the list of namespaces used by this document type.
     * @return Namespaces
     */
    public PrefixedNamespace[] getNamespaces() {
        return (PrefixedNamespace[]) namespaces.toArray(new PrefixedNamespace[namespaces.size()]);
    }

    /**
     * Sets the list of namespaces used by this document type.
     * @param value Namespaces
     */
    public void setNamespaces(PrefixedNamespace[] value) {
        if (value == null)
            throw new NullArgumentException("value");

        // _namespaces = new ArrayList(value);
        namespaces.clear();
        for (int i = 0; i < value.length; i++) {
            namespaces.add(value[i]);
        }
    }

    /**
     * Sets the list of namespaces used by this document type.
     * @param value Namespaces
     */
    public void setNamespaces(Iterator value) {
        if (value == null)
            throw new NullArgumentException("value");

        // _namespaces = new ArrayList(value);
        namespaces.clear();
        while (value.hasNext()) {
            namespaces.add((PrefixedNamespace) value.next());
        }
    }

    /**
     * Gets the definition of what type of endpoints will receive documents of this
     * type
     * @return EndpointType
     */
    public DocumentEndpointInformation getEndpointType() {
        return endpointType;
    }

    /**
     * Sets a definition of what type of endpoints will receive documents of this
     * type
     * @param value EndpointType
     */
    public void setEndpointType(DocumentEndpointInformation value) {
        if (value == null)
            throw new NullArgumentException("value");

        endpointType = value;
    }

    /**
     * Gets the path to the XSLT transform stylesheet.
     * @return XsltTransformStylesheetPath
     */
    public String getXsltTransformStylesheetPath() {
        return xsltTransformStylesheetPath;
    }

    /**
     * Sets the path to the XSLT transform stylesheet.
     * @param value XsltTransformStylesheetPath
     */
    public void setXsltTransformStylesheetPath(String value) {
        if (value == null)
            throw new NullArgumentException("value");

        xsltTransformStylesheetPath = value;
    }

    /**
     * Gets the IdentifierDiscriminators.
     * @return IdentifierDiscriminators
     */
    public XPathDiscriminatorConfigCollection getIdentifierDiscriminators() {
        return identifierDiscriminators;
    }

    /**
     * Sets the IdentifierDiscriminators.
     * @param value IdentifierDiscriminators
     */
    public void setIdentifierDiscriminators(XPathDiscriminatorConfigCollection value) {
        if (value == null)
            throw new NullArgumentException("value");

        identifierDiscriminators = value;
    }

    /**
     * Gets the schematron validation configuration.
     * @return SchematronValidationConfig
     */
    public SchematronValidationConfig getSchematronValidationConfig() {
        return schematronValidationConfig;
    }

    /**
     * Sets the schematron validation configuration.
     * @param value SchematronValidationConfig
     */
    public void setSchematronValidationConfig(SchematronValidationConfig value) {
        if (value == null)
            throw new NullArgumentException("value");

        schematronValidationConfig = value;
    }

    /**
     * Gets the configuration section for custom headers.
     * @return CustomHeaderConfiguration
     */
    public CustomHeaderConfiguration getCustomHeaderConfiguration() {
        return customHeaderConfiguration;
    }

    /**
     * Sets the configuration section for custom headers.
     * @param value CustomHeaderConfiguration
     */
    public void setCustomHeaderConfiguration(CustomHeaderConfiguration value) {
        customHeaderConfiguration = value;
    }

    /**
     * Adds a namespace
     * 
     * @param ns
     *            Namespace
     * @param prefix
     *            Prefix
     */
    public void addNamespace(String ns, String prefix) {
        namespaces.add(new PrefixedNamespace(ns, prefix));
    }

    /**
     * Adds a namespace
     * 
     * @param ns
     *            A namespace with a prefix
     */
    public void addNamespace(PrefixedNamespace ns) {
        namespaces.add(ns);
    }

    /**
     * Compares RaspDocumentTypeConfig elements on the given xml document
     * 
     * @param document
     *            The document to check for
     * @return Returns true if name and namespace of the root element matches
     */
    public boolean isDocumentOfType(XmlDocument document) throws XmlDocumentException, XmlException {
        if (document == null)
            throw new NullArgumentException("document");
        OMElement root = document.getOMDocument();

        if (!root.getLocalName().equals(getRootName()))
            return false;
        if (!root.getDefaultNamespace().getNamespaceURI().equals(getRootNamespace()))
            return false;

        boolean discriminateCollectionAnded = XPathDiscriminator.discriminateCollectionAnded(identifierDiscriminators, document,
                getNamespaces());
        return discriminateCollectionAnded;
    }

    /**
     * Return whether the given document type configuration is equal to another. They
     * are equal if they have the same id or if they have the same root name,
     * root namespace and identifier discriminators.
     * 
     * @param object the other object to compare with.
     * @return whether the two document types are equals.
     */
    public boolean equals(Object object) {
        if( object == null) {
            return false;
        }
        if( ! (object instanceof DocumentTypeConfig)) {
            return false;
        }        
        DocumentTypeConfig other = (DocumentTypeConfig) object;
        if (id.equals( other.id) ) {
            return true;
        }
        if (!rootName.equals(other.rootName)) {
            return false;
        }
        if (!rootNamespace.equals(other.rootNamespace)) {
            return false;
        }
        return identifierDiscriminators.equals(other.identifierDiscriminators);
    }
}
