/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.configuration;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
import org.apache.commons.configuration.tree.xpath.XPathExpressionEngine;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.MailAddress;
import dk.gov.oiosi.communication.RMPolicy;
import dk.gov.oiosi.communication.SendPolicy;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.communication.configuration.CustomHeaderConfiguration;
import dk.gov.oiosi.communication.configuration.CustomHeaderXPathConfiguration;
import dk.gov.oiosi.communication.configuration.KeyTypeMapping;
import dk.gov.oiosi.communication.configuration.KeyTypeMappingExpression;
import dk.gov.oiosi.communication.configuration.DocumentEndpointInformation;
import dk.gov.oiosi.communication.configuration.DocumentTypeCollectionConfig;
import dk.gov.oiosi.communication.configuration.DocumentTypeConfig;
import dk.gov.oiosi.communication.configuration.ServiceEndpointFriendlyName;
import dk.gov.oiosi.communication.configuration.ServiceEndpointKey;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.ldap.LdapLookupFactoryConfig;
import dk.gov.oiosi.security.ldap.LdapSettings;
import dk.gov.oiosi.security.ocsp.OcspConfig;
import dk.gov.oiosi.security.ocsp.OcspLookupFactoryConfig;
import dk.gov.oiosi.uddi.LookupReturnOption;
import dk.gov.oiosi.uddi.UddiConfig;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupClientFactoryConfig;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.ranges.GatewayRange;
import dk.gov.oiosi.uddi.ranges.GatewayRegistrationParameters;
import dk.gov.oiosi.xml.schematron.SchematronValidationConfig;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfig;
import dk.gov.oiosi.xml.xpath.discriminator.XPathDiscriminatorConfigCollection;

/**
 * ReadOnly version of RaspConfigurationHandler.
 * 
 * Reads RaspConfiguration.xml - and initializes configuration classes.
 */
public class RaspConfigurationHandler {

    /**
     * Local log.
     */
    private Log log = LogFactory.getLog(RaspConfigurationHandler.class);

    /**
     * Default file name.
     */
    public static final String RASPCONFIGURATION_XML_DEFAULT = "RaspConfiguration.xml";

    /**
     * Use this file.
     */
    private static String raspConfigurationFileName = RASPCONFIGURATION_XML_DEFAULT;

    /**
     * RaspConfigurationHandler instance.
     */
    private static RaspConfigurationHandler rcf = null;

    /**
     * content of RaspConfiguraiton.xml.
     */
    private XMLConfiguration raspConfigurationFile = null;

    /**
     * LDAP settings.
     */
    private LdapSettings ldapSettings = null;

    /**
     * OCSP settings.
     */
    private OcspConfig ocspConfig = null;

    /**
     * UDDI settings.
     */
    private UddiConfig uddiConfig = null;

    /**
     * LDAP Factory.
     */
    private LdapLookupFactoryConfig ldapFactory = null;

    /**
     * OCSP settings.
     */
    private OcspLookupFactoryConfig ocspFactory = null;

    /**
     * UDDI settings.
     */
    private UddiLookupClientFactoryConfig uddiClientFactory = null;

    /**
     * Document configurations.
     */
    private DocumentTypeCollectionConfig raspDocumentTypeCollectionConfig;

    /**
     * SendPolicy.
     */
    private SendPolicy sendPolicy = null;
    
    /**
     * Factory/Singleton method.
     * 
     * @return RaspConfigurationHandler
     */
    public synchronized static RaspConfigurationHandler getInstance() {
        if (rcf == null) {
            rcf = new RaspConfigurationHandler();
            rcf.loadXMLConfiguration();
        }
        return rcf;
    }

    /**
     * Bypass configuration file name.
     * 
     * @param file
     *            file name
     */
    public static void setConfigurationFile(String file) {
        raspConfigurationFileName = file;
    }

    /**
     * read configuration from file.
     */
    public synchronized void loadXMLConfiguration() {
        try {
            log.debug("Looking for configurationfile " + raspConfigurationFileName);
            try {
                raspConfigurationFile = new XMLConfiguration(raspConfigurationFileName);
            } catch (Exception e) {
                raspConfigurationFile = new XMLConfiguration("conf/" + raspConfigurationFileName);
            }
            raspConfigurationFile.setReloadingStrategy(new FileChangedReloadingStrategy());
        	raspConfigurationFile.setExpressionEngine(new XPathExpressionEngine());
        } catch (Exception e) {
            log.fatal(raspConfigurationFileName + " can not be used due to: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * @deprecated
     * LDAP factory configuration
     * 
     * @return LDAP factory configuration
     */
    public LdapSettings getLDAPSettings() {
        return getLdapSettings();
    }

    /**
     * Get the LDAP Settings.
     * 
     * @return LDAP settings
     */
    public LdapSettings getLdapSettings() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"LdapSettings\"]/";
    	
        log.debug("RaspConfigurationHandler . getLDAPSettings " + raspConfigurationFile);
        synchronized (raspConfigurationFile) {
            if (ldapSettings == null) {

                ldapSettings = new LdapSettings();

                ldapSettings.setHost(raspConfigurationFile.getString(xpathBase + "Host"));
                ldapSettings.setMaxResults(raspConfigurationFile.getInt(xpathBase + "MaxResults"));
                ldapSettings.setPort(raspConfigurationFile.getInt(xpathBase + "Port"));
                ldapSettings.setSearchServerTimeoutMsec(raspConfigurationFile
                        .getInt(xpathBase + "SearchServerTimeoutMsec"));
                ldapSettings.setSearchClientTimeoutMsec(raspConfigurationFile.getInt(xpathBase + "ConnectionTimeoutMsec"));
            }
        }
        return ldapSettings;
    }

    /**
     * Get the OCSP settings.
     * 
     * @return OCSP settings
     */
    public OcspConfig getOcspConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"OcspConfig\"]/";
    	
        log.debug("RaspConfigurationHandler . getOcspConfig ");
        synchronized (raspConfigurationFile) {
            if (ocspConfig == null) {
                ocspConfig = new OcspConfig();
                ocspConfig.setDefaultTimeoutMsec(raspConfigurationFile.getInt(xpathBase + "DefaultTimeoutMsec"));
                ocspConfig.setServerUrl(raspConfigurationFile.getString(xpathBase + "ServerUrl"));
            }
        }
        return ocspConfig;
    }
    
    /**
     * Ldap stub settings.
     * 
     * @return ldap stub settings
     */
    public String getLdapTestConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"LdapCertificateLookupTestConfig\"]/";
    	return raspConfigurationFile.getString(xpathBase + "FileName");
    }
    
    /**
     * OCSP stub settings.
     * 
     * @return ocsp stub settings
     */
    public boolean getOcspTestConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"OcspLookupTestConfig\"]/";

    	boolean response = false;
        synchronized (raspConfigurationFile) {
        	response = raspConfigurationFile.getBoolean(xpathBase + "ReturnPositiveResponse");
        }
        return response;
    }
    
    /**
     * Get the UDDI settings.
     * 
     * @return UDDI settings
     */
    public UddiConfig getUddiConfig() {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"UddiConfig\"]/";
    	
    	log.debug("RaspConfigurationHandler . getUddiConfig ");
        synchronized (raspConfigurationFile) {
            uddiConfig = new UddiConfig();

            uddiConfig.setProfileConformanceClaim(raspConfigurationFile.getString(xpathBase + "ProfileConformanceClaim"));
            uddiConfig.setRegistrationConformanceClaim(raspConfigurationFile
                    .getString(xpathBase + "RegistrationConformanceClaim"));
            
            uddiConfig.setTryOtherHostsOnFailure(raspConfigurationFile.getBoolean(xpathBase + "TryOtherHostsOnFailure",
                    false));
            uddiConfig.setUddiInquireEndpointURL(raspConfigurationFile.getString(xpathBase + "UddiInquireEndpointURL"));
            uddiConfig.setUddiInquireEndpointURLFallback(raspConfigurationFile.getString(xpathBase + "UddiInquireEndpointURLFallback"));

            uddiConfig.setUddiPublishEndpointURL(raspConfigurationFile.getString(xpathBase + "UddiPublishEndpointURL"));
            uddiConfig.setUddiSecurityEndpointURL(raspConfigurationFile.getString(xpathBase + "UddiSecurityEndpointURL"));

            uddiConfig.setFallbackTimeoutMinutes(raspConfigurationFile.getInt(xpathBase + "FallbackTimeoutMinutes"));

            uddiConfig.setLookupReturnOptions(new LookupReturnOption(raspConfigurationFile.getString(xpathBase + "LookupReturnOptions")));

            // Gateway Range
            String gatewayKey = raspConfigurationFile
                    .getString(xpathBase + "GatewayRange/GatewayRegistrationParameters/GatewayRegistrationKeyEan");
            GatewayRegistrationParameters gw = new GatewayRegistrationParameters();
            if (gatewayKey != null) {
                gw.setGatewayRegistrationKeyEan(gatewayKey);

                // uc.setGatewayRegistrationKeyEAN( gatewayKey );
            } else {
                gatewayKey = raspConfigurationFile
                        .getString(xpathBase + "GatewayRange/GatewayRegistrationParameters/GatewayRegistrationKeyCVR");
                if (gatewayKey != null) {
                    // uc.setGatewayRegistrationKeyCVR( gatewayKey );
                } else {
                    gatewayKey = raspConfigurationFile
                            .getString(xpathBase + "GatewayRange/GatewayRegistrationParameters/GatewayRegistrationKeyOVT");
                }
                if (gatewayKey != null) {
                    // uc.setGatewayRegistrationKeyOVT( gatewayKey );
                }
            }
            if (gatewayKey != null) {
                GatewayRange range = new GatewayRange();
                range.setRangeStart(raspConfigurationFile.getString(xpathBase + "GatewayRange/RangeStart"));
                range.setRangeEnd(raspConfigurationFile.getString(xpathBase + "GatewayRange/RangeEnd"));
                range.setGatewayRegistrationParameters(gw);

                uddiConfig.setGatewayRange(range);
            }
        }
        return uddiConfig;
    }
    
    /**
     * Gets the document configurations configuration.
     * 
     * @return document configurations
     * @throws ConfigurationException
     */
    public DocumentTypeCollectionConfig getRaspDocumentTypeCollectionConfig() throws ConfigurationException {
    	log.debug("RaspConfigurationHandler . getRaspDocumentTypeCollectionConfig ");
        synchronized (raspConfigurationFile) {
            if (raspDocumentTypeCollectionConfig == null) {
                try {
                    raspDocumentTypeCollectionConfig = new DocumentTypeCollectionConfig();

                    String[] docs = raspConfigurationFile
                            .getStringArray("//DocumentTypes/RaspDocumentTypeConfig/RootName");
                    for (int i = 0; i < docs.length; i++) {
                        DocumentTypeConfig typeConfig = initRaspDocumentTypeConfig(i);
                        raspDocumentTypeCollectionConfig.addDocumentType(typeConfig);
                    }
                } catch (Exception e) {
                    ConfigurationException ex = new ConfigurationException("Could not init DocumnetTypeConfig " + e);
                    ex.setStackTrace(e.getStackTrace());
                    throw ex;
                }
            }
        }
        return raspDocumentTypeCollectionConfig;
    }

    /**
     * Gets the send policy.
     * 
     * @return send policy
     * @throws ConfigurationException
     */
    public SendPolicy getSendPolicy() throws ConfigurationException {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"Axis2JavaRasp\"]/";
    	
    	log.debug("RaspConfigurationHandler . getSendPolicy");
        synchronized (raspConfigurationFile) {
            if (sendPolicy == null) {
                sendPolicy = new SendPolicy();

                String wssPolicy = raspConfigurationFile.getString(xpathBase + "WSSPolicyFile");
                log.debug("- wss policy file : " + wssPolicy);
                sendPolicy.setWSSPolicyFileName(wssPolicy);

                String axisConf = raspConfigurationFile.getString(xpathBase + "AxisConfigurationFile");
                log.debug("- axis configuration : " + axisConf);
                sendPolicy.setAxisConfigurationFile(axisConf);

                String repoDir = raspConfigurationFile.getString(xpathBase + "AxisRepositoryDir");
                log.debug("- axis repositorty dir : " + repoDir);
                sendPolicy.setAxisRepositoryDir(repoDir);

                sendPolicy.setXmlSchematron(raspConfigurationFile.getBoolean(xpathBase + "XMLSchematron", false));
                log.debug("- client side schematron : " + sendPolicy.doXmlSchematron());
                
                sendPolicy.setXmlValidation(raspConfigurationFile.getBoolean(xpathBase + "XMLValidation", false));
                log.debug("- client side schema validation : " + sendPolicy.doXmlValidation());

                sendPolicy.setCustomHeaders(raspConfigurationFile.getBoolean(xpathBase + "CustomHeaders", false));
                log.debug("- client custom header check : " + sendPolicy.doCustomHeaders());
                
                String mailPrefix = xpathBase + "RMPolicy/MailTransport/RMAssertion/Policy/";
                RMPolicy mailPolicy = initRMPolicy(mailPrefix);
                sendPolicy.setMailRMPolicy(mailPolicy);
                
                String httpPrefix = xpathBase + "RMPolicy/HTTPTransport/RMAssertion/Policy/";
                RMPolicy httpPolicy = initRMPolicy(httpPrefix);
                sendPolicy.setHttpRMPolicy(httpPolicy);
            }
            return sendPolicy;
        }
    }

    /**
     * Gets the UDDI factory configuration.
     * 
     * @return UDDI factory configuration
     */
    public UddiLookupClientFactoryConfig getUddiLookupClientFactoryConfig() {
        log.debug("RaspConfigurationHandler . getUddiLookupClientFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(uddiClientFactory==null) {
                initFactories();
            }
        }
        return uddiClientFactory;
    }

    /**
     * Gets the UDDI client test configuration
     * @return UDDI client test configuration
     * @throws ParseException
     */
    public UddiLookupResponse getUddiClientTestConfig() throws ParseException {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"UddiLookupClientTestConfig\"]/";
    	
    	synchronized(raspConfigurationFile){
	    	 	    	
	    	// Reads the configuration elements from file
	    	IIdentifier endpointKey= null;
	    	EndpointAddress endpointAddress= IdentifierUtility.getEndpointAddressFromString(raspConfigurationFile.getString(xpathBase + "EndpointAddress"));
	    	Date activationUTC= DateFormat.getDateInstance().parse(raspConfigurationFile.getString(xpathBase + "ActivationDate"));
	    	Date expirationUTC= DateFormat.getDateInstance().parse(raspConfigurationFile.getString(xpathBase + "ExpirationDate"));
	    	String cert= raspConfigurationFile.getString(xpathBase + "CertificateSubjectSerialNumber");
	    	Uri termsOfUse= new Uri(raspConfigurationFile.getString(xpathBase + "TermsOfUseUrl"));
	    	MailAddress contactMail= new MailAddress(raspConfigurationFile.getString(xpathBase + "ServiceContactEmail"));
	    	Version version= new Version(raspConfigurationFile.getString(xpathBase + "Version"));
	    	UddiId newerVersion= null;
	    	if(raspConfigurationFile.getBoolean(xpathBase + "HasNewerVersion")){
	    		newerVersion = IdentifierUtility.getUddiIDFromString(raspConfigurationFile.getString(xpathBase + "NewerVersionReference"));
	    	}
	    	
	    	// Returns the UddiLookupResponse
			return new UddiLookupResponse(endpointKey, endpointAddress, activationUTC, expirationUTC,
	                new CertificateSubject(cert), termsOfUse, contactMail, version, newerVersion);
    	}
    }

    /**
     * Gets the LDAP factory configuration
     * 
     * @return LDAP factory configuration
     */
    public LdapLookupFactoryConfig getLdapLookupFactoryConfig() {
        log.debug("RaspConfigurationHandler . getLdapLookupFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(ldapFactory==null) {
                initFactories();
            }
        }
        return ldapFactory;
    }

    /**
     * Gets the OCSP lookup client factory
     * 
     * @return OCSP lookup client factory
     */
    public OcspLookupFactoryConfig getOcspLookupFactoryConfig() {
        log.debug("RaspConfigurationHandler . getOcspLookupFactoryConfig ");
        synchronized (raspConfigurationFile) {
            if(ocspFactory==null) {
                initFactories();
            }
        }
        return ocspFactory;
    }
    
    /**
     * Initiate the reliable messaging configuration.
     * 
     * @param xpath
     *            where to find the configuration in the XML-document
     * @throws ConfigurationException
     */
    private RMPolicy initRMPolicy(String xpath) throws ConfigurationException {
        try {
            boolean eb = raspConfigurationFile.getBoolean(xpath + "ExponentialBackoff");
            long retr = raspConfigurationFile.getLong(xpath + "RetransmissionInterval");
            int max = raspConfigurationFile.getInt(xpath + "MaximumRetransmissionCount");

            RMPolicy policy = new RMPolicy(max, retr, eb);
            return policy;
        } catch (Exception e) {
            ConfigurationException ex = new ConfigurationException("Could not init SendPolicy " + e);
            ex.setStackTrace(e.getStackTrace());
            throw ex;
        }
    }

    /**
     * Read document.
     * 
     * @param docIndex
     *            ix in configuration file
     * @return configuration for document
     * @throws Exception
     */
    private DocumentTypeConfig initRaspDocumentTypeConfig(int docIndex) throws Exception {
    	// The XPath for our configuration section
    	String xpathBase = "//ConfigurationSection[@*=\"RaspDocumentTypeCollectionConfig\"]/";
    	
        DocumentTypeConfig rd = new DocumentTypeConfig();

        log.debug("Retrieving basic data...");

        String documentSelect = xpathBase + "DocumentTypes/RaspDocumentTypeConfig[" + (docIndex + 1) + "]";

        rd.setRootName(raspConfigurationFile.getString(documentSelect + "/RootName"));
        rd.setFriendlyName(raspConfigurationFile.getString(documentSelect + "/FriendlyName"));
        rd.setRootNamespace(raspConfigurationFile.getString(documentSelect + "/RootNamespace"));
        rd.setSchemaPath(raspConfigurationFile.getString(documentSelect + "/SchemaPath"));

        rd.setServiceContractTModel(raspConfigurationFile.getString(documentSelect + "/ServiceContractTModel"));
        rd.setStylesheetPath(raspConfigurationFile.getString(documentSelect + "/StylesheetPath"));

        String tmp = raspConfigurationFile.getString(documentSelect + "/XsltTransformStylesheetPath");
        if (tmp != null && tmp.length() > 0) {
            rd.setXsltTransformStylesheetPath(tmp);
        }

        //
        SchematronValidationConfig svc = new SchematronValidationConfig();
        svc.setSchematronDocumentPath(raspConfigurationFile.getString(documentSelect
                + "/SchematronValidationConfig/SchematronDocumentPath"));
        svc.setErrorXPath(raspConfigurationFile.getString(documentSelect + "/SchematronValidationConfig/ErrorXPath"));
        svc.setErrorMessageXPath(raspConfigurationFile.getString(documentSelect + "/SchematronValidationConfig/ErrorMessageXPath"));
        rd.setSchematronValidationConfig(svc);

        //
        DocumentEndpointInformation rdept = new DocumentEndpointInformation();
        rdept.setEndpointFriendlyName(new ServiceEndpointFriendlyName(raspConfigurationFile.getString(documentSelect
                + "/EndpointType/EndpointFriendlyName/XPath")));
        rdept.setSenderFriendlyName(new ServiceEndpointFriendlyName(raspConfigurationFile.getString(documentSelect
                + "/EndpointType/SenderFriendlyName/XPath")));

        rdept.setReplyAction(raspConfigurationFile.getString(documentSelect + "/EndpointType/ReplyAction"));
        rdept.setRequestAction(raspConfigurationFile.getString(documentSelect + "/EndpointType/RequestAction"));

        // Receiver Key
        ServiceEndpointKey receipientRdk = new ServiceEndpointKey();
        receipientRdk.setXPath(raspConfigurationFile.getString(documentSelect + "/EndpointType/Key/Xpath"));

        KeyTypeMappingExpression recipientKeyTypeMapping = getKeyTypeMappingExpression(documentSelect + "/EndpointType/Key");
        receipientRdk.addMappingExpression(recipientKeyTypeMapping);

        // Sender Key
        ServiceEndpointKey senderRdk = new ServiceEndpointKey();
        senderRdk.setXPath(raspConfigurationFile.getString(documentSelect + "/EndpointType/SenderKey/Xpath"));

        KeyTypeMappingExpression senderKeyTypeMapping = getKeyTypeMappingExpression(documentSelect + "/EndpointType/SenderKey");
        senderRdk.addMappingExpression(senderKeyTypeMapping);

        rdept.setKey(receipientRdk);
        rdept.setSenderKey(senderRdk);

        rd.setEndpointType(rdept);

        //
        int noOfEndNameSpaces = raspConfigurationFile.getList(documentSelect + "/Namespaces/PrefixedNamespace/Namespace").size();
        List rdns = new ArrayList();
        for (int x = 0; x < noOfEndNameSpaces; x++) {
            PrefixedNamespace rdn = new PrefixedNamespace();
            rdn.setPrefix(raspConfigurationFile.getString(documentSelect + "/Namespaces/PrefixedNamespace[" + (x+1) + "]/Prefix"));
            rdn.setNamespace(raspConfigurationFile.getString(documentSelect + "/Namespaces/PrefixedNamespace[" + (x+1) + "]/Namespace"));
            rdns.add(rdn);
        }
        rd.setNamespaces(rdns.iterator());

        // TODO IMPLEMENT CustomHeaders
        String csXpath = documentSelect + "/CustomHeaderConfiguration/XPaths";
        Iterator keys = raspConfigurationFile.getKeys(csXpath);
        if (keys.hasNext()) {
            CustomHeaderConfiguration chc = new CustomHeaderConfiguration();
            List headerList = new ArrayList();
            while (keys.hasNext()) {
                CustomHeaderXPathConfiguration chxc = new CustomHeaderXPathConfiguration();
                String key = keys.next().toString();
                String xpath = raspConfigurationFile.getString(key);
                String name = key.substring(csXpath.length() + 1);

                chxc.setName(name);
                chxc.setXPath(xpath);

                headerList.add(chxc);
            }
            chc.setXPaths((CustomHeaderXPathConfiguration[]) headerList.toArray(new CustomHeaderXPathConfiguration[headerList
                    .size()]));
            rd.setCustomHeaderConfiguration(chc);
        }

        // xpath discriminator collection
        String[] identifierList = raspConfigurationFile.getStringArray(documentSelect
                + "/IdentifierDiscriminators/XPathDiscriminatorConfig/XPathExpression");
        if (identifierList.length > 0) {
            XPathDiscriminatorConfigCollection xpathIdentifiers = new XPathDiscriminatorConfigCollection();
            for (int i = 0; i < identifierList.length; i++) {
                XPathDiscriminatorConfig identifier = new XPathDiscriminatorConfig();

                String expression = raspConfigurationFile.getString(documentSelect
                        + "/IdentifierDiscriminators/XPathDiscriminatorConfig[" + (i+1) + "]/XPathExpression");
                String expectedResult = raspConfigurationFile.getString(documentSelect
                        + "/IdentifierDiscriminators/XPathDiscriminatorConfig[" + (i+1) + "]/XPathExpectedResult");
                identifier.setXPathExpression(expression);
                identifier.setXPathExpectedResult(expectedResult);

                xpathIdentifiers.add(identifier);
            }
            rd.setIdentifierDiscriminators(xpathIdentifiers);
        }

        log.debug("Finished retrieving document");

        return rd;
    }

    /**
     * Utility to initiate key type for sender/receiver key (CVR, EAN...).
     * 
     * @param path
     *            path in document
     * @return mapping
     */
    private KeyTypeMappingExpression getKeyTypeMappingExpression(String path) {
        KeyTypeMappingExpression keyTypeMapping = new KeyTypeMappingExpression();
        String keyTypeMappingXPath = path + "/MappingExpressions/KeyTypeMappingExpressions/Name";

        String[] mappings = raspConfigurationFile.getStringArray(keyTypeMappingXPath);
        for (int i = 0; i < mappings.length; i++) {
            String mapExprKey = path + "/MappingExpressions/KeyTypeMappingExpressions[" + (i+1) + "]";
            String mapName = raspConfigurationFile.getString(mapExprKey + "/Name");
            String mapXPath = raspConfigurationFile.getString(mapExprKey + "/XPathExpression");
            keyTypeMapping.setName(mapName);
            keyTypeMapping.setXPathExpression(mapXPath);
            String[] mapList = raspConfigurationFile.getStringArray(mapExprKey + "/Mappings/Mapping/Value");
            for (int j = 0; j < mapList.length; j++) {
                String mapListKey = mapExprKey + "/Mappings/Mapping[" + (j+1) + "]";
                String value = raspConfigurationFile.getString(mapListKey + "/Value");
                String mapsTo = raspConfigurationFile.getString(mapListKey + "/MapsTo");

                KeyTypeMapping map = new KeyTypeMapping();
                map.setValue(value);
                map.setMapsTo(mapsTo);

                keyTypeMapping.addMapping(map);
            }
        }
        return keyTypeMapping;
    }

    private void initFactories() {
        uddiClientFactory = new UddiLookupClientFactoryConfig();
        uddiClientFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"UddiLookupClientFactoryConfig\"]/ImplementationNamespaceClass"));
        
        ldapFactory = new LdapLookupFactoryConfig();
        ldapFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"LdapLookupFactoryConfig\"]/ImplementationNamespaceClass"));
        
        ocspFactory = new OcspLookupFactoryConfig();
        ocspFactory.setImplementationNamespaceClass(raspConfigurationFile.getString("//ConfigurationSection[@*=\"OcspLookupFactoryConfig\"]/ImplementationNamespaceClass"));
        
    }

}
