/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security;

import java.security.cert.X509Certificate;
import java.util.Date;

import dk.gov.oiosi.security.oces.OcesCertificateType;
import dk.gov.oiosi.security.ocsp.CertificateCheckResult;
import dk.gov.oiosi.security.ocsp.CertificateRevokedTimeoutException;
import dk.gov.oiosi.security.ocsp.CheckCertificateActivatedUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckCertificateChainUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckCertificateOcspUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckCertificateRevokedUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckCertificateTypeUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckCertificateValidUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckRootCertificateActivatedUnexpectedException;
import dk.gov.oiosi.security.ocsp.CheckRootCertificateValidUnexpectedException;
import dk.gov.oiosi.security.ocsp.OcspConfig;
import dk.gov.oiosi.security.ocsp.OcspLookup;
import dk.gov.oiosi.security.ocsp.OcspResponse;
import dk.gov.oiosi.security.validation.CertificateExpiredException;
import dk.gov.oiosi.security.validation.CertificateNotActiveException;
import dk.gov.oiosi.security.validation.CertificateValidator;

/**
 * Methods for performing various certificate checks, such as revocation and
 * root certificate relations.
 */
public class CertificateChecker {

    /**
     * Oces Root.
     */
    private X509Certificate defaultOCESrootCertificate;

    /**
     * Ocsp Checker.
     */
    private OcspLookup ocsp;

    /**
     * Instantiates CertificateChecker.
     * 
     * @param ocspLookupConfiguration
     *            The ocsp server configuration
     * @param defaultRootCertificate
     *            default OCES root certificate
     * @throws Exception
     */
    public CertificateChecker(OcspConfig ocspLookupConfiguration, X509Certificate defaultRootCertificate) throws Exception {
        try {
            defaultOCESrootCertificate = defaultRootCertificate;

            // Initializes the component, that will do the actual ocsp lookup
            ocsp = new OcspLookup(ocspLookupConfiguration);
        } catch (Exception e) {
            throw e;
        }
    }

    /**
     * Checks a certificate, with default rootcertificate.
     * 
     * @param certificate
     *            certificate to check
     * @return The object that contains the result
     * @throws Exception
     */
    public CertificateCheckResult checkCertificate(X509Certificate certificate) throws Exception {
        CertificateCheckResult result = new CertificateCheckResult();
        OcspResponse response = new OcspResponse();

        try {
            // 1. that the certificate has the default rootcertificate as root
            checkCertificateChain(certificate, result);

            // 2. that the rootcertificate is not expired or not activated
            checkRootCertificateValidation(defaultOCESrootCertificate, result);
            checkRootCertificateActivated(defaultOCESrootCertificate, result);

            // 3. that the certificate is not expired or not activated
            if (checkCertificateValidation(certificate)) {
                result.setCertificateValid(true);
            }
            if (checkCertificateActivated(certificate)) {
                result.setCertificateActivated(true);
            }

            // 4. check if the certificate is revoked async
            response = checkCertificateRevocation(certificate, response);

            // 5. check certificatetype
            checkCertificateType(certificate, result);

            // 6. set the return values
            if (!response.isValid()) {
                result.setCertificateRevoked(true);
            }

            if (result.getCertificateActivated() && !result.getCertificateRevoked() && result.getCertificateValid()
                    && result.getRootCertificateActivated() && result.getRootCertificateAsRoot()
                    && result.getRootCertificateValid()) {
                result.setAllTestsPassed(true);
            }
        } catch (Exception e) {
            throw e;
        }
        return result;
    }

    /**
     * Checks a certificate, with a given rootcertificate.
     * 
     * @param certificate
     *            certificate to check
     * @param rootCertificate
     *            a given rootcertificate
     * @return The object that contains the result
     * @throws Exception
     */
    public CertificateCheckResult checkCertificate(X509Certificate certificate, X509Certificate rootCertificate) throws Exception {
        CertificateCheckResult result = new CertificateCheckResult();
        OcspResponse response = new OcspResponse();

        try {
            // 1. that the certificate has the default rootcertificate as root
            checkCertificateChain(certificate, result);

            // 2. that the rootcertificate is not expired or not activated
            checkRootCertificateValidation(rootCertificate, result);
            checkRootCertificateActivated(rootCertificate, result);

            // 3. that the certificate is not expired or not activated
            if (checkCertificateValidation(certificate)) {
                result.setCertificateValid(true);
            }
            if (checkCertificateActivated(certificate)) {
                result.setCertificateActivated(true);
            }

            // 4. check if the certificate is revoked async
            response = checkCertificateRevocation(certificate, response);

            // 5. set the return values
            if (!response.isValid()) {
                result.setCertificateRevoked(true);
            }

            if (result.getCertificateActivated() && !result.getCertificateRevoked() && result.getCertificateValid()
                    && result.getRootCertificateActivated() && result.getRootCertificateAsRoot()
                    && result.getRootCertificateValid()) {
                result.setAllTestsPassed(true);
            }
        } catch (Exception e) {
            throw e;
        }
        return result;
    }

    /**
     * Performs a certificate chain validation.
     * 
     * @param certificate
     *            the certificate to validate
     * @param result
     *            the object to store the result
     * @throws CheckCertificateChainUnexpectedException
     *             This exception is thrown, if an unexpected exception is
     *             thrown during the method
     */
    private void checkCertificateChain(X509Certificate certificate, CertificateCheckResult result)
            throws CheckCertificateChainUnexpectedException {
        try {
            CertificateValidator.validateCertificate(certificate);

            result.setRootCertificateAsRoot(true);
            result.setCertificateActivated(true);
            result.setCertificateValid(true);
            // } catch (CertificateFailedChainValidationException e) {
            // result.setRootCertificateAsRoot( false );
        } catch (CertificateNotActiveException e) {
            result.setCertificateActivated(false);
        } catch (CertificateExpiredException e) {
            result.setCertificateValid(false);
        } catch (Exception e) {
            throw new CheckCertificateChainUnexpectedException(e.getMessage());
        }
    }

    /**
     * Checks if the rootcertificate is activated.
     * 
     * @param root
     *            The rootcertificate to check
     * @param result
     *            The object to store the result of the check
     * @throws CheckRootCertificateActivatedUnexpectedException
     *             This exception is thrown, if an unexpected exception is
     *             thrown during the method
     */
    private void checkRootCertificateActivated(X509Certificate root, CertificateCheckResult result)
            throws CheckRootCertificateActivatedUnexpectedException {
        try {
            if (!root.getNotBefore().before(new Date())) {
                result.setRootCertificateActivated(true);
            }
        } catch (Exception e) {
            throw new CheckRootCertificateActivatedUnexpectedException(e);
        }
    }

    /**
     * Checks if the rootcertificate is valid.
     * 
     * @param root
     *            The rootcertificate to check
     * @param result
     *            The object to store the result of the check
     * @throws CheckRootCertificateValidUnexpectedException
     *             This exception is thrown, if an unexpected exception is
     *             thrown during the method
     */
    private void checkRootCertificateValidation(X509Certificate root, CertificateCheckResult result)
            throws CheckRootCertificateValidUnexpectedException {
        try {
            if (root.getNotAfter().after(new Date())) {
                result.setRootCertificateValid(true);
            }
        } catch (Exception e) {
            throw new CheckRootCertificateValidUnexpectedException(e);
        }
    }

    /**
     * Checks if the certificate is activated.
     * 
     * @param certificate
     *            The certificate to check
     * @return boolean stating whether the certificate is activated (true) or
     *         not (false)
     * @throws CheckCertificateActivatedUnexpectedException
     *             This exception is thrown, if an unexpected exception is
     *             thrown during the method
     */
    public boolean checkCertificateActivated(X509Certificate certificate) throws CheckCertificateActivatedUnexpectedException {
        boolean isActive = false;
        try {
            CertificateValidator.checkCertificateActivated(certificate);
            isActive = true;
        } catch (CertificateNotActiveException e) {
            isActive = false;
        } catch (Exception e) {
            throw new CheckCertificateActivatedUnexpectedException(e.getMessage());
        }
        return isActive;
    }

    /**
     * Checks if the certificate is valid.
     * 
     * @param certificate
     *            The certificate to check
     * @return boolean stating whether the certificate is valid (true) or not
     *         (false)
     * @throws CheckCertificateValidUnexpectedException
     *             This exception is thrown, if an unexpected exception is
     *             thrown during the method
     */
    public boolean checkCertificateValidation(X509Certificate certificate) throws CheckCertificateValidUnexpectedException {
        boolean isValid = false;
        try {
            CertificateValidator.checkCertificateExpired(certificate);
            isValid = true;
        } catch (CertificateExpiredException e) {
            isValid = false;
        } catch (Exception e) {
            throw new CheckCertificateValidUnexpectedException(e);
        }
        return isValid;
    }

    /**
     * Check if the certificate is revoced against ocsp server.
     * 
     * @param certificate
     *            the certificate to check
     * @param response
     *            OcspResponse object to store the result
     * @return OcspResponse object to store the result
     * @throws CheckCertificateOcspUnexpectedException
     *             This exception is thrown, if an unexpected exception is
     *             thrown during the method
     * @throws CertificateRevokedTimeoutException
     *             This exception is thrown, if the call to Ocsp server takes
     *             longer time than the allowed timeout
     * @throws CheckCertificateRevokedUnexpectedException
     *             Thrown if an unexpected error occured
     */
    private OcspResponse checkCertificateRevocation(X509Certificate certificate, OcspResponse response)
            throws CheckCertificateRevokedUnexpectedException {
        try {
            // Checks the certificate
            ocsp.checkCertificate(certificate);
        } catch (Exception e) {
            throw new CheckCertificateRevokedUnexpectedException(e);
        }
        return response;
    }

    /**
     * Returns the type of OCES certificate, e.g. "company" or "employee"
     * certificates.
     * 
     * @param certificate
     *            The certificate to check
     * @return Returns the OCES certificate type
     */
    public static OcesCertificateType getCertificateType(X509Certificate certificate)
            throws CheckCertificateTypeUnexpectedException {
        OcesCertificateType certType = new OcesCertificateType(OcesCertificateType.NonOces);

        try {
            // no check included for LRA and Device
            String subjectString = certificate.getSubjectDN().toString();
            if (subjectString.indexOf("PID") != -1) {
                certType = new OcesCertificateType(OcesCertificateType.OcesPersonal);
            } else if (subjectString.indexOf("CVR") != -1) {
                if (subjectString.indexOf("RID") != -1) {
                    certType = new OcesCertificateType(OcesCertificateType.OcesEmployee);
                } else if (subjectString.indexOf("UID") != -1) {
                    certType = new OcesCertificateType(OcesCertificateType.OcesOrganisation);
                }
            }
        } catch (Exception e) {
            throw new CheckCertificateTypeUnexpectedException(e);
        }
        return certType;
    }

    /**
     * Sets the certificate type.
     * 
     * @param certificate
     *            The certificate to check
     * @param result
     *            The result object to store the result
     */
    private void checkCertificateType(X509Certificate certificate, CertificateCheckResult result) throws Exception {
        try {
            result.setCertificateType(getCertificateType(certificate));
        } catch (CheckCertificateTypeUnexpectedException e) {
            throw e;
        } catch (Exception e) {
            throw e;
        }
    }
}
