/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security;

import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Vector;

import javax.security.auth.x500.X500Principal;

import org.bouncycastle.jce.X509Principal;

import dk.gov.oiosi.exception.ArgumentException;
import dk.gov.oiosi.exception.EmptyStringException;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.exception.NullArgumentException;

/**
 * Represents the Subject Number of an OCES certificate. Example format:
 * 'SERIALNUMBER=CVR:25767535-UID:1100080130597 + CN=TDC TOTALL�SNINGER A/S -
 * TDC Test, O=TDC TOTALL�SNINGER A/S // CVR:25767535, C=DK' Note that windows
 * certificate viewer shows these differently
 */
public class CertificateSubject {

    /**
     * subject. 
     */
    private String subjectString;

    /**
     * subject serial number.
     */
    private String subjectSerialNumber;

    /**
     * Constructor that takes a string representation of the certificate subject.
     * 
     * @param subject subject to parse
     * @throws MainException
     */
    public CertificateSubject(String subject) {
        if (subject == null) {
            throw new NullArgumentException("CertificateSubject");
        }
        X500Principal principal = new X500Principal(subject);
        setSubjectString(principal.getName(X500Principal.RFC1779));
    }

    /**
     * Constructor.
     * @param certificate certificate
     * @throws MainException
     * @throws CertificateEncodingException 
     */
    public CertificateSubject(X509Certificate certificate) {
        if (certificate == null) {
            throw new NullArgumentException("CertificateSubject");
        }
        setSubjectString(certificate.getSubjectX500Principal().getName(X500Principal.RFC1779));
    }

    /**
     * Constructor.
     * @param subjectDN subject to parse
     * @throws MainException
     */
    public CertificateSubject(X500Principal subjectDN) {
        if (subjectDN == null) {
            throw new NullArgumentException("CertificateSubject");
        }
        setSubjectString(subjectDN.getName(X500Principal.RFC1779));
    }

    /**
     * Util to parse subject for serial number.
     * 
     * @param subject subject
     * @throws MainException
     */
    private void setSubjectString(String subject) {
        if (subject == null) {
            throw new NullArgumentException("CertificateSubject");
        }
        if (subject.trim().length() < 1) {
            throw new EmptyStringException("CertificateSubject");
        }
        subjectString = subject;
        // parse
        subjectSerialNumber = parseSerialNumber(subjectString);
    }

    /**
     * Subject as String.
     * @return subject
     */
    public String getSubjectString() {
        return subjectString;
    }

    /**
     * Subject Serial Number
     * @return subject serial number
     */
    public String getSerialNumberValue() {
        return subjectSerialNumber;
    }

    /**
     * Subject Serial Number prefixed with
     * @return subject serial number
     */
    public String getPrefixedSerialNumber() {
        return SERIALNUMBERPREFIX + "=" + subjectSerialNumber;
    }

    /**
     * Serial number as text - for parsing.
     */
    public static final String SERIALNUMBERPREFIX = "serialNumber";

    /**
     * Serial number as OID - for parsing.
     */
    private static final String OIDVERSION = "OID.2.5.4.5";

    /**
     * Parse subject for serial number.
     * @param keyValue
     * @return the parsed serial number
     * @throws MainException
     */
    public static String parseSerialNumber(String keyValue) {
        X509Principal principal = new X509Principal(keyValue);
        Vector v = principal.getValues(X509Principal.SERIALNUMBER);
        if (v.size() > 0) {
            String ssn = v.get(0).toString();
            return ssn;
        }
        throw new ArgumentException("could not parse serialnumber [" + keyValue + "]");
    }
}
