/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.ldap;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Collection;

import javax.naming.CommunicationException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.lookup.CertificateLookupException;
import dk.gov.oiosi.security.lookup.ICertificateLookup;

/**
 * The ldap certificate lookup uses the ldap protocol to query for a given
 * certificate.
 * 
 * It implements the ICertificate lookup to garantee it has a speicific
 * interface for querying certificates.
 */
public class LdapCertificateLookup implements ICertificateLookup {

    /**
     * Local log.
     */
    private Log log = LogFactory.getLog(LdapCertificateLookup.class);

    /**
     * Settings.
     */
    private LdapSettings settings;

    /**
     * The LdapCertificateLookup constructor takes an ldap settings object as a
     * paramter. The setting object is used everytime a lookup is done in the
     * GetCertificate function.
     * 
     * @param s
     *            Setting object that contains the ldap settings to be used.
     */
    public LdapCertificateLookup(LdapSettings s) {
        settings = s;
    }

    /**
     * Default constructor. Attempts to read configuration settings from
     * configuration file
     */
    public LdapCertificateLookup() {
        settings = RaspConfigurationHandler.getInstance().getLdapSettings();
    }

    /**
     * The implementation of the interface ICertificateLookups function.
     * 
     * It takes the subject string and queries the ldap server for a certificate
     * that satifies the condition in the subject string.
     * 
     * @param subject
     *            The subject string of an OCES certificate.
     * @return The certificate that satifies the conditions of the subject
     *         string.
     * @throws CertificateLookupException
     */
    public X509Certificate getCertificate(CertificateSubject subject) throws CertificateLookupException {
        if (subject == null)
            throw new NullArgumentException("subjectSerialNumber");

        StringBuffer ldapQueryString = new StringBuffer();
        
        // protocol and server
        ldapQueryString.append("ldap://");
        ldapQueryString.append( settings.getHost() );
        ldapQueryString.append(":");
        ldapQueryString.append(settings.getPort());

        // search url
        ldapQueryString.append("/c=dk??sub?(");
        ldapQueryString.append(subject.getPrefixedSerialNumber());
        ldapQueryString.append(")");

        //
        X509Certificate certificate = null;

        log.debug("LDAP query is being executed...");
        log.debug("LDAP searchprefix: " + ldapQueryString);

        NamingEnumeration answer;
        boolean hasmore;
        try {
            // Create the initial context
            DirContext ctx = new InitialLdapContext();

            // Perform the search by using the URL
            answer= ctx.search(ldapQueryString.toString(), null);
            // Close the context when we're done
            ctx.close();

            hasmore = answer.hasMore();
        } catch (CommunicationException e) {
            log.error("Could not perform search: " + e, e);
            throw new ConnectingToLdapServerFailedException(settings, e);
        } catch (NamingException e) {
            log.error("Could not create Ldap Search Context : " + e, e);
            throw new LdapCertificateLookupInitializationFailedException("Could not create Ldap Search Context", e);
        }
  
        if(!hasmore) {
            throw new LdapCertificateNotFoundException(subject);
        }

        byte[] certificateBytes;
        try {
            SearchResult sr = (SearchResult) answer.next();
            Attributes attrs = sr.getAttributes();
            Attribute attribute = (Attribute) attrs.get("userCertificate;binary");

            certificateBytes = (byte[]) attribute.get();
        } catch(NamingException e) {
            log.error("Could not get certificate attribute from Ldap Search : " + e, e);
            throw new ConvertingSearchResultToCertificateFailedException("Could not get certificate attribute from Ldap Search", e);            
        }

        try {
            CertificateFactory cf = CertificateFactory.getInstance("X.509");
            Collection c = cf.generateCertificates(new ByteArrayInputStream(certificateBytes));
            certificate = (X509Certificate) c.iterator().next();
        } catch (CertificateException e) {
            log.error("Could not create certificate form ldap result : " + e, e);
            throw new ConvertingSearchResultToCertificateFailedException("Could not create certificate form ldap result", e);            
        }

        if (certificate == null) {
            log.info("Certificate not found with LDAP query:" + subject.getSerialNumberValue());
            throw new LdapCertificateNotFoundException(subject);
        }

        try {
            hasmore = answer.hasMore();
        } catch (NamingException e) {
            log.error("Could not create Ldap Search Context : " + e, e);
            throw new LdapCertificateLookupInitializationFailedException("Could not create Ldap Search Context", e);
        }        
        if(hasmore) {
            throw new LdapMultipleCertificatesFoundException(subject);
        }

        log.debug("LDAP query done...");
        return certificate;
    }

}
