/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.oces;

import java.security.cert.X509Certificate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bouncycastle.asn1.ASN1Sequence;
import org.bouncycastle.asn1.x509.PolicyInformation;
import org.bouncycastle.x509.extension.X509ExtensionUtil;

import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.security.OcspCheckStatus;
import dk.gov.oiosi.security.X509CheckStatus;
import dk.gov.oiosi.security.ocsp.IOcspLookup;
import dk.gov.oiosi.security.ocsp.OcspException;
import dk.gov.oiosi.security.ocsp.OcspResponse;

/**
 * Represents an OCES X509 certificate. Encapsulates an X509Certificate2 object.
 */
public class OcesX509Certificate {
    private static Log log = LogFactory.getLog(OcesX509Certificate.class);

    private static final String PERSONAL_OCES_CERTIFICATE_OID = "1.2.208.169.1.1.1.1.1";
    private static final String EMPLOYEE_OCES_CERTIFICATE_OID = "1.2.208.169.1.1.1.2.1";
    private static final String COMPANY_OCES_CERTIFICATE_OID = "1.2.208.169.1.1.1.3.1";
    private static final String DEVICE_OCES_CERTIFICATE_OID = "1.2.208.169.1.1.1.4.1";

    private static final String TEST_PERSONAL_OCES_CERTIFICATE_OID = "1.2.208.169.1.1.1.1.2";
    private static final String TEST_EMPLOYEE_OCES_CERTIFICATE_OID = "1.1.1.1.1.1.1.1.1.2";
    private static final String TEST_COMPANY_OCES_CERTIFICATE_OID = "1.1.1.1.1.1.1.1.1.3";
    private static final String TEST_DEVICE_OCES_CERTIFICATE_OID = "1.1.1.1.1.1.1.1.1.3";

    /**
     * Certificate.
     */
    private X509Certificate x509Certificate;

    /**
     * Status of logical check.
     */
    private X509CheckStatus x509CheckStatus = new X509CheckStatus(X509CheckStatus.NotChecked);

    /**
     * OCSP status.
     */
    private OcspCheckStatus ocspCheckStatus = new OcspCheckStatus(OcspCheckStatus.NotChecked);

    /**
     * Certificate type.
     */
    private OcesCertificateType ocesCertificateType = new OcesCertificateType(OcesCertificateType.NonOces);

    /**
     * Certificate subject.
     */
    private CertificateSubject subject;

    /**
     * Constructor that takes the X509Certificate wrapped. If the certificate is
     * not an OCES-certificate an exception will be thrown.
     * 
     * @param certificate
     *            An OCES x509 certificate
     * @throws MainException
     */
    public OcesX509Certificate(X509Certificate certificate) throws InvalidOcesCertificateException {
        if (certificate == null) {
            throw new NullArgumentException("certificate");
        }

        x509Certificate = certificate;
        subject = new CertificateSubject(x509Certificate);
        setCertificateType();

        if (ocesCertificateType.getType() == OcesCertificateType.NonOces) {
            throw new InvalidOcesCertificateException(certificate);
        }
    }

    /**
     * Checks the status of the certificate against an OCSP server. Updates the
     * internal state with the result.
     * 
     * @param ocspLookupClient
     *            The OCSP client to use for the request
     * @return Returns the check status
     * @throws OcspException
     * @throws MainException
     */
    public OcspCheckStatus checkOcspStatus(IOcspLookup ocspLookupClient) throws OcspException {
        OcspResponse response;
        try {
            response = ocspLookupClient.checkCertificate(x509Certificate);
        } catch (OcspException e) {
            ocspCheckStatus = new OcspCheckStatus(OcspCheckStatus.UnknownIssue);
            throw e;
        }

        if (response.isValid()) {
            ocspCheckStatus = new OcspCheckStatus(OcspCheckStatus.AllChecksPassed);
        } else {
            ocspCheckStatus = new OcspCheckStatus(OcspCheckStatus.CertificateRevoked);
        }

        return ocspCheckStatus;
    }

    /**
     * Gets the X509Certificate2 that is the basis of the OCES Certificate
     * 
     * @return certificate
     */
    public X509Certificate getCertificate() {
        return x509Certificate;
    }

    /**
     * Gets the certificate subject of the OCES certificate.
     * 
     * @return Certificate subject
     */
    public CertificateSubject getSubject() {
        return subject;
    }

    /**
     * Gets the certificate check status
     * 
     * @return x509 check status
     */
    public X509CheckStatus getX509CheckStatus() {
        return x509CheckStatus;
    }

    /**
     * Gets the OCSP check status
     * 
     * @return OCSP status
     */
    public OcspCheckStatus getOcspCheckStatus() {
        return ocspCheckStatus;
    }

    /**
     * Gets the certificate OCSP certificate type
     * 
     * @return certificate type
     */
    public OcesCertificateType getOcesCertificateType() {
        return ocesCertificateType;
    }

    /**
     * @deprecated
     * Gets certificate serial number
     * 
     * @return subject serial number
     * @throws MainException
     */
    public CertificateSubject getSubjectSerialNumber() throws MainException {
        return subject;
    }

    /**
     * Sets the certificate type.
     */
    private void setCertificateType() {
        ocesCertificateType = getOcesCertificateType(x509Certificate);
    }

    /**
     * Get the OCES certificate type from a given certificate.
     * 
     * @param certificate
     *            certificate
     * @return certificate type
     */
    public static OcesCertificateType getOcesCertificateType(X509Certificate certificate) {
        if (certificate == null) {
            throw new NullArgumentException("certificate");
        }

        try {
            // get extension from Java certificate
            byte[] extensionValueBytes = certificate.getExtensionValue("2.5.29.32");

            ASN1Sequence asn1Object = (ASN1Sequence) X509ExtensionUtil.fromExtensionValue(extensionValueBytes);
            PolicyInformation pi = new PolicyInformation((ASN1Sequence)asn1Object.getObjectAt(0));
            
            String policyIdentifier = pi.getPolicyIdentifier().getId();
            
            if( DEVICE_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return new OcesCertificateType(OcesCertificateType.OcesDevice);
            }
            if( COMPANY_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return new OcesCertificateType(OcesCertificateType.OcesOrganisation);
            }
            if( EMPLOYEE_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return new OcesCertificateType(OcesCertificateType.OcesEmployee);
            }
            if( PERSONAL_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return new OcesCertificateType(OcesCertificateType.OcesPersonal);
            }
            
            // TEST CERTIFICATES!!! DEVICE + COMPANY BOTH ARE : 1.1.1.1.1.1.1.1.1.3
            if( TEST_DEVICE_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return getFromSubject(certificate);
            }
            if( TEST_EMPLOYEE_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return new OcesCertificateType(OcesCertificateType.OcesEmployee);
            }
            if( TEST_PERSONAL_OCES_CERTIFICATE_OID.equals(policyIdentifier)) {
                return new OcesCertificateType(OcesCertificateType.OcesPersonal);
            }
        } catch (Exception e) {
            log.error("Could not get Oces certificate type from extension " + e, e);
        }

        try {
            return getFromSubject(certificate);
        } catch (Exception e) {
            log.error("Could not get Oces certificate type from subject " + e, e);
        }
        
        return new OcesCertificateType(OcesCertificateType.NonOces);
    }


    /**
     * Resolve certificate type from subject.
     * 
     * @param certificate
     *            certificate
     * @return certificate type
     * @throws MainException
     */
    private static OcesCertificateType getFromSubject(X509Certificate certificate) throws MainException {
        CertificateSubject subject = new CertificateSubject(certificate.getSubjectX500Principal().toString());
        String ssn = subject.getSubjectString();

        if (ssn == null) {
            return new OcesCertificateType(OcesCertificateType.NonOces);
        }

        if (ssn.indexOf("RID:") != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesEmployee);
        }
        if (ssn.indexOf("UID:") != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesOrganisation);
        }
        if (ssn.indexOf("PID:") != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesPersonal);
        }
        if (ssn.indexOf("FID:") != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesDevice);
        }
        
        // Old OCES test certificates have DID
        if (ssn.indexOf("DID:") != -1) {
            return new OcesCertificateType(OcesCertificateType.OcesDevice);
        }

        return new OcesCertificateType(OcesCertificateType.NonOces);
    }

}
