/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.security.ocsp;

import java.math.BigInteger;
import java.security.cert.X509Certificate;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.bouncycastle.ocsp.OCSPException;

import dk.certifikat.ocsp.client.OcspCertificateResponse;
import dk.certifikat.ocsp.client.OcspClient;
import dk.certifikat.ocsp.client.OcspRegistry;
import dk.certifikat.ocsp.client.OcspSystemException;
import dk.certifikat.ocsp.client.OcspCertificateResponse.OcspCertificateStatus;
import dk.gov.oiosi.configuration.ConfigurationException;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.CertificateSubject;

/**
 * Class for checking certificate revocation status against an OCSP server.
 */
public class OcspLookup implements IOcspLookup {

    /**
     * local log.
     */
    private Log log = LogFactory.getLog(OcspLookup.class);

    /**
     * ca for test.
     */
    private static final String TEST_CA_CERT_NAME = "ocestestca2.cer";

    /**
     * TDC production ca.
     */
    private static final String CA_CERT_NAME = "ocesca.cer";

    /**
     * OCSP configuration.
     */
    private OcspConfig configuration;

    /**
     * Gets the configuration of the lookup client.
     * @return OCSP configuration
     */
    public OcspConfig getConfiguration() {
        return configuration;
    }

    /**
     * Instantiates OcspLookup and loads the OCES default root certificate.
     * 
     * @param conf
     *            Configuration parameters
     * @throws MainException
     */
    public OcspLookup(OcspConfig conf) throws MainException {
        configuration = conf;
    }

    /**
     * Default constructor. Attempts to load configuration from configuration
     * file.
     * @throws ConfigurationException
     */
    public OcspLookup() throws ConfigurationException {
        configuration = RaspConfigurationHandler.getInstance().getOcspConfig();
    }

    /**
     * Checks a certificate status on a ocsp server.
     * 
     * @param certificate
     *            The certificate to check
     * @return The OcspResponse object that contains the result
     * @throws OcspException
     */
    public OcspResponse checkCertificate(X509Certificate certificate) throws OcspException {
        OcspResponse response = new OcspResponse();
        OcspCertificateResponse r = null;
        OcspClient client = null;
        try {
            log.debug("Setting up OCSP client:");
            client = OcspRegistry.getOcspClient();
            String serverUrl = configuration.getServerUrl();
            client.setResponderUrl(serverUrl);
            if (serverUrl.indexOf("test") > 0) {
                log.warn("OCSP using Test server");
                client.setCaCertInClassPath(TEST_CA_CERT_NAME);
            } else {
                log.debug("OCSP using production server ");
                client.setCaCertInClassPath(CA_CERT_NAME);
            }
        } catch (Exception e) {
            log.error("Error initailizing validator : " + e.getMessage(), e);
            throw new OcspException(e);
        }

        BigInteger[] serNos = {certificate.getSerialNumber()};
        try {
            log.warn("Getting certificate status for:" + serNos[0]);
            r = client.getCertificateStatus(serNos);
            log.debug("Validating certificate...");
            client.validateCerts(r);
        } catch (OcspSystemException e) {
            log.error("Error validating certificate : " + e.getMessage(), e);
            throw new OcspException(e);
        } catch (OCSPException e) {
            log.error("Error validating certificate : " + e.getMessage(), e);
            throw new OcspException(e);
        }

        if (r.isOk()) {
            OcspCertificateStatus status = r.getCertificateStatus(serNos[0]);
            if ("Good".equals(status.toString())) {
                log.debug("Status of cert " + certificate.getSubjectDN() + " : [" + status + "]");
                response.setIsValid(true);
                response.setNextUpdate(r.getNextUpdate());
            } else {
                log.warn("Certificate validation failed " + serNos[0] + " : [" + status + "]");
                throw new CertificateRevokedException(new CertificateSubject(certificate));
            }
        } else {
            log.error("Response returned an error: " + r.getError());
            throw new OcspException("Response returned an error: " + r.getError());
        }
        log.debug("Validating X509 certificate finished.");

        return response;
    }
}
