/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import java.util.List;

import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.uddi.category.ConformanceClaim;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;

/**
 * Key for use in dictionaries. Combines all the unique properties of a LookupParameter. Instantiate using the GetLookupKey in
 * LookupParameters class.
 */
public class LookupKey {

    private List addressTypeFilter;

    private UddiId businessProcessDefinitionTModel;

    private IIdentifier endpointKey;

    private EndpointKeytype endpointKeyType;

    private ConformanceClaim profileConformanceClaim;

    private RegistrationConformanceClaim registrationConformanceClaim;

    private BusinessProcessRoleIdentifier roleIdentifier;

    private UddiId serviceContractTModel;

    /**
     * Creates a lookup key. Usually called from within the LookupParameters class as this is a subset of the properties.
     * 
     * @param addressTypeFilter
     *            List of EndpointAddresssTypCode
     * @param businessProcessDefinitionTModel
     * @param endpointKey
     * @param endpointKeyType
     * @param profileConformanceClaim
     * @param registrationConformanceClaim
     * @param roleIdentifier
     * @param serviceContractTModel
     */
    public LookupKey(List addressTypeFilter, UddiId businessProcessDefinitionTModel, IIdentifier endpointKey,
            EndpointKeytype endpointKeyType, ConformanceClaim profileConformanceClaim,
            RegistrationConformanceClaim registrationConformanceClaim, BusinessProcessRoleIdentifier roleIdentifier,
            UddiId serviceContractTModel) {
        this.addressTypeFilter = addressTypeFilter;
        this.businessProcessDefinitionTModel = businessProcessDefinitionTModel;
        this.endpointKey = endpointKey;
        this.endpointKeyType = endpointKeyType;
        this.profileConformanceClaim = profileConformanceClaim;
        this.registrationConformanceClaim = registrationConformanceClaim;
        this.roleIdentifier = roleIdentifier;
        this.serviceContractTModel = serviceContractTModel;
    }

    /**
     * Returns the hashcode of the endpoint key
     * 
     * @return
     */
    public long getHashCode() {
        long hashCode = this.endpointKey.toString().hashCode();
        return hashCode;
    }

    /**
     * Compares to instances of a lookup key. All property values are compared.
     * 
     * @param obj
     * @return
     */
    public boolean equals(LookupKey other) {
        if (other == null) {
            return false;
        }

        if (!areEndpointAddressTypeCodesEqual(this.addressTypeFilter, other.addressTypeFilter))
            return false;
        if (!areEqual(this.businessProcessDefinitionTModel, other.businessProcessDefinitionTModel))
            return false;
        if (!areEqual(this.endpointKey, other.endpointKey))
            return false;
        if (!areEqual(this.endpointKeyType, other.endpointKeyType))
            return false;
        if (!areEqual(this.profileConformanceClaim, other.profileConformanceClaim))
            return false;
        if (!areEqual(this.registrationConformanceClaim, other.registrationConformanceClaim))
            return false;
        if (!areEqual(this.roleIdentifier, other.roleIdentifier))
            return false;
        if (!areEqual(this.serviceContractTModel, other.serviceContractTModel))
            return false;

        return true;
    }

    /**
     * Compare values.
     * 
     * @param obj1
     * @param obj2
     * @return
     */
    private boolean areEqual(Object obj1, Object obj2) {
        if (obj1 == null && obj2 == null)
            return true;
        if (obj1 != null && obj2 == null)
            return false;
        if (obj1 == null && obj2 != null)
            return false;

        if (!obj1.equals(obj2))
            return false;
        return true;
    }

    /**
     * Compare list of EndpointAddressTypes.
     * 
     * @param list1
     * @param list2
     * @return
     */
    private boolean areEndpointAddressTypeCodesEqual(List list1, List list2) {
        if (list1 != null && list2 == null)
            return false;
        if (list1 == null && list2 != null)
            return false;
        if (list1 == null && list2 == null)
            return true;
        if (list1.size() != list2.size())
            return false;

        for (int i = 0; i < list1.size(); i++) {
            if (!list2.contains(list1.get(i)))
                return false;
        }
        return true;
    }

}
