/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import java.util.List;

import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.cache.ITimedCache;
import dk.gov.oiosi.common.cache.TimedNullCache;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.ConformanceClaim;
import dk.gov.oiosi.uddi.category.ConformanceClaimCode;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaimCode;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;

/**
 * Holds a set of parameters and options for a specific lookup call.
 */
public class LookupParameters {

    /**
     * The key and key type of the endpoint.
     */
    private IIdentifier endpointKey;

    /**
     * The key of the endpoint.
     */
    public IIdentifier getEndpointKey() {
        return endpointKey;
    }

    /**
     * The type of the endpoint.
     */
    private EndpointKeytype endpointKeyType;

    /**
     * Gets the keytype of the endpoint.
     * 
     */
    public EndpointKeytype getEndpointKeyType() {
        return endpointKeyType;
    }

    /**
     * uddi id of ServiceContractModel
     */
    private UddiId serviceContractTModel;

    /**
     * The UDDI uuid of the tModel representing the service contract. This is
     * not the uuid of the endpoint tModel, but of the central service
     * definition tModel, e.g. the central electronic invoice endpoint tModel.
     * 
     * @return uddi id of ServiceContractModel
     */
    public UddiId getServiceContractTModel() {
        return serviceContractTModel;
    }

    /**
     * The UDDI uuid of the tModel representing the business process definition.
     */
    private UddiId businessProcessDefinitionTModel;

    /**
     * The UDDI uuid of the tModel representing the business process definition.
     * 
     * @return UDDI uiid of BusinessProcessDefinitionTModel
     */
    public UddiId getBusinessProcessDefinitionTModel() {
        return businessProcessDefinitionTModel;
    }

    /**
     * The type of the business process role identifier
     * 
     */
    private BusinessProcessRoleIdentifierType roleIdentifierType;

    /**
     * The type of the business process role identifier
     * 
     * @return type of business process role identifier
     */
    public BusinessProcessRoleIdentifierType getRoleIdentifierType() {
        return roleIdentifierType;
    }

    /**
     * The business process role identifier
     */
    private BusinessProcessRoleIdentifier roleIdentifier;

    /**
     * The preferred scheme of the endpoint
     */
    PreferredEndpointType preferredEndpointType;

    /**
     * The business process role identifier
     */
    public BusinessProcessRoleIdentifier getRoleIdentifier() {
        return roleIdentifier;
    }

    /**
     * Registration conformance claim. Instantiated with default claim (OIOSI
     * 1.0 UddiRegistrationModel)
     */
    private RegistrationConformanceClaim registrationConformanceClaim = new RegistrationConformanceClaim(
            new RegistrationConformanceClaimCode(RegistrationConformanceClaimCode.oiosi1_1));

    private ConformanceClaim profileConformanceClaim = new ConformanceClaim(new ConformanceClaimCode(
            ConformanceClaimCode.secureReliableAsyncProfile1_0));

    /**
     * The value of the UDDI registration conformance claim custom identifier
     * keyedReference value.
     * 
     */
    public RegistrationConformanceClaim getRegistrationConformanceClaim() {
        return registrationConformanceClaim;
    }

    /**
     * The value of the profile registration conformance claim custom identifier
     * keyedReference value. Set to the default (RASP 1.0 profile)
     */
    public ConformanceClaim getProfileConformanceClaim() {
        return profileConformanceClaim;
    }

    /**
     * List of EndpointAddressTypeCode
     */
    private List addressTypeFilter;

    /**
     * List of endpoint address types that may be returned in the search. If
     * this property is null or empty, all address types should be returned.
     * 
     * @return List of EndpointAddressTypeCode
     */
    public List getAddressTypeFilter() {
        return addressTypeFilter;
    }

    /**
     * The return semantics of the Lookup() operation. See the
     * LookupReturnOption for details.
     */
    private LookupReturnOption lookupReturnOption = new LookupReturnOption(LookupReturnOption.allResults);

    /**
     * The return semantics of the Lookup() operation. See the
     * LookupReturnOption for details.
     * 
     * @return LookupReturnOption
     */
    public LookupReturnOption getLookupReturnOption() {
        return lookupReturnOption;
    }

    /**
     * The preferred scheme of the endpoint returned
     * 
     * @return PreferredEndpointType
     */
    public PreferredEndpointType getPreferredEndpointType() {
        return preferredEndpointType;
    }

    /**
     * LookupCache
     * 
     * DotNet original : private ITimedCache<LookupKey, List<UddiLookupResponse>>
     * lookupCache = new TimedNullCache<LookupKey, List<UddiLookupResponse>>();
     */
    private ITimedCache lookupCache = new TimedNullCache();

    /**
     * The caching model to use in uddi lookups. The default is no cache.
     */
    public ITimedCache getLookupCache() {
        return lookupCache;
    }

    /**
     * The caching model to use in uddi lookups.
     * @param value cache
     */
    public void setLookupCache(ITimedCache value) {
        if (value == null) {
            throw new NullArgumentException("value was null");
        }
        lookupCache = value;
    }

    /**
     * Constructor.
     * 
     * @param endpointKey
     *            The endpoint key
     * @param endpointKeyType
     *            The type of the endpoint key
     * @param addressTypeFilter
     *            List of endpoint address types that should be returned in the
     *            search. If this property is null or empty, all address types
     *            should be returned.
     * @param preferredEndpointType
     *            The preferred type of endpoint address, e.g. http or email
     * @param lookupReturnOption
     *            Set the return semantics of the Lookup() operation. See the
     *            LookupReturnOption for details.
     * @param serviceContractTModel
     *            The UDDI id of the tModel representing the service contract
     * @param roleIdentifierType
     *            The type of process role identifier. If this value is null,
     *            the roleIdentifier must also be null
     * @param roleIdentifier
     *            The type of the process role identifier. If this value is
     *            null, the roleIdentifierType must also be null
     * @param processDefinitionTModel
     *            The UDDI id of the tModel representing the business process
     *            definition
     */
    public LookupParameters(IIdentifier endpointKey,
            EndpointKeytype endpointKeyType,
            List addressTypeFilter, PreferredEndpointType preferredEndpointType, LookupReturnOption lookupReturnOption,
            UddiId serviceContractTModel, BusinessProcessRoleIdentifierType roleIdentifierType,
            BusinessProcessRoleIdentifier roleIdentifier, UddiId processDefinitionTModel) {
        
        this.addressTypeFilter = addressTypeFilter;
        this.lookupReturnOption = lookupReturnOption;
        this.endpointKey = endpointKey;
        this.endpointKeyType = endpointKeyType;
        this.serviceContractTModel = serviceContractTModel;
        this.roleIdentifierType = roleIdentifierType;
        this.roleIdentifier = roleIdentifier;
        this.businessProcessDefinitionTModel = processDefinitionTModel;
        this.preferredEndpointType = preferredEndpointType;
    }

    /**
     * Compares the object members that constitutes a look up key and returns
     * true if the two objects have the same key
     * 
     * @param lookupParameters
     * @return
     */
    public boolean hasKeyValueEqualTo(LookupParameters lookupParameters) {
        if (this.addressTypeFilter != lookupParameters.addressTypeFilter)
            return false;
        if (this.businessProcessDefinitionTModel != lookupParameters.businessProcessDefinitionTModel)
            return false;
        if (this.endpointKey != lookupParameters.endpointKey)
            return false;
        if (this.endpointKeyType != lookupParameters.endpointKeyType)
            return false;
        if (this.profileConformanceClaim != lookupParameters.profileConformanceClaim)
            return false;
        if (this.registrationConformanceClaim != lookupParameters.registrationConformanceClaim)
            return false;
        if (this.roleIdentifier != lookupParameters.roleIdentifier)
            return false;
        if (this.serviceContractTModel != lookupParameters.serviceContractTModel)
            return false;

        return true;
    }

    /**
     * Returns the lookup key object used in the timed UDDI cache
     * 
     * @return Returns the lookup key object used in the timed UDDI cache
     */
    public LookupKey getLookupKey() {
        LookupKey lookupKey = new LookupKey(addressTypeFilter, businessProcessDefinitionTModel, endpointKey, endpointKeyType,
                profileConformanceClaim, registrationConformanceClaim, roleIdentifier, serviceContractTModel);
        return lookupKey;
    }

}
