/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.EndpointAddressSMTP;
import dk.gov.oiosi.addressing.EndpointAddressHttp;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.MailAddress;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.CertificateSubject;
import dk.gov.oiosi.uddi.category.ArsCategory;
import dk.gov.oiosi.uddi.category.ArsEndpointKey;
import dk.gov.oiosi.uddi.category.AuthenticationRequired;
import dk.gov.oiosi.uddi.category.BusinessProcessDefinitionReference;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.ConformanceClaim;
import dk.gov.oiosi.uddi.category.EndpointActivationDate;
import dk.gov.oiosi.uddi.category.EndpointAddressTypeCode;
import dk.gov.oiosi.uddi.category.EndpointCertificate;
import dk.gov.oiosi.uddi.category.EndpointContactEmail;
import dk.gov.oiosi.uddi.category.EndpointExpirationDate;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.category.NewerVersionReference;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.TermsOfUseUrl;
import dk.gov.oiosi.uddi.category.VersionMajor;
import dk.gov.oiosi.uddi.category.VersionMinor;
import dk.gov.oiosi.uddi.category.VersionRevision;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;
import dk.itst.uddi.client.ConnectionInfo;
import dk.itst.uddi.client.InquiryClient;
import dk.itst.uddi.client.UDDIClient;
import dk.itst.uddi.client.exceptions.UDDIException;
import dk.itst.uddi.client.query.FindServiceQuery;
import dk.itst.uddi.client.query.FindServiceResult;
import dk.itst.uddi.client.types.core.AccessPoint;
import dk.itst.uddi.client.types.core.BindingTemplate;
import dk.itst.uddi.client.types.core.BusinessService;
import dk.itst.uddi.client.types.core.CategoryBag;
import dk.itst.uddi.client.types.core.IdentifierBag;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.ServiceInfo;
import dk.itst.uddi.client.types.core.ServiceInfos;
import dk.itst.uddi.client.types.core.ServiceList;
import dk.itst.uddi.client.types.core.TModel;
import dk.itst.uddi.client.types.core.TModelInstanceInfo;

/** 
 * This class represents an inquiry to the UDDI inquiry API.
 * It uses the Oiosi UDDI SDK to perform a set of inquiries to a UDDI.
 * Creates a new instance per request.
 */
public class UddiInquiry {
	private static final int NO_OF_RESULTS = 128;
	protected UddiConfig configuration;
	
	private UDDIClient client;
	private UDDIClient clientFallback;
	/**
	 * local logger.
	 */
	private Log log = LogFactory.getLog(UddiInquiry.class);
	
	/**
	 * Constructor that takes the configuration as parameter
	 * 
	 * @param configuration The UDDI configuration.
	 * @throws MainException
	 */
	public UddiInquiry(UddiConfig configuration) throws MainException {
		try {
			this.configuration = configuration;
			ConnectionInfo ci = new ConnectionInfo();
			ci.setInquiry(configuration.getUddiInquireEndpointURL());
			client = new UDDIClient(ci);
			if (configuration.getTryOtherHostsOnFailure()) {
			    ConnectionInfo ciFallback = new ConnectionInfo();
			    ciFallback.setInquiry(configuration.getUddiInquireEndpointURLFallback());
			    clientFallback = new UDDIClient(ciFallback);
			}
		} catch (Exception ex) {
			throw new UddiException(ex);
		}
	}
	
	/**
	 * Inquire method that takes the endpoint key and the lookup parameters
	 * that is used for the search in the UDDI.
	 * 
	 * The result is returned in a list where the objects are of the type UddiLookupResponse.
	 * 
	 * @param endpointKey
	 * @param inquiryParameters
	 * @return
	 * @throws MainException
	 */
	public List Inquire(IIdentifier endpointKey, LookupParameters inquiryParameters) throws MainException {
		try {
			return Inquire(client, endpointKey, inquiryParameters);
		} catch (UDDIException ex) {
			if (clientFallback == null) throw ex;
			return Inquire(clientFallback, endpointKey, inquiryParameters);
		}
	}

	private List Inquire(UDDIClient client, IIdentifier endpointKey, LookupParameters inquiryParameters) throws MainException {
		List result = new ArrayList();
		
		ArsEndpointKey arsEndpointKey = new ArsEndpointKey(inquiryParameters.getEndpointKey());
		ServiceList serviceList = getServices(client, arsEndpointKey, inquiryParameters.getServiceContractTModel().getID(), inquiryParameters.getEndpointKeyType(), inquiryParameters.getProfileConformanceClaim(), inquiryParameters.getRegistrationConformanceClaim());
		
		ServiceInfo[] serviceInfos = serviceList.getServiceInfos().getServiceInfoArray();
		if (serviceInfos == null) return result;
		if (serviceInfos.length < 1) return result;
		
		String[] servicekeysFound = new String[serviceInfos.length];
		for (int i=0; i<serviceInfos.length; i++) {
			servicekeysFound[i] = serviceInfos[i].getServiceKey();
		}
		BusinessService[] services = getServiceDetails(client, servicekeysFound);
		
		for (int i=0; i<services.length; i++) {
			BusinessService service = services[i];
			if (isInactiveOrExpired(service)) continue;
			
			BindingTemplate[] bindingTemplates = service.getBindingTemplates().getBindingTemplateArray();
			for(int j=0; j<bindingTemplates.length; j++) {
				List tModelKeyList = new ArrayList();
				BindingTemplate bindingTemplate = bindingTemplates[j];
				TModelInstanceInfo[] infos = bindingTemplate.getTModelInstanceDetails().getTModelInstanceInfoArray();
				for (int k=0; k<infos.length; k++) {
					tModelKeyList.add(infos[k].getTModelKey());
				}
				String[] tModelKeys = new String[tModelKeyList.size()];
				for (int k=0; k<tModelKeyList.size(); k++) {
					tModelKeys[k] = tModelKeyList.get(k).toString();
				}
				TModel[] tmodels = getTModelDetail(client, tModelKeys);
				addEndpointAddress(endpointKey, tmodels, service, bindingTemplate, inquiryParameters, result);
			}
		}
		return result;
	}
	
	private ServiceList getServices(UDDIClient client, ArsEndpointKey arsEndpointKey, String centralServiceDefinitionTModelKey, EndpointKeytype endpointKeyType, ConformanceClaim profileConformanceClaim, RegistrationConformanceClaim registrationConformanceClaim) throws MainException {
		log.debug("getServices : " + arsEndpointKey.getValue());
		FindServiceQuery q = client.getInquiry().findService();
		
		//Set categories
		if (profileConformanceClaim != null) {
			KeyedReference kr = profileConformanceClaim.getAsKeyedReference();
		    q.addCategory(kr.getKeyName(), kr.getKeyValue(), kr.getTModelKey());
		}
		if (registrationConformanceClaim != null) {
			KeyedReference kr = registrationConformanceClaim.getAsKeyedReference();
			q.addCategory(kr.getKeyName(), kr.getKeyValue(), kr.getTModelKey());
		}

		// NOTE: adding the EAN key LAST in the category bag seems to increase systinet 
		// UDDI registry performance dramatically. 
		{
		KeyedReference kr = arsEndpointKey.getAsKeyedReference();
		q.addCategory(kr.getKeyName(), kr.getKeyValue(), kr.getTModelKey());
		}

		if (endpointKeyType != null) {
			KeyedReference kr = endpointKeyType.getAsKeyedReference();
		    q.addCategory(kr.getKeyName(), kr.getKeyValue(), kr.getTModelKey());
		}

		q.setLimit(NO_OF_RESULTS);
		q.addTModel(centralServiceDefinitionTModelKey);

		//Get services
		log.debug("Executing services query...");

		ServiceList sl = ServiceList.Factory.newInstance();
		ServiceInfos sis = ServiceInfos.Factory.newInstance();

		FindServiceResult services = q.execute();
		log.debug("Service Result..." + services.getCount() + " : " + services.getTotalCount() + " : "  + services.getServiceInfos());

		sis.setServiceInfoArray(services.getServiceInfos());
		sl.setServiceInfos(sis);
		
		log.debug("SIZE : " + services.getTotalCount());
		
		return sl;
	}
		
	private BusinessService[] getServiceDetails(UDDIClient client, String[] servicekeysFound) throws UddiException {
		try {
			InquiryClient inquiry = client.getInquiry();
			return inquiry.getServices(servicekeysFound);
		} catch (Exception e) {
			throw new UddiException(e.getMessage());
		}
	}
	
	public TModel[] getTModelDetail(UDDIClient client, String[] modelKeys) throws MainException {
		try {
			InquiryClient inquiry = client.getInquiry();
			return inquiry.getTModels(modelKeys);
		} catch (Exception e) {
			throw new UddiException(e);
		}
	}
	
	private boolean isInactiveOrExpired(BusinessService service) {
		Date activationDateUTC = getActivationDate(service);
		Date expirationDateUTC = getExpirationDate(service);
		Date nowUTC = new Date();
	
		if (nowUTC.before(activationDateUTC)) {
			return true;
		}
		if (nowUTC.after(expirationDateUTC)) {
			return true;
		}
		return false;
	}

	/** 
	 * Gets endpoint activation date from category value, in UTC
	 */
	private Date getActivationDate(BusinessService service) {
		Date activation = new Date(Long.MIN_VALUE);
	
		CategoryBag bag = service.getCategoryBag();
		KeyedReference act = findKeyedReferenceByName(bag, new EndpointActivationDate().getCategoryName());
	
		if (act != null) {
			activation = getDatetimeFromLifetimeDates(act.getKeyValue(), false);
		}
		return activation;
	}

	/** 
	 * Gets endpoint expiration date from category value, in UTC
	 */
	private Date getExpirationDate(BusinessService service) {
		Date expiration = new Date(Long.MAX_VALUE);
	
		CategoryBag bag = service.getCategoryBag();
		KeyedReference exp = findKeyedReferenceByName(bag, new EndpointExpirationDate().getCategoryName());
	
		if (exp != null) {
			expiration = getDatetimeFromLifetimeDates(exp.getKeyValue(), false);
		}
		return expiration;
	}
	
	/**	
	 * Search CatagoryBag for named item.	
	 * @param bag
	 * @param categoryName
	 * @return
	 */
	private KeyedReference findKeyedReferenceByName(CategoryBag bag, String categoryName) {
		KeyedReference[] list = bag.getKeyedReferenceArray();
		for (int i = 0; i < list.length; i++) {
			if (list[i].getKeyName().equals(categoryName)) {
				return list[i];
			}
		}
		return null;
	}


	/** 
	 * Returns a DateTime representation of either an endpoint expiration
	 * or activation date, in UTC
	 * 
	 * @param  datestring The date of an endpoint expiration or activation
	 * @param  getActivationDate True if the date represents an activation date,
	 * false if expiration date. This influences which default is returned, if no date
	 * is present in the string (DateTime.MinValue for activation date, 
	 * DateTime.MaxValue for expiration time
	 * @return 
	 * Returns a DateTime representation of either an endpoint expiration
	 * or activation date, in UTC
	 * 
	 */
	private Date getDatetimeFromLifetimeDates(String date, boolean getActivationDate) {
		if (ValUtil.isNullOrEmpty(date)) {
			// Then set default date:
			if (getActivationDate) {
				return new Date(Long.MIN_VALUE);
			} else {
				return new Date(Long.MAX_VALUE);
			}
		}
		// 1 - try correct format : "2019-09-07T00:00:00.0000000Z"
		try {
			return new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSSSSS'Z'").parse(date);
		} catch (Exception ignore) {
		}
		// 2 - fall back to "yyyy-MM-dd"
		try {
			return new SimpleDateFormat("yyyy-MM-dd").parse(date);
		} catch (Exception ignore) {
		}
		return null;
	}

	/** 
	 * Creates endpoint information, if the category value are as expected,
	 * and adds it to the list of results
	 */
	private void addEndpointAddress(IIdentifier endpointKey, TModel[] tmodels, BusinessService service, BindingTemplate bindingTemplate,
		LookupParameters parameters, List results) throws MainException {
		
		//TODO: Should these be used in anyway? Compare with .Net
		//String bindingKey = bindingTemplate.getBindingKey();
		//String serviceKey = bindingTemplate.getServiceKey();
		
		
		AccessPoint accessPoint = bindingTemplate.getAccessPoint();
		log.debug("addEndpointAddress : " + accessPoint.getStringValue());
		// 1. Check properties of associated tModels:
		if (!meetsTModelCriteria(tmodels, parameters)) {
			// Do not add to the result set:
			log.warn("Endpoint does not meet model criteria : Do not add to the result set");
			return;
		}
	
		// 2. make a new endpointaddress
		log.error("accessPoint.getStringValue() : " + accessPoint.getStringValue());
		EndpointAddress endpointAddress = IdentifierUtility.getEndpointAddressFromString(accessPoint.getStringValue());
		
		// 3. Check against address type filter:
		if (!isAcceptableAddressType(endpointAddress, parameters)) {
			return;
		}
	
		// 4. get authenticationrequired
		//TODO: remove, if it should not be used
		//boolean autRequired = getAuthenticationRequired(service);
	
		// 5. get endpoint activation and expiration time
		Date activationUTC = getActivationDate(service);
		Date expirationUTC = getExpirationDate(service);
	
		// 6. get endpoint certificate subject
		String cert = getCertificate(service);
	
		// 7. get terms of use url:
		Uri termsOfUse = getTermsOfUseUrl(service);
	
		// 8. Get contact address:
		MailAddress contactMail = getContactMail(service);
	
		// 9. Get version:
		Version version;
		try {
			version = getVersion(service);
		} catch (Exception e) {
			throw new MainException(e);
		}
		// 10. Get newer version reference:
		UddiId newerVersion = getNewerVersion(service);
	
		// 11. make a new UddiLookupResponse
		UddiLookupResponse resp = new UddiLookupResponse(endpointKey, endpointAddress, activationUTC, expirationUTC,
				new CertificateSubject(cert), termsOfUse, contactMail, version, newerVersion);
	
		// 12. add the endpointinformation to the result object
		results.add(resp);
	}
	
	/** 
	 * Checks tModel parameters of a binding, e.g. if the required process-related
	 * categories are set correctly. At least one process-related tModel must 
	 * meet all the specified criteria.
	 * 
	 * @param  tmodels The tModels for the binding to check
	 * @param  parameters The lookup parameters specifying the criteria
	 * @return True if criteria are met
	 */
	private boolean meetsTModelCriteria(TModel[] tmodels, LookupParameters parameters) {
		log.debug("meetsTModelCriteria - # of tmodels " + tmodels.length);
		// 1. Have any process-related criteria been set?
		if (parameters.getBusinessProcessDefinitionTModel() == null && parameters.getRoleIdentifier() == null
				&& parameters.getRoleIdentifierType() == null) {
			// If no criteria are set, return true
			log.debug("meetsTModelCriteria - # no process related criteria");
			return true;
		}
		for (int i = 0; i < tmodels.length; i++) {
			TModel tm = tmodels[i];
			if (isProcessInstance(tm, parameters)) {
				if (isAcceptableProcessInstance(tm, parameters)) {
					return true;
				}
				;
			}
		}
		return false;
	}

	/** 
	 * Returns true if this TModel represents a valid process instance
	 * 
	 * @param  tmodel The TModel to check
	 * @param  parameters The lookup parameters
	 * @return True if valid
	 */
	private boolean isProcessInstance(TModel tmodel, LookupParameters parameters) {
		log.debug("isProcessInstance : " + tmodel.getTModelKey());
		if (tmodel == null || tmodel.getCategoryBag() == null && tmodel.getCategoryBag().sizeOfKeyedReferenceArray() < 1) {
			return false;
		}
		// 1. Check registration conformance claim
		RegistrationConformanceClaim regConfClaim = new RegistrationConformanceClaim();
		KeyedReference confClaimKeyref = findKeyedReferenceById(tmodel.getCategoryBag(), regConfClaim.getCategoryID());
		if (confClaimKeyref == null) {
			return false;
		}
		if (!confClaimKeyref.getKeyValue().equals(parameters.getRegistrationConformanceClaim().getValue())) {
			return false;
		}

		// 2. Check that the businessProcessDefinitionReference category exists:
		BusinessProcessDefinitionReference procRef = new BusinessProcessDefinitionReference();
		KeyedReference procKeyref = findKeyedReferenceById(tmodel.getCategoryBag(), procRef.getCategoryID());
		
		if (procKeyref == null) {
			return false;
		}
		return true;
	}

	/**
	 * Search CatagoryBag for specific id.
	 * @param bag
	 * @param id
	 * @return
	 */
	private KeyedReference findKeyedReferenceById(CategoryBag bag, String id) {
		KeyedReference[] list = bag.getKeyedReferenceArray();
		for (int i = 0; i < list.length; i++) {
			if (list[i].getKeyValue().equals(id)) {
				return list[i];
			}
		}
		return null;
	}		
	
	/** 
	 * Returns true if this process instance tModel matches the criteria  specified
	 * by the user
	 * 
	 * @param  tmodel The tModel to check
	 * @param  parameters Lookup parameters
	 * @return True if criteria are matched
	 */
	private boolean isAcceptableProcessInstance(TModel tmodel, LookupParameters parameters) {
		log.debug("isAcceptableProcessInstance");
		// NOTE: .Net starts with test no 2...

		// 2. Check for businessProcessDefinitionReference:
		if (parameters.getBusinessProcessDefinitionTModel() != null) {
			BusinessProcessDefinitionReference procRef = new BusinessProcessDefinitionReference();
			KeyedReference procKeyref = findKeyedReferenceById(tmodel.getCategoryBag(), procRef.getCategoryID());
			if (procKeyref == null) {
				return false;
			}
			if (!procKeyref.getKeyValue().equalsIgnoreCase(parameters.getBusinessProcessDefinitionTModel().getID())) {
				return false;
			}
		}

		// 3. Check the role identifier type, if relevant:
		if (parameters.getRoleIdentifierType() != null) {
			BusinessProcessRoleIdentifierType roleIdType = new BusinessProcessRoleIdentifierType();
			KeyedReference roleTypeKeyref = findKeyedReferenceById(tmodel.getCategoryBag(), roleIdType.getCategoryID());
			if (roleTypeKeyref == null) {
				return false;
			}
			if (!roleTypeKeyref.getKeyValue().equalsIgnoreCase(parameters.getRoleIdentifierType().getValue())) {
				return false;
			}
		}

		// 4. Check the Role identifier itself, if relevant:
		if (parameters.getRoleIdentifier() != null) {
			BusinessProcessRoleIdentifier roleId = new BusinessProcessRoleIdentifier();
			KeyedReference roleKeyref = findKeyedReferenceById(tmodel.getIdentifierBag(), roleId.getIdentifierID());
			if (roleKeyref == null) {
				return false;
			}
			if (!roleKeyref.getKeyValue().equalsIgnoreCase(parameters.getRoleIdentifier().getValue())) {
				return false;
			}
		}
		// 5. All tests passed, return true
		return true;
	}
	
	private UddiId getNewerVersion(BusinessService service) {
		NewerVersionReference newRef = new NewerVersionReference();
		CategoryBag bag = service.getCategoryBag();
		KeyedReference newVersKeyref = findKeyedReferenceById(bag, newRef.getCategoryID());
		if (newVersKeyref != null) {
			if (!ValUtil.isNullOrEmpty(newVersKeyref.getKeyValue())) {
				return dk.gov.oiosi.common.IdentifierUtility.getUddiIDFromString(newVersKeyref.getKeyValue());
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
	
	/** 
	 * Gets authenticationRequired bool from category value
	 */
	private boolean getAuthenticationRequired(BusinessService service) {
		boolean authenticationRequired = false;
		CategoryBag bag = service.getCategoryBag();
		KeyedReference autRequired = findKeyedReferenceByName(bag, new AuthenticationRequired().getCategoryName());
		if (autRequired != null) {
			if (autRequired.getKeyValue().equals("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/authenticationRequired/")) {
				authenticationRequired = true;
			}
		}
		return authenticationRequired;
	}
	
	/** 
	 * Checks if an endpoint address is of a type that is on the endpoint filter list.
	 * 
	 * @param  address The address to check
	 * @param  parameters The parameter object holding the filter
	 * @return True if the address type is on the list
	 */
	private boolean isAcceptableAddressType(EndpointAddress address, LookupParameters parameters) {
		if (parameters.getAddressTypeFilter() == null || parameters.getAddressTypeFilter().size() < 1) {
			return true;
		}
		if (address instanceof EndpointAddressSMTP) {
			List list = parameters.getAddressTypeFilter();
			for (int i = 0; i < list.size(); i++) {
				EndpointAddressTypeCode typecode = (EndpointAddressTypeCode) list.get(i);
				if (typecode.getCode() == EndpointAddressTypeCode.email)
					return true;
			}
		} else if (address instanceof EndpointAddressHttp) {
			List list = parameters.getAddressTypeFilter();
			for (int i = 0; i < list.size(); i++) {
				EndpointAddressTypeCode typecode = (EndpointAddressTypeCode) list.get(i);
				if (typecode.getCode() == EndpointAddressTypeCode.http)
					return true;
			}
		}
		return false;
	}
	
	/** 
	 * Gets certificate from category value.
	 * @param service service
	 * @return 
	 */
	private String getCertificate(BusinessService service) {
		String cert = "";
		CategoryBag bag = service.getCategoryBag();
		KeyedReference certificate = findKeyedReferenceByName(bag, new EndpointCertificate().getCategoryName());
		if (certificate != null) {
			cert = certificate.getKeyValue();
		}
		return cert;
	}	
	
	/**
	 * @param service
	 * @return
	 */
	private Uri getTermsOfUseUrl(BusinessService service) {
		TermsOfUseUrl terms = new TermsOfUseUrl();
		CategoryBag bag = service.getCategoryBag();
		KeyedReference termsRef = findKeyedReferenceById(bag, terms.getCategoryID());
		if (termsRef != null) {
			if (!ValUtil.isNullOrEmpty(termsRef.getKeyValue())) {
				return new Uri(termsRef.getKeyValue());
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
	
	private MailAddress getContactMail(BusinessService service) {
		EndpointContactEmail mail = new EndpointContactEmail();
		CategoryBag bag = service.getCategoryBag();
		KeyedReference mailRef = findKeyedReferenceById(bag, mail.getCategoryID());
		if (mailRef != null) {
			if (!ValUtil.isNullOrEmpty(mailRef.getKeyValue())) {
				return new MailAddress(mailRef.getKeyValue());
			} else {
				return null;
			}
		} else {
			return null;
		}
	}
	
	/**
	 * Search IdentifierBag for specific id.
	 * @param bag
	 * @param id
	 * @return
	 */
	private KeyedReference findKeyedReferenceById(IdentifierBag bag, String id) {
		KeyedReference[] list = bag.getKeyedReferenceArray();
		for (int i = 0; i < list.length; i++) {
			if (list[i].getKeyValue().equals(id)) {
				return list[i];
			}
		}
		return null;
	}
	
	private Version getVersion(BusinessService service) throws Exception {
		Version v = null;
		VersionMajor major = new VersionMajor();
		major = (VersionMajor) getCategory(service, major);
		if (major == null)
			return null;
		VersionMinor minor = new VersionMinor();
		minor = (VersionMinor) getCategory(service, minor);
		VersionRevision rev = new VersionRevision();
		rev = (VersionRevision) getCategory(service, rev);
		// 2. Create the version number
		int majorInt = 0;
		int minorInt = 0;
		int revisionInt = 0;
		majorInt = Integer.parseInt(major.getValue());
		if (minor != null)
			minorInt = Integer.parseInt(minor.getValue());
		if (rev != null)
			revisionInt = Integer.parseInt(rev.getValue());
		v = new Version(majorInt, minorInt, revisionInt);
		return v;
	}
	
	/** 
	 * Gets an ARS category by the category ID of the 'categoryObject' 
	 * parameter. Note that we need an actual ArsCategory descendant 
	 * object to get the category identifier. If no match is found, null is returned.
	 * 
	 * @param  service The business service that holds the category bag to search
	 * @param  category The category object from which to get the category ID.
	 * Another object, with key and value set, is returned as a result.
	 * @return 
	 */
	private ArsCategory getCategory(BusinessService service, ArsCategory category) throws Exception {
		if (category == null) {
			return null;
		}
		CategoryBag bag = service.getCategoryBag();
		KeyedReference keyRef = findKeyedReferenceById(bag, category.getCategoryID());
		if (keyRef == null) {
			return null;
		}
		category.setCategoryValue(keyRef.getKeyName(), keyRef.getKeyValue());
		return category;
	}
}