/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.addressing.EndpointAddressSMTP;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.MainException;

/**
 * Class for resolving endpoints on the UDDI-based Address Resolution Service
 * (ARS). Features of this class:
 *  * Resolves a business level endpoint to a service endpoint, using the UDDI
 * 2.0 inquiry API * Performs automatic caching of resolving attempts * Supports
 * caching policies based on call result feedback
 * 
 * Extension points:
 *  * Extension mechanism for adding additional key types for resolving *
 * Extendable configuration model
 */
public class UddiLookupClient implements IUddiLookupClient {
    private UddiConfig configuration;

    /**
     * Constructor that takes the configuration as parameter.
     */
    public UddiLookupClient(UddiConfig configuration) throws MainException {
        this.configuration = configuration;
    }

    /**
     * Default constructor. Attempts to load configuration from file.
     */
    public UddiLookupClient() throws MainException {
        this.configuration = RaspConfigurationHandler.getInstance().getUddiConfig();
    }

    /**
     * Translates a business level key ("EndpointKey", e.g. an EAN number) to an
     * endpoint address (e.g. an URL).
     * 
     * @param parameters
     *            The business level key of the endpoint, e.g. an EAN number as
     *            well as options of the translation, i.e. address type filters
     * @return Returns a collection of matching addresses
     */
    public List lookup(LookupParameters parameters) throws MainException {
        List inquiryResult = null;

        // 1. Check cache, if not found there, inquire UDDI
        inquiryResult = cachedInquire(parameters);

        // 2. Is the request in the gateway range (if a gateway range exists)?
        if ((inquiryResult == null || inquiryResult.size() < 1)
                && parameters.getEndpointKey() instanceof IdentifierEan
                && configuration.getGatewayRange() != null
                && configuration.getGatewayRange().isInRange(parameters.getEndpointKey().getAsString())
                && configuration.getGatewayRange().getGatewayRegistrationParameters() != null
                && !ValUtil.isNullOrEmpty(configuration.getGatewayRange().getGatewayRegistrationParameters()
                        .getGatewayRegistrationKeyEan())) {
            inquiryResult = gatewayLookup(parameters);
        }

        // 3. If inquiry result is null, return empty list
        if (inquiryResult == null)
            return new ArrayList();

        // 4. Filter result
        List filteredResult = filterResponse(parameters, inquiryResult);

        // 5. Return result
        return filteredResult;
    }
    
    /**
     * Check cache, if not found there, inquire UDDI
     * 
     * @param parameters
     *            The business level key of the endpoint, e.g. an EAN number
     * @param inquiryResult
     *            The result of the inquiry as well as options of the
     *            translation, i.e. address type filters
     * @return Returns a List containing UddiLookupResponse objects
     */
    private List cachedInquire(LookupParameters parameters) throws MainException {
        UddiInquiry inquiry = new UddiInquiry(configuration);
        List inquiryResult;
        // Get result from cache or "live"
        LookupKey cacheLookupKey = parameters.getLookupKey();
        inquiryResult = (List)parameters.getLookupCache().tryGetValue(cacheLookupKey);
        if (inquiryResult == null) {
            inquiryResult = inquiry.Inquire(parameters.getEndpointKey(), parameters);
            // Add result to cache, if relevant
            if (inquiryResult != null && inquiryResult.size() != 0) {
                parameters.getLookupCache().add(cacheLookupKey, inquiryResult);
            }
        }
        return inquiryResult;
    }

    /**
     * Filters the response
     * 
     * @return
     */
    private List filterResponse(LookupParameters parameters, List inquiryResult) throws UddiException {
        List newFilteredResult = new ArrayList();
        // How many endpoints are we expecting?
        if (inquiryResult.size() > 0) {
            switch (parameters.getLookupReturnOption().getCode()) {

            case LookupReturnOption.allResults:
                return inquiryResult;
            case LookupReturnOption.firstResult:
                newFilteredResult.add(inquiryResult.get(0));
                return newFilteredResult;
            case LookupReturnOption.noMoreThanOneSetOrFail:
                if (inquiryResult.size() > 2) {
                    throw new UddiMoreThanOneEndpointException();
                } else if (inquiryResult.size() == 2) {
                    UddiLookupResponse resp0 = (UddiLookupResponse) inquiryResult.get(0);
                    UddiLookupResponse resp1 = (UddiLookupResponse) inquiryResult.get(1);
                    if (resp0.getEndpointAddress().getKeyAsString().equals(resp1.getEndpointAddress().getKeyTypeAsString())) {
                        throw new UddiTwoEqualEndpointsException();
                    } else {
                        int preferredIndex = 0;
                        if (resp0.getEndpointAddress() instanceof EndpointAddressSMTP) {
                            if (parameters.getPreferredEndpointType().getCode() == PreferredEndpointType.mailto) {
                                preferredIndex = 0;
                            } else {
                                preferredIndex = 1;
                            }
                        } else {
                            if (parameters.getPreferredEndpointType().getCode() == PreferredEndpointType.http) {
                                preferredIndex = 0;
                            } else {
                                preferredIndex = 1;
                            }
                        }
                        newFilteredResult.add(inquiryResult.get(preferredIndex));
                        return newFilteredResult;
                    }
                } else { // inquiryResult.Count == 1
                    newFilteredResult.add(inquiryResult.get(0));
                    return newFilteredResult;
                }
            default:
                return inquiryResult;
            }
        } else {
            return inquiryResult;
        }
    }

    /**
     * Translates a business level key ("EndpointKey", e.g. an EAN number) to an
     * endpoint address (e.g. an URL) for a gateway registration. Performs the
     * same actions as first part of Lookup(), but without checking for gateway
     * ranges.
     * 
     * @param parameters
     *            The business level key of the endpoint, e.g. an EAN number as
     *            well as options of the translation, i.e. address type filters
     * @return Returns a collection of matching addresses
     */
    private List gatewayLookup(LookupParameters parameters) throws MainException {
        List inquiryResult = null;

        // 1. Change the endpoint key to that of the gateway registration:
        parameters.getEndpointKey().set(
                configuration.getGatewayRange().getGatewayRegistrationParameters().getGatewayRegistrationKeyEan());

        // 2. Check cache, if not found there, inquire UDDI
        inquiryResult = cachedInquire(parameters);

        return inquiryResult;
    }
}
