/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi;

import java.util.Date;

import dk.gov.oiosi.addressing.EndpointAddress;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.communication.MailAddress;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.security.CertificateSubject;

/**
 * Response from a ARS UDDI lookup
 */
public class UddiLookupResponse {

    /**
     * Gets or sets the actual endpoint identifier of the request. This may be
     * different from what the user/client specified only if that EAN number was
     * in the gateway range configuration, and if no individual UDDI
     * registration matching the client search criteria was found on the UDDI.
     * In this case this property holds the value of the gateway identifier.
     * 
     */
    private IIdentifier endpointIdentifierActual;
    /**
     * Found endpoint address
     */
    private EndpointAddress endpointAddress;
    /**
     * Expiration time for endpoint.
     */
    private Date expirationDate;
    /**
     * Activation date for endpoint.
     */
    private Date activationDate;
    /**
     * CertificateSubject == SubjectDN from endpoint certificate
     */
    private CertificateSubject certificateSubjectSerialNumber;
    /**
     * Version.
     */
    private Version version;
    /**
     * Terms of use for endpoint.
     */
    private Uri termsOfUseUrl;
    
    /**
     * Gets the EndpointIdentifierActual
     * 
     * @return EndpointIdentifierActual
     */
    public IIdentifier getEndpointIdentifierActual() {
        return endpointIdentifierActual;
    }

    /**
     * EndpointIdentifierActual
     * 
     * @param value
     *            EndpointIdentifierActual
     * @deprecated use setEndpointIdentifierActual(IIdentifier value)
     */
    public void getEndpointIdentifierActual(IIdentifier value) {
        endpointIdentifierActual = value;
    }

    /**
     * Sets the EndpointIdentifierActual
     */
    public void setEndpointIdentifierActual(IIdentifier value) {
    	endpointIdentifierActual = value;
    }

    /**
     * Gets found endpoint address
     * 
     * @return EndpointAddress
     */
    public EndpointAddress getEndpointAddress() {
        return endpointAddress;
    }

    /**
     * Sets found endpoint address
     * 
     * @param value
     *            EndpointAddress
     */
    public void setEndpointAddress(EndpointAddress value) {
        endpointAddress = value;
    }

    /**
     * Gets expiration date of found endpoint registration
     * 
     * @return ExpirationDate
     */
    public Date getExpirationDate() {
        return expirationDate;
    }

    /**
     * Sets expiration date of found endpoint registration.
     * 
     * @param value
     *            ExpirationDate
     */
    public void setExpirationDate(Date value) {
        expirationDate = value;
    }

    /**
     * Gets activation date of found endpoint registration
     * 
     * @return ActivationDate
     */
    public Date getActivationDate() {
        return activationDate;
    }

    /**
     * Sets activation date of found endpoint registration
     * 
     * @param value
     *            ActivationDate
     */
    public void setActivationDate(Date value) {
        activationDate = value;
    }

    /**
     * Gets the certificate subject
     * 
     * @return CertificateSubject
     */
    public CertificateSubject getCertificateSubjectSerialNumber() {
        return certificateSubjectSerialNumber;
    }

    /**
     * Sets the certificate subject
     * 
     * @param value
     *            CertificateSubject
     */
    public void setCertificateSubjectSerialNumber(CertificateSubject value) {
        certificateSubjectSerialNumber = value;
    }


    /**
     * Gets or set terms of use uri's of found endpoint address
     * 
     * @return TermsOfUseUrl
     */
    public Uri getTermsOfUseUrl() {
        return termsOfUseUrl;
    }

    /**
     * Gets or set terms of use uri's of found endpoint address
     * 
     * @param value
     *            TermsOfUseUrl
     */
    public void setTermsOfUseUrl(Uri value) {
        termsOfUseUrl = value;
    }

    /**
     * 
     */
    private MailAddress serviceContactEmail;

    /**
     * Gets or set service contact e-mails of found endpoint address contacts
     * 
     * @return ServiceContactEmail
     */
    public MailAddress getServiceContactEmail() {
        return serviceContactEmail;
    }

    /**
     * Gets or set service contact emails of found endpoint address contacts
     * 
     * @param value
     *            ServiceContactEmail
     */
    public void getServiceContactEmail(MailAddress value) {
        serviceContactEmail = value;
    }

    /**
     * Gets or set version of found endpoint address
     * 
     * @return Version
     */
    public Version getVersion() {
        return version;
    }

    /**
     * Gets or set version of found endpoint address
     * 
     * @param value
     *            Version
     */
    public void setVersion(Version value) {
        version = value;
    }

    /**
     * NewerVersionReference for endpoint.
     */
    UddiId newerVersionReference;

    /**
     * Gets newer version reference on found endpoint address
     * 
     * @return newerVersionReference
     */
    public UddiId getNewerVersionReference() {
        return newerVersionReference;
    }

    /**
     * Sets newer version reference on found endpoint address
     * 
     * @param value
     *            newerVersionReference
     */
    public void setNewerVersionReference(UddiId value) {
        newerVersionReference = value;
    }

    /**
     * Gets a value indicating if there is a newer verion of found
     * endpoints
     * 
     * @return HasNewerVersion
     */
    public boolean hasNewerVersion() {
        if (newerVersionReference == null) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Default constructor.
     */
    public UddiLookupResponse() {
    }

    /**
     * Constructor.
     * 
     * @param endpointIdentifierActual
     *            The actual endpoint identifier of the request. This may be
     *            different from what the user/client specified, e.g. it may be
     *            a gateway range EAN.
     * @param endpointAddress
     *            Address of the endpoint
     * @param activationDate
     *            Activation date of the endpoint registration
     * @param expirationDate
     *            Expiration date of the endpoint registration
     * @param certificateSubjectSerialNumber
     *            Subject of the certificate
     * @param termsOfUseUrl
     *            URL to a file describing the terms of use
     * @param serviceContactEmail
     *            Email of the service contact
     * @param version
     *            Version of the endpoint
     * @param newerVersionReference
     *            Possible reference to a newer version
     */
    public UddiLookupResponse(IIdentifier endpointIdentifierActual, EndpointAddress endpointAddress, Date activationDate,
            Date expirationDate, CertificateSubject certificateSubjectSerialNumber, Uri termsOfUseUrl,
            MailAddress serviceContactEmail, Version version, UddiId newerVersionReference) {
        this.endpointIdentifierActual = endpointIdentifierActual;
        this.endpointAddress = endpointAddress;
        this.activationDate = activationDate;
        this.expirationDate = expirationDate;
        this.certificateSubjectSerialNumber = certificateSubjectSerialNumber;
        this.termsOfUseUrl = termsOfUseUrl;
        this.serviceContactEmail = serviceContactEmail;
        this.version = version;
        this.newerVersionReference = newerVersionReference;
    }

}
