/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ars;

import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.common.IdentifierUtility;
import dk.gov.oiosi.common.Version;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.IRegistrationEntity;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.ArsCategory;
import dk.gov.oiosi.uddi.category.ArsEndpointKey;
import dk.gov.oiosi.uddi.category.EndpointActivationDate;
import dk.gov.oiosi.uddi.category.EndpointAddressType;
import dk.gov.oiosi.uddi.category.EndpointCertificate;
import dk.gov.oiosi.uddi.category.EndpointContactEmail;
import dk.gov.oiosi.uddi.category.EndpointExpirationDate;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.category.NewerVersionReference;
import dk.gov.oiosi.uddi.category.OIOTaxonomy;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.category.TermsOfUseUrl;
import dk.itst.uddi.client.types.core.BusinessService;
import dk.itst.uddi.client.types.core.Description;
import dk.itst.uddi.client.types.core.KeyedReference;
import dk.itst.uddi.client.types.core.Name;

/** 
 * Represents a ARS endpoint registration, complete with:
 * a) UDDI businessService info
 * b) All relevant custom identifieres and categories
 * c) Process definition references
 */
public class ArsEndpoint extends RegistrationEntity implements IRegistrationEntity {

    private BusinessService _businessService;


    private void init() {
        _businessService = BusinessService.Factory.newInstance();
    }

    /** 
     * Default constructor. Constructs emtpy businessService with an ID
     */
    public ArsEndpoint() {
        init();
        setDefaultProperties();
    }

    /** 
     * Constructor used when initializing an existing businessservice
     * 
     * @param  service an existing businessservice
     */ 
    public ArsEndpoint(BusinessService service) {
        //try {
        _businessService = service;
        //} catch (ArgumentNullException e) {
        //    throw e;
        //}
        //catch (Exception exp) {
        //    throw new ArsEndpointUnexpectedException(exp);
        //}
    }

    /** 
     * Sets the default properties
     */
    private void setDefaultProperties() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setDefaultProperties - TODO Implement" );        

//        try {
//            CategoryBag bag = new CategoryBag();
//
//            RegistrationConformanceClaim = 
//                new RegistrationConformanceClaim(RegistrationConformanceClaimCode.oiosi1_1);
//            bag.AddCategory(RegistrationConformanceClaim.GetAsKeyedReference());
//
//            AuthenticationRequired authRequired = new AuthenticationRequired(
//                    AuthenticationRequiredCode.authenticationRequired);
//            bag.AddCategory(authRequired.GetAsKeyedReference());
//
//            ConformanceClaim confClaim =
//                new ConformanceClaim(ConformanceClaimCode.secureReliableAsyncProfile1_0);
//            bag.AddCategory(confClaim.GetAsKeyedReference());
//
//            // 2. Add bag:
//            _businessService.Value.categoryBag = bag.Value;
//        } catch (Exception exp) {
//            throw new ArsEndpointUnexpectedException(exp);
//        }
    }

    /** 
     * Constructor used when creating a new businessservice
     * 
     * @param  oioTaxonomy OIO taxonomy
     * @param  endpointContactEmail contact email for a service contact
     * @param  endpointKey endpointkey. e.g. ean key, cvr key, etc.
     * @param  endpointKeytype key type. e.g. cvr, ean, etc.
     * @param  endpointActivationDate the date the endpoint will activate
     * @param  endpointExpirationDate the date the endpoint will expire
     * @param  termsOfUseUrl the url for the 'terms of use' document
     * @param  endpointCertificate the certificate
     * @param  version Major, minor and revisions version (e.g. '1.0.0')
     * @param  newerVersionReference a reference for a newer version of the endpoint
     * @param  serviceName the name of the service
     * @param  serviceDescription the description of the service
     * @param  businessKey the key to the businessentity of the service
     * @throws ArsException 
     */ 
    public ArsEndpoint(OIOTaxonomy oioTaxonomy,
            EndpointContactEmail endpointContactEmail,
            ArsEndpointKey endpointKey,
            EndpointKeytype endpointKeytype,
            EndpointActivationDate endpointActivationDate,
            EndpointExpirationDate endpointExpirationDate,
            TermsOfUseUrl termsOfUseUrl,
            EndpointCertificate endpointCertificate,
            Version version,
            NewerVersionReference newerVersionReference,
            Name serviceName,
            Description serviceDescription,
            UddiId businessKey
    ) throws ArsException {

        init();

        //1. set serviceentity attributes, names and descriptions
        try {
            setBusinessServiceDetails(oioTaxonomy, endpointContactEmail,
                    endpointKey, endpointKeytype, endpointActivationDate, endpointExpirationDate,
                    termsOfUseUrl, endpointCertificate, version, newerVersionReference,
                    serviceName, serviceDescription, businessKey);
        } catch (Exception exp) {
            throw new ArsEndpointUnexpectedException(exp);
        }
    }


    /** 
     * Gets the underlying BusinessService object
     */
    public BusinessService getService() {
        return _businessService;
    }
    
    
    /** 
     * Adds a category to the category bag
     * 
     * @param  category The category to set
     */
    private void setCategory(ArsCategory category) {
        if (category != null) {
            setCategory(category.getAsKeyedReference());
        } 
    }

    /** 
     * Adds a category to the category bag
     * 
     * @param  uddiKeyRef A keyed reference
     */
    private void setCategory(KeyedReference uddiKeyRef) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setCategory - TODO Implement" );        

//        KeyedReference keyRef = new KeyedReference(uddiKeyRef);
//        if (keyRef != null) {
//            if (_businessService != null) {
//                if (_businessService.CategoryBag == null) {
//                    _businessService.CategoryBag = new CategoryBag(keyRef);
//                } else {
//                    _businessService.CategoryBag.SetCategory(keyRef);
//                }
//            } else {
//                return;
//            }
//        } 
    }


    /** 
     * Gets the registration conformance claim regarding Uddi registration model
     */ 
    public RegistrationConformanceClaim getRegistrationConformanceClaim() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - getRegistrationConformanceClaim - TODO Implement" );        
        else return null;
        
//        if (_businessService != null && _businessService.CategoryBag != null) {
//            RegistrationConformanceClaim regClaim = new RegistrationConformanceClaim();
//
//            KeyedReference keyRef = _businessService.CategoryBag.GetCategoryByIdentifier(regClaim.CategoryID);
//            if (keyRef != null) {
//                regClaim.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//                return regClaim;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    
    public void setRegistrationConformanceClaim (RegistrationConformanceClaim value){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setRegistrationConformanceClaim - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new RegistrationConformanceClaim().CategoryID);
//        }
    }

    /** 
     * Gets the identifier (guid) of the businessservice
     */ 
    public UddiId getID() {
        if (_businessService != null && _businessService.getServiceKey()!= null && _businessService.getServiceKey().length() > 0) {
            return new UddiGuidId(_businessService.getServiceKey(), true);
        } else {
            return null;
        }
    }
    
    public void setID(UddiId value) {
        try {
            if (value != null) {
                _businessService.setServiceKey(value.getID());
            } else {
                _businessService.setServiceKey(null);
            }
        } catch (Exception exp) {
            // throw new ArsEndpointUnexpectedException(exp);
        }
    }

    /** 
     * Gets or sets bindings
     */
    public ArsBindingSet getBindings() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - getBindings - TODO Implement" );        
        else return null;

//        if (_businessService != null) {
//            if (_businessService.BindingTemplates != null) {
//                ArsBindingSet bindingSet = 
//                    new ArsBindingSet(_businessService.BindingTemplates.GetBindingTemplateCollection());
//                return bindingSet;
//            } else {
//                return null;
//            }
//        } else {
//            return null;
//        }
    }
    
    public void setBindings(ArsBindingSet value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setBindings - TODO Implement" );        

//        if (value != null) {
//            _businessService.BindingTemplates = value.GetBindingTemplates();
//        } else {
//            _businessService.BindingTemplates = null;
//        }
    }

    /** 
     * Gets or sets a (UDDI serviceKey) reference to a newer version of the 
     * endpoint.
     * 
     */ 
    public NewerVersionReference getNewerVersionReference() {
        NewerVersionReference newVersionRef = new NewerVersionReference();
        return (NewerVersionReference)getCategory(newVersionRef);
    }
    public void setNewerVersionReference (NewerVersionReference value){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setNewerVersionReference - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new NewerVersionReference().CategoryID);
//        }
    }


    /** 
     * Gets or sets the version of the endpoint (major, minor, revision)
     * Note that 'revision' is mapped to the 'build' field of the 
     * .NET builtin 'System.Version' class.
     * 
     */ 
    public Version getVersion() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - getVersion - TODO Implement" );        
        else return null;
        
//        // 1. Get the individual version parts:
//        Version v = null;
//        try {
//            VersionMajor major = new VersionMajor();
//            major = (VersionMajor)GetCategory(major);
//            if (major == null) return null;
//            
//            VersionMinor minor = new VersionMinor();
//            minor = (VersionMinor)GetCategory(minor);
//            
//            VersionRevision rev = new VersionRevision();
//            rev = (VersionRevision)GetCategory(rev);
//            
//            // 2. Create the version number
//            int majorInt = 0;
//            int minorInt = 0;
//            int revisionInt = 0;
//            
//            majorInt = Int32.Parse(major.Value);
//            if (minor != null) minorInt = Int32.Parse(minor.Value);
//            if (rev != null) revisionInt = Int32.Parse(rev.Value);
//            
//            v = new Version(majorInt, minorInt, revisionInt);
//        } catch (Exception exp) {
//            throw new ArsEndpointUnexpectedException(exp);
//        }
//        return v;
    }
    
    public void setVersion(Version value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setVersion - TODO Implement" );        

//        if (value != null) {
//            setCategory(new VersionMajor(value.Major.ToString()));
//            setCategory(new VersionMinor(value.Minor.ToString()));
//            setCategory(new VersionRevision(value.Build.ToString()));
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new VersionMajor().CategoryID);
//            _businessService.CategoryBag.RemoveCategoryById(new VersionMinor().CategoryID);
//            _businessService.CategoryBag.RemoveCategoryById(new VersionRevision().CategoryID);
//        }
    }
        

    /** 
     * Gets or sets the subject string of the endpoint certificate
     */
    public EndpointCertificate getCertificateSubject() {
        EndpointCertificate cert = new EndpointCertificate();
        return (EndpointCertificate)getCategory(cert);
    }

    /** 
     * Gets or sets the subject string of the endpoint certificate
     */
    public void setCertificateSubject(EndpointCertificate value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setCertificateSubject - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new EndpointCertificate().CategoryID);
//        }
    }


    /** 
     * Gets or sets the url to a document with terms of use
     */
    public TermsOfUseUrl getTermsOfUseUrl() {
        TermsOfUseUrl terms = new TermsOfUseUrl();
        return (TermsOfUseUrl)getCategory(terms);
    }
    
    /** 
     * Gets or sets the url to a document with terms of use
     */
    public void setTermsOfUseUrl( TermsOfUseUrl value ){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setTermsOfUseUrl - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new TermsOfUseUrl().CategoryID);
//        }
    }


    /** 
     * Gets or sets the endpoint activation date
     */
    public EndpointActivationDate getEndpointActivationDate() {
        EndpointActivationDate activationDate = new EndpointActivationDate();
        return (EndpointActivationDate)getCategory(activationDate);
    }

    /** 
     * Gets or sets the endpoint activation date
     */
    public void setEndpointActivationDate(  EndpointActivationDate value){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setEndpointActivationDate - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new EndpointActivationDate().CategoryID);
//        }
    }


    /** 
     * Gets or sets the endpoint expiration date
     */
    public EndpointExpirationDate getEndpointExpirationDate() {
        EndpointExpirationDate expirationDate = new EndpointExpirationDate();
        return (EndpointExpirationDate)getCategory(expirationDate);
    }

    /** 
     * Gets or sets the endpoint expiration date
     */
    public void setEndpointExpirationDate( EndpointExpirationDate value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setEndpointExpirationDate - TODO Implement" );        

//        if (value != null) {
//            setcategory(value);
//        } else {
//            _businessservice.categorybag.removecategorybyid(new endpointexpirationdate().categoryid);
//        }
    }


    /** 
     * Gets or sets the endpoint address type
     */
    public EndpointAddressType getEndpointAddressType() {
        EndpointAddressType endpointAddressType = new EndpointAddressType();
        return (EndpointAddressType)getCategory(endpointAddressType);
    }

    /** 
     * Gets or sets the endpoint address type
     */
    public void setEndpointAddressType( EndpointAddressType value){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setEndpointAddressType - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new EndpointAddressType().CategoryID);
//        }
    }


    /** 
     * Gets or sets the endpoint key type
     */
    public EndpointKeytype getEndpointKeytype() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - getEndpointKeytype - TODO Implement" );        
        else return null;
        
//        EndpointKeytype keyType = new EndpointKeytype();
//        return (EndpointKeytype)getCategory(keyType);
    }

    /** 
     * Gets or sets the endpoint key type
     */
    public void setEndpointKeytype (EndpointKeytype value){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setEndpointKeytype - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new EndpointKeytype().CategoryID);
//        }
    }


    /** 
     * Gets or sets the endpoint key
     */
    public IIdentifier getEndpointKey() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setEndpointKey - TODO Implement" );        
        else return null;
        
//        ArsEndpointKey endpointKey = new ArsEndpointKey();
//        KeyedReference keyRef =_businessService.CategoryBag.GetCategoryByIdentifier(endpointKey.CategoryID);
//        if (keyRef != null) {
//            return IdentifierUtility.GetIdentifierFromKeyType(keyRef.KeyValue,
//                    this.EndpointKeyType.GetEndpointKeyTypeCode());
//        } else {
//            return null;
//        }
    }

    /** 
     * Gets or sets the endpoint key
     */
    public void setEndpointKey(IIdentifier value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setEndpointKeytype - TODO Implement" );        

//        if (value != null) {
//            ArsEndpointKey key = new ArsEndpointKey(value);
//            setCategory(key);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new ArsEndpointKey().CategoryID);
//        }
    }


    /** 
     * Email of the endpoint contact. Returns null if this property has 
     * not been set.
     */
    public EndpointContactEmail getContactEmail() {
        EndpointContactEmail contactEmail = new EndpointContactEmail();
        return (EndpointContactEmail)getCategory(contactEmail);
    }

    /** 
     * Email of the endpoint contact. Returns null if this property has 
     * not been set.
     */
    public void setContactEmail(EndpointContactEmail value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setContactEmail - TODO Implement" );        

//        if (value != null) {
//            setCategory(value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new EndpointContactEmail().CategoryID);
//        }
    }



    /** 
     * The OIO taxonomy. Null if none is defined.
     */ 
    public OIOTaxonomy getTaxonomy() {
        OIOTaxonomy tax = new OIOTaxonomy();
        return (OIOTaxonomy)getCategory(tax);
    }

    /** 
     * The OIO taxonomy. Null if none is defined.
     */ 
    public void setTaxonomy(OIOTaxonomy value) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setTaxonomy - TODO Implement" );        

//        if (value != null){
//            setCategory((OIOTaxonomy) value);
//        } else {
//            _businessService.CategoryBag.RemoveCategoryById(new OIOTaxonomy().CategoryID);
//        }
    }


    /** 
     * The name of the endpoint
     */ 
    public Name getName() {
        if (_businessService != null && _businessService.getNameArray() != null) {
            return _businessService.getNameArray(0);
        } else {
            return null;
        }
    }

    /** 
     * The name of the endpoint
     */ 
    public void setName( Name value) {
        if (_businessService != null) {
            _businessService.setNameArray(new Name[]{value});
        }
    }


    /** 
     * Description of the endpoint
     */ 
    public Description getDescription() {
        if (_businessService != null && _businessService.getDescriptionArray() != null) {
            return _businessService.getDescriptionArray(0);
        } else {
            return null;
        }
    }

    /** 
     * Description of the endpoint
     */ 
    public void setDescription (Description value) {
        _businessService.setDescriptionArray( new Description[] { value });
    }



    /** 
     * Gets or sets the business key
     */
    public UddiId getBusinessKey() {
        if (_businessService != null) {
            return IdentifierUtility.getUddiIDFromString(_businessService.getBusinessKey());
        } else {
            return null;
        }
    }

    /** 
     * Gets or sets the business key
     */
    public void setBusinessKey(UddiId value) {
        if (value != null) {
            _businessService.setBusinessKey(value.getID());
        } else {
            _businessService.setBusinessKey(null);
        }
    }


    private void saveServiceEntity() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - saveServiceEntity - TODO Implement" );        

//        //1. add the bindingtemplate to the service
//        /*_businessService.Value.bindingTemplates = 
//          _bindingSet.GetBindingTemplates().GetBindingTemplateCollection();*/
//
//        //2. add the service to an array
//        BusinessService[] services = new BusinessService[1];
//        services[0] = _businessService;
//
//        try {
//            //3. call Publication
//            Publication uddiPublicationProxy = new Publication(Connection);
//            serviceDetail detail = uddiPublicationProxy.Save(services);
//            _businessService = new BusinessService(detail.businessService[0]);
//        } catch (SaveServiceException) {
//            throw;
//        } catch (PublicationUnexpectedException) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsEndpointUnexpectedException(exp);
//        }
    }

        

    /** 
     * Constructs an ArsEndpoint object from a business service ID. 
     * 
     * @param  businessServiceId business service uuid
     * @param  getRecursively If getRecursively = true, all sub entities are loaded as well
     * @return 
     */ 
    public static ArsEndpoint get(UddiId businessServiceId, boolean getRecursively){
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - get - TODO Implement" );        
        else return null;

//        //1. create a temp serviceentity
//        ArsEndpoint tempBusinessService = null;
//
//        //2. create a uddi inquire instance
//        Inquiry inq = new Inquiry();
//
//        //3. call getbusinessdetail
//        BusinessService[] entities;
//        try {
//            GetServiceDetail detail = new GetServiceDetail(businessServiceId.ID);
//            entities = inq.GetDetail(detail.Value);
//
//            if (entities.Length == 1) {
//                tempBusinessService = new ArsEndpoint(entities[0]);
//            }
//        } catch (GetServiceDetailException) {
//            throw;
//        } catch (InquiryUnexpectedException) {
//            throw;
//        } catch (Exception exp) {
//            throw new ArsEndpointUnexpectedException(exp);
//        }
//
//        if (getRecursively) {
//
//            //1. add bindings                
//            if (tempBusinessService._businessService.Value.bindingTemplates.Length > 0) {
//                tempBusinessService.Bindings = new ArsBindingSet(tempBusinessService._businessService.Value.bindingTemplates);
//            }
//        }
//
//        return tempBusinessService;
    }

    /** 
     * This method is used to gather all attributes and call there relative set methods
     */
    private void setBusinessServiceDetails(
            OIOTaxonomy oioTaxonomy, 
            EndpointContactEmail endpointContactEmail, 
            ArsEndpointKey endpointKey,
            EndpointKeytype endpointKeytype, 
            EndpointActivationDate endpointActivationDate,
            EndpointExpirationDate endpointExpirationDate,
            TermsOfUseUrl termsOfUseUrl, 
            EndpointCertificate endpointCertificate, 
            Version version, 
            NewerVersionReference newerVersionReference,
            Name serviceName, 
            Description serviceDescription, 
            UddiId businessKey
    ) {

        //1. set custom attributes
        setCustomAttributes(oioTaxonomy, endpointContactEmail,
                endpointKey, endpointKeytype, endpointActivationDate, endpointExpirationDate,
                termsOfUseUrl, endpointCertificate, version, newerVersionReference);

        //2. set names
        setName( serviceName);

        //3. description
        setDescription(serviceDescription);

        //4. set businesskey
        setBusinessKey(businessKey);
    }        

    /** 
     * Set all custom categories
     */
    private void setCustomAttributes( 
            OIOTaxonomy oioTaxonomy, 
            EndpointContactEmail endpointContactEmail, 
            ArsEndpointKey endpointKey,
            EndpointKeytype endpointKeytype, 
            EndpointActivationDate endpointActivationDate,
            EndpointExpirationDate endpointExpirationDate, 
            TermsOfUseUrl termsOfUseUrl, 
            EndpointCertificate endpointCertificate, 
            Version version, 
            NewerVersionReference newerVersionReference
    ) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - setCustomAttributes - TODO Implement" );        
            
//        //1. Reset categorybag:
//        _businessService.Value.categoryBag = null;
//        CategoryBag bag = new CategoryBag();
//            
//        AuthenticationRequired authRequired = new AuthenticationRequired(
//                AuthenticationRequiredCode.authenticationRequired);
//        bag.AddCategory(authRequired.GetAsKeyedReference());
//        ConformanceClaim confClaim =
//            new ConformanceClaim(ConformanceClaimCode.secureReliableAsyncProfile1_0);
//        bag.AddCategory(confClaim.GetAsKeyedReference());
//
//        // 2. Add bag:
//        _businessService.Value.categoryBag = bag.Value;
//
//        // 3. Add predefined categories:
//        setDefaultProperties();
//            
//        // 4. Set other custom properties:
//        setTaxonomy(oioTaxonomy);
//        setContactEmail(endpointContactEmail);
//
//        // 5. Set endpointkey, based on endpointkeytype
//        switch (endpointKeytype.GetEndpointKeyTypeCode()) {
//
//            case EndpointKeyTypeCode.ean:
//                EndpointKey = new IdentifierEan(endpointKey.Value);
//                break;
//            case EndpointKeyTypeCode.ovt:
//                EndpointKey = new IdentifierOvt(endpointKey.Value);
//                break;
//            case EndpointKeyTypeCode.cvr:
//                EndpointKey = new IdentifierCvr(endpointKey.Value);
//                break;
//            case EndpointKeyTypeCode.other:
//                throw new ArgumentException("Unsupported endpointkeytype");
//            default:
//                throw new ArgumentException("Unsupported endpointkeytype");
//        }
//
//        setEndpointKeyType(endpointKeytype);
//        setEndpointActivationDate(endpointActivationDate);
//        setEndpointExpirationDate(endpointExpirationDate);
//        setTermsOfUseUrl(termsOfUseUrl);
//        setCertificateSubject(endpointCertificate);
//        setNewerVersionReference(newerVersionReference);
//        setVersion(version);
    }


    /** 
     * Gets an ARS category by the category ID of the 'categoryObject' 
     * parameter. Note that we need an actual ArsCategory descendant 
     * object to get the category identifier. If no match is found, null is returned.
     * 
     * @param  category The category object from which to get the category ID.
     * Another object, with key and value set, is returned as a result.
     * @return  
     */
    private ArsCategory getCategory(ArsCategory category) {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - getCategory - TODO Implement" );        
        else return null;

//        if (category == null) return null;
//        CategoryBag bag = new CategoryBag(_businessService.Value.categoryBag);
//        KeyedReference keyRef = bag.GetCategoryByIdentifier(category.CategoryID);
//        if (keyRef == null) return null;
//        //category.SetCategoryValue(category.CategoryName /*keyRef.KeyName*/, keyRef.KeyValue);
//        category.SetCategoryValue(keyRef.KeyName, keyRef.KeyValue);
//        return category;
    }


    /** 
     * Saves the entity
     */
    public void save() throws MainException {
        // 1. Verify properties
        validate();

        // 2. Save
        try {
            saveServiceEntity();
        } catch (Exception exp) {
            throw new ArsEndpointUnexpectedException(exp);
        }
    }

    /** 
     * Validates the entity
     */
    public void validate() {
    }

    /** 
     * Updates the entity
     * @throws MainException 
     */
    public void update() throws MainException {
        // 1. Verify properties
        validate();

        // 2. Update in uddi
        try {
            saveServiceEntity();
        } catch (Exception exp) {
            throw new ArsEndpointUnexpectedException(exp);
        }
    }

    /** 
     * Deletes all underlying UDDI entities associated with this endpoint
     */
    public void delete() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - delete - TODO Implement" );        
        
//        //1. check that an id exists
//        if (_businessService.Value.serviceKey.Length > 0) {
//            
//            //2. delete related businessprocessinstances
//            deleteRelatedBusinessProcessInstances();                
//
//            String[] temp = new String[1];
//            temp[0] = _businessService.Value.serviceKey;
//
//            //2. call Publication.Delete
//            try {
//                Publication uddiPublicationProxy = new Publication(Connection);
//                uddiPublicationProxy.DeleteService(temp);
//                _businessService = null; 
//            } catch (DeleteServiceException) {
//                throw;
//            } catch (PublicationUnexpectedException) {
//                throw;
//            } catch (Exception exp) {
//                throw new ArsEndpointUnexpectedException(exp);
//            }
//        } else {
//            throw new Exception("No service key was found");
//        }
    }

    /** 
     * Find and deletes all business process instances related to the endpoint
     */
    public void deleteRelatedBusinessProcessInstances() {
        if( true ) throw new NoSuchMethodError( "ArsEndpoint - deleteRelatedBusinessProcessInstances - TODO Implement" );        

//        List<string> tmodelsToDelete = new List<string>();            
//        Inquiry inq = new Inquiry();
//
//        try {
//            //1. run through all bindingtemplates
//            if (_businessService.BindingTemplates != null) {
//                foreach (BindingTemplate template in _businessService.BindingTemplates.Value) {
//                    //1a. run through all tmodelinstanceinfoes
//                    foreach (TModelInstanceInfo info in template.TModelInstanceInfos) {
//                        //1b. get each tmodel
//                        GetTModelDetail detail = new GetTModelDetail(info.Value.tModelKey);
//                        TModel[] tmodel = inq.GetDetail(detail.Value);
//
//                        //1c. run through all categories and see if it is a businessprocessinstance
//                        if (tmodel != null && tmodel.Length > 0) {
//                            KeyedReference keyref = tmodel[0].CategoryBag.GetCategoryByName(new BusinessProcessDefinitionReference().CategoryName);
//
//                            if (keyref != null) {
//                                //1d. add tmodelkey to the list of tmodels to delete
//                                tmodelsToDelete.Add(tmodel[0].Value.tModelKey);
//                            }
//                        }
//                    }
//                }
//            }
//            //2. delete the found tmodels
//            String[] tmodelkeys = new String[tmodelsToDelete.Count];
//            if (tmodelkeys.Length > 0) {
//                tmodelsToDelete.CopyTo(tmodelkeys);
//                Publication uddiPublicationProxy = new Publication(Connection);
//                uddiPublicationProxy.DeleteTModel(tmodelkeys);
//            }
//        } catch (Exception exp) {
//            throw new ArsEndpointUnexpectedException(exp);
//        }
    }

    /** 
     * Validates the embedded data, and eventually returns a structured report if validations fails
     * 
     * @param  EntityName 
     * @param  Failures 
     * @return 
     */
    public boolean isValid(String EntityName) { // , ref dk.gov.oiosi.uddi.Validation.ValidationFailureCollection Failures) {
//        ValidationFailureCollection ChildFailures = null;
//
//        BusinessService.IsValid(_businessService, "_business", ref ChildFailures);
//
//        if (Bindings == null) {
//            DataValidationFailure.AddFailure(MissingEndpointBindingFailure.Message(), EntityName, this.GetType(), ref Failures);
//                
//            if (ChildFailures != null)
//                ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
//                        ChildFailures, ref Failures);
//
//            return Failures == null;
//        }
//        Bindings.IsValid("_bindingSet", ref ChildFailures);
//        if (Bindings.Bindings.Count == 0)
//            DataValidationFailure.AddFailure(MissingEndpointBindingFailure.Message(), EntityName, Bindings.GetType(),
//                    ref Failures);
//
//        if (ChildFailures != null)
//            ChildValidationFailure.AddFailure(ChildFailure.Message(), EntityName, this.GetType(),
//                    ChildFailures, ref Failures);
//
//        return Failures == null;

        boolean valid = _businessService.validate();
        if( ! valid ) {
            return false;
        }
        valid = getBindings().isValid("_bindingSet");
        if( ! valid ) {
            return false;
        }
        return true;
    }

    
}
