/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.common.ValUtil;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.itst.uddi.client.types.core.KeyedReference;

/**
 * Base class for all ARS UDDI categories. Holds a list of categoris and possible values for each category. NOTE: All static fields
 * MUST BE set in the static constructor of the descendant class.
 */
public abstract class ArsCategory {

    private static Log log = LogFactory.getLog(ArsCategory.class);

    /**
     * Static list of possible category names. List of String.
     */
    protected static List pCategoryNames = new ArrayList();

    /**
     * Static index of list of possible values for a specific category name.
     * 
     * Maps from String to List of Strings
     */
    protected static Map pCategoryValues = new Hashtable();

    /**
     * The category key.
     */
    protected String pKey;

    /**
     * The category value.
     */
    protected String pValue;

    /**
     * Default constructor.
     */
    protected ArsCategory() {
        // 1. Set default key / value
        pKey = getDefaultCategory();
        pValue = getDefaultCategoryValue();
    }

    /**
     * Sets the category name and value.
     * 
     * @param category
     *            The category name
     * @param value
     *            The category value
     */
    public void setCategoryValue(String category, String value) throws Exception {
        if (category == null) {
            throw new NullArgumentException("category");
        }
        if (value == null) {
            throw new NullArgumentException("value");
        }

        if (!pCategoryNames.contains(category)) {
            throw new Exception("Category " + category + " does not exist in the category list. ");
        }
        pKey = category;

        List possibleValues = (ArrayList) pCategoryValues.get(category);
        if (possibleValues.size() > 0 && !possibleValues.contains(value)) {
            throw new Exception("Category '" + category + "' cannot take the value '" + value + "'. ");
        }
        pValue = value;
    }

    /**
     * Sets a catgory/value pair in the static category list to the selected value.
     * 
     * @param categoryAndValue
     *            The value to set as category name/value
     */
    protected static void setCategoryAndValues(String categoryAndValue) {
        ArrayList list = new ArrayList();
        list.add(categoryAndValue);
        setCategoryAndValues(categoryAndValue, list);
    }

    /**
     * Associates a list of possible values with a single category.
     * 
     * @param category
     *            The category
     * @param possibleValues
     *            The possible values for the category
     */
    protected static void setCategoryAndValues(String category, String[] possibleValues) {
        if (possibleValues == null) {
            setCategoryAndValues(category, new ArrayList());
        } else {
            ArrayList list = new ArrayList();
            for (int i = 0; i < possibleValues.length; i++) {
                list.add(possibleValues[i]);
            }
            setCategoryAndValues(category, new ArrayList());
        }
    }

    /**
     * Associates a list of possible values with a single category.
     * 
     * @param category
     *            The category
     * @param possibleValues
     *            The possible values for the category
     */
    protected static void setCategoryAndValues(String category, ArrayList possibleValues) {
        pCategoryNames.add(category);
        pCategoryValues.put(category, possibleValues);
    }

    /**
     * Gets the category name ('key' in UDDI terminology)
     * 
     * @return catagory name
     */
    public String getCategory() {
        return pKey;
    }

    /**
     * Gets the category value.
     * 
     * @return catagory value
     */
    public String getValue() {
        return pValue;
    }

    /**
     * Sets the category value.
     * 
     * @param value
     *            catagory value
     */
    public void setValue(String value) {
        pValue = value;
    }

    /**
     * Gets the category identifier (uuid).
     * 
     * @return catagory identifier
     */
    public abstract String getCategoryID();

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public abstract String getCategoryName();

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public abstract String getDefaultCategory();

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public abstract String getDefaultCategoryValue();

    /**
     * Returns a list of possible values.
     * 
     * @return Map of category values
     */
    public Map getCategoryValues() {
        return pCategoryValues;
    }

    /**
     * Returns the category as a UDDI keyedreference entity.
     * 
     * @return Return the value as a keyed reference
     */
    public KeyedReference getAsKeyedReference() {
        KeyedReference keyRef = KeyedReference.Factory.newInstance();
        keyRef.setTModelKey(getCategoryID());
        keyRef.setKeyName(pKey);
        keyRef.setKeyValue(pValue);
        return keyRef;
    }

    /**
     * Validate Value within the categorylist idenfied by Category.
     * 
     * @param ValueName
     * @param Required
     * @return true if value
     */
    public boolean isValid(String ValueName, boolean Required) {
        boolean Valid = true;

        if (!ValUtil.isNullOrEmpty(getValue())) {
            List possibleValues = (List) pCategoryValues.get(getDefaultCategory());
            if (!possibleValues.contains(getValue())) {
                // DataValidationFailure.AddFailure(InvalidCategoryValueFailure.Message(Value),
                // ValueName, typeof(ArsCategory), ref Failures);
                log.error("InvalidCategoryValueFailure.Message(" + getValue() + " : " + ValueName
                        + ", typeof(ArsCategory), ref Failures)");
                Valid = false;
            }
        } else if (Required) {
            // If field is required then it must not be null

            log.error("RequiredFieldFailure.Message(ValueName), " + ValueName + ", typeof(ArsCategory), ref Failures)");
            Valid = false;
        }

        return Valid;
    }

    /**
     * Validate Value within the categorylist idenfied by Category.
     * 
     * @param ValueName
     * @param Failures
     * @return
     */
    public boolean isValid(String ValueName) {
        return isValid(ValueName, false);
    }

    /**
     * Compares the two objects and returns true if they have equal values.
     * 
     * @param other
     *            The object to compare to
     * @return Returns true if the two objects have identical values
     */
    public boolean equals(ArsCategory other) {
        if (other == null)
            return false;

        if (!getCategory().equals(other.getCategory())) {
            return false;
        }
        if (!getValue().equals(other.getValue())) {
            return false;
        }
        return true;
    }

}
