/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.category;

import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;

/**
 * Indicates the type of the AccessPoint entity, e.g. http, email or other type. NOTE: the value of this category should probably be
 * constrained to match a possible reference to a binding tModel, so that e.g. a http binding is not implemented with an email
 * endpoint.
 */
public class EndpointAddressType extends ArsCategory {

    /**
     * Static constructor. Sets list of categories and possible values for each.
     */
    static {
        // 1. Set baseclass default values:
        categoryId = new UddiGuidId("uddi:14248882-f226-4caa-92ed-0a2ec40d3112", true);
        categoryName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/endpointAddressType/";
        defaultKeyName = "http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/endpointAddressType/";
        defaultKeyValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/http/";

        // 2. Set list of categories & possible values for each category
        String[] values = {"http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/http/",
                "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/https/",
                "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/email/",
                "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/ftp/",
                "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/other/"};

        // Indicates an http address
        setCategoryAndValues("http://oio.dk/profiles/OIOSI/1.0/UDDI/Categories/endpointAddressType/", values);
    }

    /**
     * Default constructor. Sets to default.
     */
    public EndpointAddressType() {
    }

    /**
     * Use this constructor to set a value.
     * 
     * @param endpointAddressType
     *            endpoint addresstype
     */
    public EndpointAddressType(EndpointAddressTypeCode endpointAddressType) {
        switch (endpointAddressType.getCode()) {
        case EndpointAddressTypeCode.http:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/http/";
            break;
        case EndpointAddressTypeCode.https:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/https/";
            break;
        case EndpointAddressTypeCode.email:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/email/";
            break;
        case EndpointAddressTypeCode.ftp:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/ftp/";
            break;
        case EndpointAddressTypeCode.other:
            pValue = "http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/other/";
            break;
        default:
            pValue = "";
            break;
        }
    }

    /**
     * From an endpoint address, gets the UDDI key value code representing it.
     * 
     * @return
     */
    public EndpointAddressTypeCode toCode() {
        if (pValue.equals("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/http/")) {
            return new EndpointAddressTypeCode(EndpointAddressTypeCode.http);
        }
        if (pValue.equals("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/https/")) {
            return new EndpointAddressTypeCode(EndpointAddressTypeCode.https);
        }
        if (pValue.equals("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/email/")) {
            return new EndpointAddressTypeCode(EndpointAddressTypeCode.email);
        }
        if (pValue.equals("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/ftp/")) {
            return new EndpointAddressTypeCode(EndpointAddressTypeCode.ftp);
        }
        if (pValue.equals("http://oio.dk/profiles/OWSA/modelT/1.0/UDDI/Identifiers/other/")) {
            return new EndpointAddressTypeCode(EndpointAddressTypeCode.other);
        }

        return new EndpointAddressTypeCode(EndpointAddressTypeCode.other);
    }

    // Start ArsCategory abstract members.

    /**
     * category id.
     */
    private static UddiId categoryId;

    /**
     * Gets the category identifier (uuid).
     * 
     * @return category id
     */
    public String getCategoryID() {
        return categoryId.toString();
    }

    /**
     * category name.
     */
    private static String categoryName;

    /**
     * Gets the category name.
     * 
     * @return category name
     */
    public String getCategoryName() {
        return categoryName;
    }

    /**
     * default key name.
     */
    private static String defaultKeyName;

    /**
     * Gets the default category name.
     * 
     * @return default category name
     */
    public String getDefaultCategory() {
        return defaultKeyName;
    }

    /**
     * default key value.
     */
    private static String defaultKeyValue;

    /**
     * Gets the default category value.
     * 
     * @return default category value
     */
    public String getDefaultCategoryValue() {
        return defaultKeyValue;
    }

    // End ArsCategory abstract members.

}
