/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.identifier;

import java.util.List;

import dk.itst.uddi.client.types.core.KeyedReference;

import dk.gov.oiosi.common.ValUtil;

/**
 * Base class for all ARS UDDI identifiers. May hold a list of possible values
 * for the identifier. May hold a default value. NOTE: All static fields MUST BE
 * set in the static constructor of the descendant class.
 * 
 * TODO: add a method that returns the data of this class as a UDDI entity.
 * 
 */
public abstract class ArsIdentifier {

    /**
     * Internal Value value
     * 
     */
    protected String pValue;

    /**
     * Default constructor
     * 
     */
    protected ArsIdentifier() {
        // 1. Set default value
        pValue = getDefaultValue();
    }

    /**
     * Gets or sets the value of the identifier
     * 
     * @return value of identifier
     */
    public String getValue() {
        return pValue;
    }

    /**
     * Gets or sets the value of the identifier
     * 
     * @param value
     *            of identifier
     */
    public void setValue(String value) {
        pValue = value;
    }

    /**
     * Gets the UDDI name of the identifier
     * 
     * @return uddi name of identifier
     */
    public abstract String getIdentifierName();

    /**
     * Gets the UDDI tModel id of the identifier
     * 
     * @return uddi name of identifier
     */
    public abstract String getIdentifierID();

    /**
     * True if a default value exists.
     * 
     * @return true if default values exits
     */
    public abstract boolean getDefaultValueExists();

    /**
     * True if a predefined list of possible values exist for this identifier.
     * 
     * @return true if fixed values exits
     */
    public abstract boolean getFixedValueListExists();

    /**
     * Returns the default value of this identifier if any exist, else returns
     * null. You may check "DefaultValueExists" before calling.
     * 
     * @return default value
     */
    public abstract String getDefaultValue();

    /**
     * Returns a list of possible values
     * 
     * @return list of possible values
     */
    public abstract List getPossibleValues();

    /**
     * Returns the category as a UDDI keyedreference entity
     * 
     * @return Returns the category as a UDDI keyedreference entity
     */
    public KeyedReference getAsKeyedReference() {

        KeyedReference keyRef = KeyedReference.Factory.newInstance();
        keyRef.setTModelKey(getIdentifierID());
        keyRef.setKeyName(getIdentifierName());
        keyRef.setKeyValue(getValue());

        return keyRef;
    }

    /**
     * Validate Value within the categorylist idenfied by Category
     * 
     * @param ValueName
     * @param Required
     * @param Failures
     * @return
     */
    // public bool IsValid(string ValueName, bool Required,
    // ref ValidationFailureCollection Failures) {
    public boolean isValid(String valueName, boolean required) {
        boolean valid = true;
        if (!ValUtil.isNullOrEmpty(getValue())) {

            valid = true;
            if (!valid) {
                // DataValidationFailure.AddFailure(InvalidIdentifierValueFailure.Message(Value),
                // ValueName, typeof(ArsIdentifier), ref Failures);
                valid = false;
            }
        } else {
            // If field is required then it must not be null
            if (required) {
                // DataValidationFailure.AddFailure(
                // RequiredFieldFailure.Message(ValueName), ValueName,
                // typeof(ArsIdentifier), ref Failures);
                valid = false;
            }
        }
        return valid;
    }

    /**
     * Validate Value within the categorylist idenfied by Category
     * 
     * @param ValueName
     * @param Failures
     * @return
     */
    public boolean isValid(String valueName) {
        return isValid(valueName, false);
    }

}
