/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.uddi.ranges;

import dk.gov.oiosi.common.ValUtil;

/**
 * Configuration of gateway endpoint key ranges, i.e. a range of keys that a specific gateway handles.
 */
public class GatewayRange {

    /**
     * range start (ean)number.
     */
    private String rangeStart = "";

    /**
     * range end (ean)number.
     */
    private String rangeEnd = "";

    /**
     * parameter to return when in range.
     */
    private GatewayRegistrationParameters registrationParameters = null;

    /**
     * Default constructor.
     */
    public GatewayRange() {
    }

    /**
     * Checks if an EAN number is within the range.
     * 
     * @param eanNumber
     *            EAN number to check
     * @return True if the EAN number is within the range
     */
    public boolean isInRange(String eanNumber) {
        long ean = Long.parseLong(eanNumber);
        if (ean >= getRangeStartInt() && ean <= getRangeEndInt()) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Gets or sets first number of the range, inclusive.
     * 
     * @return range start
     */
    public String getRangeStart() {
        return rangeStart;
    }

    /**
     * Gets or sets first number of the range, inclusive.
     * 
     * @param value
     *            range start
     */
    public void setRangeStart(String value) {
        rangeStart = value;
    }

    /**
     * Gets or sets last number of the range, inclusive.
     * 
     * @return range end
     */
    public String getRangeEnd() {
        return rangeEnd;
    }

    /**
     * Gets or sets last number of the range, inclusive.
     * 
     * @param value
     *            range end
     */
    public void setRangeEnd(String value) {
        rangeEnd = value;
    }

    /**
     * First number of the range, inclusive, as a long integer. In case of conversion errors, an exception is thrown. In case of an
     * empty setting in config, 0 is returned.
     * 
     * @return range start as int
     */
    public long getRangeStartInt() {
        if (ValUtil.isNullOrEmpty(rangeStart)) {
            return 0;
        } else {
            return Long.parseLong(rangeStart);
        }
    }

    /**
     * Last number of the range, inclusive, as a long integer. In case of conversion errors, an exception is thrown. In case of an
     * empty setting in config, 0 is returned.
     * 
     * @return range end as int
     */
    public long getRangeEndInt() {
        if (ValUtil.isNullOrEmpty(rangeEnd)) {
            return 0;
        } else {
            return Long.parseLong(rangeEnd);
        }
    }

    /**
     * The parameters used to look up the gateway registration in UDDI, when combined with the parameters given by the user/client
     * requesting the UDDI lookup
     * 
     * @return gateway paramters
     */
    public GatewayRegistrationParameters getGatewayRegistrationParameters() {
        return registrationParameters;
    }

    /**
     * The parameters used to look up the gateway registration in UDDI, when combined with the parameters given by the user/client
     * requesting the UDDI lookup
     * 
     * @param value
     *            gateway parameters
     */
    public void setGatewayRegistrationParameters(GatewayRegistrationParameters value) {
        registrationParameters = value;
    }
}
