/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is .NET RASP toolkit.
 *
 * The Initial Developer of the Original Code is Accenture and Avanade.
 * Portions created by Accenture and Avanade are Copyright (C) 2007
 * Danish National IT and Telecom Agency (http://www.itst.dk). 
 * All Rights Reserved.
 *
 * Contributor(s):
 *   Gert Sylvest (gerts@avanade.com)
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Michael Nielsen (michaelni@avanade.com)
 *   Dennis Søgaard (dennis.j.sogaard@accenture.com)
 *   Ramzi Fadel (ramzif@avanade.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk)
 *
 */
package dk.gov.oiosi.xml.documenttype;

import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.exception.ArgumentException;
import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;

/**
 * Holds the paramaterers that can be extracted from a message relevant to performing a RASP call and UDDI lookup.
 */
public class MessageParameters {

    /**
     * The endpoint key.
     */
    private IIdentifier endpointKey;

    /**
     * The endpoint key.
     * 
     * @return endpoint key
     */
    public IIdentifier getEndpointKey() {
        return endpointKey;
    }

    /**
     * The keytype of the endpoint.
     */
    private EndpointKeytype endpointKeyType;

    /**
     * The keytype of the endpoint.
     * 
     * @return endpoint key type
     */
    public EndpointKeytype getEndpointKeyType() {
        return endpointKeyType;
    }

    /**
     * The UDDI id of the service contract tModel.
     */
    private UddiId serviceContractTModel;

    /**
     * The UDDI id of the service contract tModel.
     * 
     * @return service contract TModel
     */
    public UddiId getServiceContractTModel() {
        return serviceContractTModel;
    }

    /**
     * The UDDI id of the tModel representing the relevant process.
     */
    private UddiId processTModel;

    /**
     * The UDDI id of the tModel representing the relevant process.
     * 
     * @return uddi id of TModel
     */
    public UddiId getProcessTModel() {
        return processTModel;
    }

    /**
     * The type of the business process role identifier.
     */
    private BusinessProcessRoleIdentifierType roleIdentifierType;

    /**
     * The type of the business process role identifier
     * 
     * @return business process role
     */
    public BusinessProcessRoleIdentifierType getRoleIdentifierType() {
        return roleIdentifierType;
    }

    /**
     * The business process role identifier.
     */
    private BusinessProcessRoleIdentifier roleIdentifier;

    /**
     * The business process role identifier.
     * 
     * @return business process role identifier
     */
    public BusinessProcessRoleIdentifier getRoleIdentifier() {
        return roleIdentifier;
    }

    /**
     * Constructor.
     * 
     * @param endpointKey
     *            The endpoint key of the endpoint, e.g. an EAN number
     * @param endpointKeyType
     *            The endpoint keytype of the endpoint, e.g. EAN number
     * @param serviceContractTModel
     *            The tModel UDDI ID of the service contract tModel
     * @param processTModel
     *            The UDDI ID of the process. May be null.
     * @param roleIdentifierType
     *            The process role identifier type of the process. May be null, if roleIdentifier is null
     * @param roleIdentifier
     *            The process role identifier. May be null, if the role identifier is null
     */
    public MessageParameters(IIdentifier endpointKey, EndpointKeytype endpointKeyType, UddiId serviceContractTModel,
            UddiId processTModel, BusinessProcessRoleIdentifierType roleIdentifierType, BusinessProcessRoleIdentifier roleIdentifier) {
        init(endpointKey, endpointKeyType, serviceContractTModel, processTModel, roleIdentifierType, roleIdentifier);
    }

    /**
     * Initializes the object.
     * 
     * @param endpointKey
     *            The endpoint key of the endpoint, e.g. an EAN number
     * @param endpointKeyType
     *            The endpoint keytype of the endpoint, e.g. EAN number
     * @param serviceContractTModel
     *            The tModel UDDI ID of the service contract tModel
     * @param processTModel
     *            The UDDI ID of the process. May be null.
     * @param roleIdentifierType
     *            The process role identifier type of the process. May be null, if roleIdentifier is null
     * @param roleIdentifier
     *            The process role identifier. May be null, if the role identifier is null
     */
    private void init(IIdentifier endpointKey, EndpointKeytype endpointKeyType, UddiId serviceContractTModel, UddiId processTModel,
            BusinessProcessRoleIdentifierType roleIdentifierType, BusinessProcessRoleIdentifier roleIdentifier) {
        if (endpointKey == null)
            throw new NullArgumentException("endpointKey");
        if (endpointKeyType == null)
            throw new NullArgumentException("endpointKeyType");
        if (serviceContractTModel == null)
            throw new NullArgumentException("serviceContractTModel");
        if (processTModel == null && (roleIdentifier != null || roleIdentifierType != null))
            throw new ArgumentException("If processTModel is null, roleIdentifier and roleIdentifierType cannot be set");
        if ((roleIdentifier == null && roleIdentifierType != null) || (roleIdentifier != null && roleIdentifierType == null))
            throw new ArgumentException("RoleIdentifier and roleIdentifierType must both be either set or null");

        this.endpointKey = endpointKey;
        this.endpointKeyType = endpointKeyType;
        this.serviceContractTModel = serviceContractTModel;
        this.processTModel = processTModel;
        this.roleIdentifierType = roleIdentifierType;
        this.roleIdentifier = roleIdentifier;
    }

    /**
     * Constructor.
     * 
     * @param endpointKey
     *            The endpoint key of the endpoint, e.g. an EAN number
     * @param serviceContractTModel
     *            The tModel UDDI ID of the service contract tModel
     * @param endpointKeyType
     *            The type of the endpoint key (EAN/CVR...)
     */
    public MessageParameters(IIdentifier endpointKey, EndpointKeytype endpointKeyType, UddiId serviceContractTModel) {
        init(endpointKey, endpointKeyType, serviceContractTModel, null, null, null);
    }

}
