/**
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.xml.schematron;

import dk.gov.oiosi.exception.NullArgumentException;
import dk.gov.oiosi.xml.XmlDocument;
import dk.gov.oiosi.xml.XmlException;
import dk.gov.oiosi.xml.xpath.DocumentXPathResolver;
import dk.gov.oiosi.xml.xpath.PrefixedNamespace;
import dk.gov.oiosi.xml.xslt.XsltUtility;

/**
 * Schematron validates a given XML-document.
 */
public class SchematronValidator {

    /**
     * XPath to error.
     */
    private String errorXPath;

    /**
     * XPath to error message.
     */
    private String errorMessageXPath;

    /**
     * Schematron document.
     */
    private XmlDocument schematronDocument;

    /**
     * XSLT utility.
     */
    private XsltUtility xlstUtil;

    /**
     * Default constructor.
     */
    private SchematronValidator() {
        xlstUtil = new XsltUtility();
    }

    /**
     * Constructor that takes the error XPath and the error message XPath as parameter.
     * 
     * @param errorXPath
     *            XPath to error
     * @param errorMessageXPath
     *            XPath to error message
     */
    public SchematronValidator(String errorXPath, String errorMessageXPath) {
        this();
        this.errorXPath = errorXPath;
        this.errorMessageXPath = errorMessageXPath;
    }

    /**
     * Constructor that takes the configuration as a parameter.
     * 
     * @param config
     *            configuration
     * @throws SchematronErrorException
     */
    public SchematronValidator(SchematronValidationConfig config) throws SchematronErrorException {
        this();
        this.errorXPath = config.getErrorXPath();
        this.errorMessageXPath = config.getErrorMessageXPath();
        this.schematronDocument = config.getSchematronDocument();
    }

    /**
     * Schematron validates the given XML-document.
     * 
     * @param xmlDocument
     *            document
     * @throws SchematronErrorException
     * @throws SchematronValidationFailedException
     * @throws XmlException
     */
    public void schematronValidateXmlDocument(XmlDocument xmlDocument) throws SchematronErrorException,
            SchematronValidationFailedException, XmlException {
        if (schematronDocument == null)
            throw new NullArgumentException("No schematron document is set");

        schematronValidateXmlDocument(xmlDocument, schematronDocument);
    }

    /**
     * Schematron validates the given XML-document with the given schematron document.
     * 
     * @param xmlDocument
     *            document to validate
     * @param xmlSchematronStylesheet
     *            stylesheet to use
     * @throws SchematronValidationFailedException
     * @throws XmlException
     * @throws SchematronErrorException
     */
    public void schematronValidateXmlDocument(XmlDocument xmlDocument, XmlDocument xmlSchematronStylesheet)
            throws SchematronValidationFailedException, XmlException, SchematronErrorException {
        XmlDocument result = null;
        try {
            result = xlstUtil.transformXML(xmlDocument, xmlSchematronStylesheet);
        } catch (Exception ex) {
            throw new SchematronValidationFailedException(xmlDocument, ex);
        }
        PrefixedNamespace[] prefixedNamespaces = new PrefixedNamespace[0];
        boolean hasAnyErrors = DocumentXPathResolver.hasAnyElementsByXpath(result, errorXPath, prefixedNamespaces);
        if (hasAnyErrors) {
            String firstErrorMessage = DocumentXPathResolver.getFirstElementValueByXPath(result, errorMessageXPath,
                    prefixedNamespaces);
            throw new SchematronErrorException(result, firstErrorMessage);
        }
    }
}
