/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.security.ocsp;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentException;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.security.ocsp.IOcspLookup;
import dk.gov.oiosi.security.ocsp.OcspException;
import dk.gov.oiosi.security.ocsp.OcspLookupFactory;
import dk.gov.oiosi.security.ocsp.OcspResponse;
import dk.gov.oiosi.security.validation.CertificateValidator;
import dk.gov.oiosi.test.stubs.OcspLookupStub;

/**
 * jUnit test class for testing certificate agatinst ocsp server.
 */
public class OcspRequestTest extends TestCase {

    /**
     * local log.
     */
    private Log log = LogFactory.getLog(OcspRequestTest.class);

    static {
        RaspConfigurationHandler.setConfigurationFile("./conf/RaspConfigurationTestServers.xml");
    }

    /**
     * Read certificate from keystore.
     * 
     * @param keylabel
     *            keylabel
     * @return certificate
     * @throws Exception
     */
    protected X509Certificate getCertificate(String keylabel) throws Exception {
        // define receiver certificate
        KeyStore ks = KeyStore.getInstance("JKS");
        ks.load(new FileInputStream("./keys/oiosi_test_certificates.jks"), "Test1234".toCharArray());

        X509Certificate client = (X509Certificate) ks.getCertificate(keylabel);
        return client;
    }

    /**
     * Test validating certerficate.
     * 
     * @param keylabel
     *            keylabel
     * @param expectValid
     *            should certificate be valid
     * @return certficate
     */
    private X509Certificate performQuery(String keylabel, boolean expectValid) {
        log.info("Check Certificate : " + keylabel);
        X509Certificate cert = null;
        try {
            KeyStore ks = KeyStore.getInstance("JKS");
            ks.load(new FileInputStream("./keys/oiosi_test_certificates.jks"), "Test1234".toCharArray());
            cert = (X509Certificate) ks.getCertificate(keylabel);
            assertTrue(true);
        } catch (Exception e) {
            fail("Could not read certificate from keystore : " + keylabel);
        }

        IOcspLookup lookup = null;
        try {
            OcspLookupFactory lookupFactory = new OcspLookupFactory();
            lookup = lookupFactory.createOcspLookupClient();
            assertTrue(true);
        } catch (Exception e) {
            fail("Could not init OcspLookup");
        }

        try {
            log.info("Start check : " + cert.getSubjectDN());
            OcspResponse response = lookup.checkCertificate(cert);
            if (response.isValid() && expectValid) {
                assertTrue("Ok : Certificate validated : " + keylabel, true);
                log.info("Ok : Check Certificate : " + keylabel);
            } else {
                log.info("Failed : Check Certificate should have failed: " + keylabel);
                fail("Failed : Check Certificate should have failed : " + keylabel);
            }
        } catch (OcspException e) {
            if (expectValid) {
                log.info("Failed : Certificate expected to be validated : " + keylabel + " - Failure : " + e);
                fail("Failed : Certificate expected to be validated : " + keylabel + " - Failure : " + e);
            } else {
                log.info("Ok : Certificate failed to validated - as expected : " + keylabel);
                assertTrue("Ok : Certificate failed to validated - as expected : " + keylabel, true);
            }
        } catch (Throwable e) {
            fail("Ocsp Check Failed for : " + keylabel + " - Failure : " + e);
            e.printStackTrace();
        }
        return cert;
    }

    /**
     * Test ocsp for valid certificate.
     */
    public void testValid() {
        X509Certificate cert = performQuery("server_1", true);
        // true );
        try {
            assertTrue("We have certificate", cert != null);
            CertificateValidator.validateCertificate(cert);
            assertTrue("Certificate is valid", true);
        } catch (MainException e) {
            fail("Certicate should be valid " + e);
        } catch (Exception e) {
            fail("Unknow failure " + e);
        }
    }

    /**
     * Test ocsp for revoked certificate.
     */
    public void testRevoked() {
        X509Certificate cert = performQuery("server_tdc_revoked", false);
        try {
            assertTrue("We have certificate", cert != null);
            CertificateValidator.validateCertificate(cert);
            assertTrue("Certificate is valid", true);
        } catch (MainException e) {
            fail("Certicate should be valid");
        } catch (Exception e) {
            fail("Unknow failure " + e);
        }
    }

    /**
     * test ocsp for expired certificate.
     */
    public void testExpired() {
        X509Certificate cert = performQuery("server_tdc_expired", true);
        try {
            assertTrue("We have certificate", cert != null);
            CertificateValidator.validateCertificate(cert);
            assertTrue("Certificate shoud be expired", false);
        } catch (MainException e) {
            assertTrue("Certicate should be expired", true);
        } catch (Exception e) {
            fail("Unknow failure " + e);
        }
    }

    public void testDummy() throws OcspException{
    	X509Certificate cert = performQuery("server_tdc_expired", true);
    	IOcspLookup ocspClient = new OcspLookupStub();
    	boolean response = ocspClient.checkCertificate(cert).isValid();
    	assertTrue(response);
    }
    
    
    public void testDummyWithNullArgument() throws OcspException{
    	try{
    		IOcspLookup ocspClient = new OcspLookupStub();
    		ocspClient.checkCertificate(null);
    		assertTrue(false);
    	}
    	catch(ArgumentException ex){
    		
    	}
    	
    }
    
    
    /**
     * psvm for testing.
     * 
     * @param args
     *            args
     */
    public static void main(String[] args) {
        OcspRequestTest test = new OcspRequestTest();

        test.testValid();
        test.testRevoked();
        test.testExpired();

    }
}
