/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.uddi;

import java.util.List;

import junit.framework.TestCase;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.uddi.LookupParameters;
import dk.gov.oiosi.uddi.LookupReturnOption;
import dk.gov.oiosi.uddi.PreferredEndpointType;
import dk.gov.oiosi.uddi.UddiConfig;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupClient;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;

/**
 * perform uddi lookup. TODO: CHECK EANS
 */
public class UddiFallbackServerTest extends TestCase {

    static {
        RaspConfigurationHandler.setConfigurationFile("./conf/RaspConfigurationTestServers.xml");
    }

    /**
     * uddi client.
     */
    private static UddiLookupClient uddiClient = null;

    /**
     * uddi config.
     */
    private static UddiConfig uddiConfig = null;

    /**
     * Read UDDI Configuration.
     * 
     * @throws MainException
     */
    public void testInitUddiConfig() throws MainException {
        uddiConfig = RaspConfigurationHandler.getInstance().getUddiConfig();
    }


    /**
     * @throws MainException
     */
    public void testInitUddiClient() throws MainException {
        //
        String realUrl = uddiConfig.getUddiInquireEndpointURL();
        uddiConfig.setTryOtherHostsOnFailure(true);
        uddiConfig.setUddiInquireEndpointURLFallback( realUrl );
        String illegalUrl = "http://nonexisting.publish.uddi.ehandel.gov.dk/registry/uddi/inquiry";
        uddiConfig.setUddiInquireEndpointURL(illegalUrl);
        uddiClient = new UddiLookupClient(uddiConfig);
    }
    
    /**
     * Prepares the lookup parameters.
     * 
     * @param ean
     *            The ean of the lookup
     * @param serviceDefTModel
     *            The service definition tModel key
     * @return Returns the prepared lookup parameters
     * @throws MainException
     */
    private LookupParameters getUddiLookupParameters(String ean, String serviceDefTModel) throws MainException {
        LookupParameters parameters = new LookupParameters((IIdentifier) new IdentifierEan(ean), new EndpointKeytype(
                new EndpointKeyTypeCode(EndpointKeyTypeCode.ean)), (List) null, new PreferredEndpointType(
                PreferredEndpointType.http), new LookupReturnOption(LookupReturnOption.allResults), new UddiGuidId(
                serviceDefTModel, true), // serviceContractTModel
                (BusinessProcessRoleIdentifierType) null, // role identifier
                // type
                (BusinessProcessRoleIdentifier) null, // role identifier
                (UddiId) null // process definition tModel key
        );

        return parameters;
    }

    /**
     * Calls the UDDI with an EAN outside the range (5798000000000-5798009999999).
     * 
     * @param endpointKey
     *            ean of receiver
     * @param expectedResult
     *            number of expected results
     * @throws MainException
     */
    private void eanTestLookup(String endpointKey, int expectedResult) throws MainException {
        testInitUddiConfig();
        testInitUddiClient();
    	System.out.println("eanTestLookup " + endpointKey + " - expect : " + expectedResult);
        // Invoice 2.01 portType tModel ID
        LookupParameters lookupParameters = getUddiLookupParameters(endpointKey, "uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47");
        List response = uddiClient.lookup(lookupParameters);
        if (response == null || response.size() == 0) {
            if (expectedResult != 0) {
                System.out.println("Result expected - # : " + expectedResult);
                assertTrue("Result expected", false);
            } else {
                System.out.println("Empty result as expected");
                assertTrue("Empty result expected", true);
            }
        } else {
            if (response.size() == expectedResult) {
                System.out.println("Got correct number of responses : " + response.size());
                UddiLookupResponse r = (UddiLookupResponse) response.get(0);
                assertTrue("Keys must be identical", endpointKey.equals(r.getEndpointIdentifierActual().getAsString()));
            } else {
                System.out.println("Wrong number of results - got : " + response.size() + " - expected : " + expectedResult);
                assertTrue("Wrong number of results - got : " + response.size() + " - expected : " + expectedResult, false);
            }
        }
    }

    /**
     * search valid business.
     * 
     * @throws MainException
     */
    public void testValidLookupTestBusiness() throws MainException {
        System.out.println("testValidLookupTestBusiness()");
        String endpointKey = "5798009811585";
        eanTestLookup(endpointKey, 2);
    }

    /**
     * psvm for testing.
     * 
     * @param args
     *            args
     */
    public static void main(String[] args) {
        try {
            UddiFallbackServerTest test = new UddiFallbackServerTest();

            test.testInitUddiConfig();
            test.testInitUddiClient();
            test.testValidLookupTestBusiness();

        } catch (Exception e) {
            System.out.println("Failed : " + e);
            e.printStackTrace();
        }
    }
}
