/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.uddi;

import java.util.ArrayList;
import java.util.List;

import dk.gov.oiosi.addressing.EndpointAddressHttp;
import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.addressing.IdentifierCvr;
import dk.gov.oiosi.common.TimeSpan;
import dk.gov.oiosi.common.cache.TimedCache;
import dk.gov.oiosi.communication.Uri;
import dk.gov.oiosi.uddi.LookupKey;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.category.ConformanceClaim;
import dk.gov.oiosi.uddi.category.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.category.RegistrationConformanceClaim;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;
import junit.framework.TestCase;

/**
 * Test caching of UDDI lookups.
 */
public class UddiLookupCacheTest extends TestCase {

    /**
     * Verifies that the lookup key can be used as a cache key.
     */
    public void testLookupKey() {
        try {
            TimeSpan timeSpan = new TimeSpan();
            timeSpan.setMinuttes(2);

            TimedCache cache = new TimedCache(timeSpan);

            LookupKey lookupKey1 = getLookupKey("11111111");
            LookupKey lookupKey2 = getLookupKey("22222222");
            LookupKey lookupKey3 = getLookupKey("33333333");

            cache.add(lookupKey1, getResponseList("http://www.example.org"));
            cache.add(lookupKey2, getResponseList("http://www.domain.org"));

            boolean containsKey1 = cache.containsKey(lookupKey1);
            assertTrue("Cache must contain key ", containsKey1);

            boolean containsKey3 = cache.containsKey(lookupKey3);
            assertFalse("Cache should not have key", containsKey3);
        } catch (Exception e) {
            fail("This should not fail : " + e);
        }
    }

    /**
     * Verifies that the lookup key can be used as a cache key.
     */
    public void testLookupKeyDualAdd() {
        try {
            TimeSpan timeSpan = new TimeSpan().setMinuttes(2);

            TimedCache cache = new TimedCache(timeSpan);

            LookupKey lookupKey1 = getLookupKey("11111111");
            LookupKey lookupKey2 = getLookupKey("11111111");

            cache.add(lookupKey1, getResponseList("http://www.example.org"));
            cache.add(lookupKey2, getResponseList("http://www.domain.org"));

            List responseList = (List) cache.tryGetValue(lookupKey1);
            assertTrue("List must exist ", responseList != null && responseList.size() > 0);
            UddiLookupResponse uddiResponse = (UddiLookupResponse) responseList.get(0);
            assertTrue("http://www.example.org".equals(uddiResponse.getEndpointAddress().getAsUri().toString()));
            assertTrue("Only one result expected", responseList.size() == 1);
        } catch (Exception e) {
            fail("This should not fail : " + e);
        }
    }

    /**
     * Create uddi lookup key.
     * 
     * @param cvr
     *            cvr for lookup
     * @return uddi lookup key
     * @throws Exception
     */
    private LookupKey getLookupKey(String cvr) throws Exception {
        // <EndpointAddressTypeCode>
        List addressTypeFilter = new ArrayList();
        UddiId businessProcessDefinitionTModel = new UddiGuidId();
        IIdentifier endpointKey = new IdentifierCvr(cvr);
        EndpointKeytype endpointKeyType = new EndpointKeytype(EndpointKeyTypeCode.getCvr());
        ConformanceClaim profileConformanceClaim = new ConformanceClaim();
        RegistrationConformanceClaim registrationConformanceClaim = new RegistrationConformanceClaim();
        BusinessProcessRoleIdentifier roleIdentifier = new BusinessProcessRoleIdentifier();
        UddiId serviceContractTModel = new UddiGuidId();

        LookupKey lookupKey = new LookupKey(addressTypeFilter, businessProcessDefinitionTModel, endpointKey, endpointKeyType,
                profileConformanceClaim, registrationConformanceClaim, roleIdentifier, serviceContractTModel);
        return lookupKey;
    }

    /**
     * return list of uddi responsed.
     * 
     * @param uri
     *            endpoint url
     * @return list
     */
    private List getResponseList(String uri) {
        UddiLookupResponse response = new UddiLookupResponse();
        response.setEndpointAddress(new EndpointAddressHttp(new Uri(uri)));

        // <UddiLookupResponse>
        List responseList = new ArrayList();
        responseList.add(response);
        return responseList;
    }

}
