/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain
 * a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * The Original Code is Java RASP toolkit.
 *
 * The Initial Developer of the Original Code is Lenio. Portions
 * created by Lenio are Copyright (C) 2007 Danish National IT and
 * Telecom Agency (http://www.itst.dk). All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Dejbjerg Pedersen (tdp@lenio.dk)
 *   Hans Guldager Knudsen, Lenio
 *   Patrik Johansson (p.johansson@accenture.com)
 *   Dennis Sgaard (dennis.j.sogaard@accenture.com)
 *   Mikkel Hippe Brun (mhb@itst.dk)
 *   Finn Hartmann Jordal (fhj@itst.dk)
 *   Christian Lanng (chl@itst.dk).
 *
 */

package dk.gov.oiosi.test.unit.uddi;

import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import dk.gov.oiosi.addressing.IIdentifier;
import dk.gov.oiosi.addressing.IdentifierEan;
import dk.gov.oiosi.configuration.RaspConfigurationHandler;
import dk.gov.oiosi.exception.ArgumentException;
import dk.gov.oiosi.exception.MainException;
import dk.gov.oiosi.test.stubs.UddiLookupClientStub;
import dk.gov.oiosi.uddi.IUddiLookupClient;
import dk.gov.oiosi.uddi.LookupParameters;
import dk.gov.oiosi.uddi.LookupReturnOption;
import dk.gov.oiosi.uddi.PreferredEndpointType;
import dk.gov.oiosi.uddi.UddiConfig;
import dk.gov.oiosi.uddi.UddiGuidId;
import dk.gov.oiosi.uddi.UddiId;
import dk.gov.oiosi.uddi.UddiLookupClient;
import dk.gov.oiosi.uddi.UddiLookupResponse;
import dk.gov.oiosi.uddi.category.BusinessProcessRoleIdentifierType;
import dk.gov.oiosi.uddi.category.EndpointKeyTypeCode;
import dk.gov.oiosi.uddi.category.EndpointKeytype;
import dk.gov.oiosi.uddi.identifier.BusinessProcessRoleIdentifier;

/**
 * perform UDDI lookup. TODO: CHECK TESTCASE testWithinRangeWithoutMatchAndMissGateway
 */
public class UddiLookupTest extends TestCase {

    /**
     * local log.
     */
    private static Log log = LogFactory.getLog(UddiLookupTest.class);

    static {
        RaspConfigurationHandler.setConfigurationFile("./conf/RaspConfigurationTestServers.xml");
    }

    /**
     * is UDDI client initialized.
     */
    private static boolean isInitialized = false;

    /**
     * UDDI client.
     */
    private static UddiLookupClient uddiClient = null;

    /**
     * UDDI config.
     */
    private static UddiConfig uddiConfig = null;

    /**
     * If not initialized, initializes the UDDI lookup client.
     * 
     * @throws MainException
     */
    private void initUddi() throws MainException {
        if (isInitialized)
            return;

        // 1. Set default UDDI configuration:

        uddiConfig = RaspConfigurationHandler.getInstance().getUddiConfig();

        // 2. Instantiate lookup client
        uddiClient = new UddiLookupClient(uddiConfig);

        isInitialized = true;
    }

    /**
     * Prepares the lookup parameters.
     * 
     * @param ean
     *            The EAN of the lookup
     * @param serviceDefTModel
     *            The service definition tModel key
     * @return Returns the prepared lookup parameters
     * @throws MainException
     */
    private LookupParameters getUddiLookupParameters(String ean, String serviceDefTModel) throws MainException {
        LookupParameters parameters = new LookupParameters((IIdentifier) new IdentifierEan(ean), new EndpointKeytype(
                new EndpointKeyTypeCode(EndpointKeyTypeCode.ean)), (List) null, new PreferredEndpointType(
                PreferredEndpointType.http), new LookupReturnOption(LookupReturnOption.allResults), new UddiGuidId(
                serviceDefTModel, true), // serviceContractTModel
                (BusinessProcessRoleIdentifierType) null, // role identifier
                // type
                (BusinessProcessRoleIdentifier) null, // role identifier
                (UddiId) null // process definition tModel key
        );

        return parameters;
    }

    /**
     * Calls the UDDI with an EAN outside the range (5798000000000-5798009999999).
     * 
     * @param endpointKey
     *            ean of receiver
     * @param expectedResult
     *            number of expected results
     * @throws MainException
     */
    private void eanTestLookup(String endpointKey, int expectedResult) throws MainException {
        initUddi();
        log.info("eanTestLookup " + endpointKey + " - expect : " + expectedResult);

        // Invoice 2.01 portType tModel ID
        LookupParameters lookupParameters = getUddiLookupParameters(endpointKey, "uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47");

        List response = uddiClient.lookup(lookupParameters);
        if (response == null || response.size() == 0) {
            if (expectedResult != 0) {
                System.out.println("Result expected - # : " + expectedResult);
                assertTrue("Result expected", false);
            } else {
                log.info("Empty result as expected");
                assertTrue("Empty result expected", true);
            }
        } else {
            if (response.size() == expectedResult) {
                log.info("Got correct number of responses : " + response.size());

                UddiLookupResponse r = (UddiLookupResponse) response.get(0);

                assertTrue("Keys must be identical", endpointKey.equals(r.getEndpointIdentifierActual().getAsString()));
            } else {
                System.out.println("Wrong number of results - got : " + response.size() + " - expected : " + expectedResult);
                assertTrue("Wrong number of results - got : " + response.size() + " - expected : " + expectedResult, false);

            }
        }
    }

    /*
     * Tests the dummy UDDI class, to make sure it reads configuration correctly, 
     * and returns a valid UDDIResponse     * 
     */
    public void testDummyLookup() throws MainException{
    	IUddiLookupClient client = new UddiLookupClientStub();
    	LookupParameters lookupParameters = getUddiLookupParameters("5700000000001", "uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47");
    	List reply = client.lookup(lookupParameters);
    	
    	assertNotNull(reply);
    	assertTrue(!reply.isEmpty());
    	UddiLookupResponse response = (UddiLookupResponse)reply.get(0);
    	
    	assertNotNull(response.getEndpointAddress());
    	assertNotNull(response.getActivationDate());
    	assertNotNull(response.getCertificateSubjectSerialNumber());
    	assertNotNull(response.getExpirationDate());
    	assertNotNull(response.getServiceContactEmail());
    	assertNotNull(response.getTermsOfUseUrl());
    	assertNotNull(response.getVersion());
    }
    
    public void testDummyLookupWithNullParameter() throws MainException{
    	IUddiLookupClient client = new UddiLookupClientStub();
    	try{
    		List reply = client.lookup(null);
    		assertTrue("Result expected", false);
    	}
    	catch(ArgumentException ex){}
    	
    }
    
    
    /**
     * search valid business.
     * 
     * @throws MainException
     */
    public void testValidLookupTestBusiness() throws MainException {
        log.info("testValidLookupTestBusiness()");
        String endpointKey = "5798009811585";
        eanTestLookup(endpointKey, 2);
    }

    /**
     * perform valid lookup.
     * 
     * @throws MainException
     */
    public void testValidLookup() throws MainException {
        log.info("testValidLookup()");
        String endpointKey = "5798009811578";
        eanTestLookup(endpointKey, 1);
    }

    /**
     * search unknown outside gateway range.
     * 
     * @throws MainException
     */
    public void testOutsideRange() throws MainException {
        log.info("testOutsideRange()");
        String endpointKey = "5798019999999";
        eanTestLookup(endpointKey, 0);
    }

    /**
     * search unknown but inside gateway range
     * 
     * @throws MainException
     */
    public void testGatewayServer() throws MainException {
        log.info("testGatewayServer()");
        String endpointKey = "5798009811646";
        eanTestLookup(endpointKey, 1);
    }

    /**
     * Calls the UDDI with an EAN within the range, but the specific EAN also exists in a separate registration outside the range
     * (5798000000000-5798009999999).
     * 
     * @throws MainException
     */
    public void testWithinRangeWithMatch() throws MainException {
        log.info("testWithinRangeWithMatch()");
        initUddi();
        String endpointKey = "5798009811578";

        // The regular invoice 2.01 test endpoint
        // Invoice 2.01 portType tModel ID
        LookupParameters lookupParameters = getUddiLookupParameters(endpointKey, "uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47");

        List response = uddiClient.lookup(lookupParameters);
        if (response == null || response.size() == 0) {
            assertTrue("Result expected", false);
        } else {
            assertTrue("One result expected", response.size() == 1);
            UddiLookupResponse r = (UddiLookupResponse) response.get(0);

            assertTrue("Keys must be identical", endpointKey.equals(r.getEndpointIdentifierActual().getAsString()));
        }
    }

    /**
     * Calls the UDDI with an EAN within the range. The EAN does not exist in any registration outside the range
     * (5798000000000-5798009999999), so the endpoint of the gateway should be returned.
     * 
     * @throws MainException
     */
    public void testWithinRangeWithoutMatch() throws MainException {
        log.info("testWithinRangeWithoutMatch()");
        initUddi();
        String endpointKey = "5798009811554";

        // Invoice 2.01 portType tModel ID
        LookupParameters lookupParameters = getUddiLookupParameters(endpointKey, "uddi:2e0b402a-7a5e-476b-8686-b33f54fd1f47");

        List response = uddiClient.lookup(lookupParameters);

        // System.out.println("response size : " + response.size());
        if (response == null || response.size() == 0) {
            // System.out.println("response - size == 0 ");
            assertTrue("Result expected", false);
        } else {
            assertTrue("One result expected", response.size() == 1);
            UddiLookupResponse r = (UddiLookupResponse) response.get(0);

            // CHECK FOR GATEWAY ADDRESS

            String foundEndpoint = r.getEndpointIdentifierActual().getAsString();
            String gatewayEndpoint = uddiConfig.getGatewayRange().getGatewayRegistrationParameters().getGatewayRegistrationKeyEan();

            assertTrue("Keys must be identical", foundEndpoint.equals(gatewayEndpoint));

        }
    }

    /**
     * Calls the UDDI with an EAN within the range. The EAN does not exist in any registration outside the range
     * (5798000000000-5798009999999), so the endpoint of the gateway should be returned. However the document type is not registered
     * for this gateway, so nothing is returned.
     * 
     * @throws MainException
     */
    public void testWithinRangeWithoutMatchAndMissGateway() throws MainException {
        log.info("testWithinRangeWithoutMatchAndMissGateway()");
        log.warn("testWithinRangeWithoutMatchAndMissGateway() - will fail - as document exist");

        initUddi();
        String endpointKey = "5798009811554";

        // Reminder 2.01 portType tModel ID
        LookupParameters lookupParameters = getUddiLookupParameters(endpointKey, "uddi:4fbbbdef-0a8e-4d5e-9b9d-23c8fd98e9ce");

        List response = uddiClient.lookup(lookupParameters);
        log.info("response size : " + response.size());
        if (response == null || response.size() == 0) {
            log.info("response - size == 0 - as expected");
            assertTrue("No Result expected", true);
        } else {
            UddiLookupResponse r = (UddiLookupResponse) response.get(0);

            log.info(" - " + r.getEndpointAddress().getAsUri());
            log.info(" - " + r.getEndpointIdentifierActual().getAsString());
            log.info(" - " + r.getCertificateSubjectSerialNumber().getSubjectString());

            fail("We expected that credit note - was not registered at gateway - but this is not the case. Find out how to test.");
            assertTrue("No Result expected", false);
        }
    }
}
